# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
About dialog for Scribes

This module contains a function responsible for showing the about dialog for
the editor.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from gc import collect

from pygtk import require
require("2.0")
from gtk import AboutDialog, about_dialog_set_url_hook as uhook, keysyms
from gtk import RESPONSE_CLOSE
from gnome import url_show

from license import license_string
from internationalization import msg0001, msg0187, msg0204
from info import version, translators, artists, author, website, documenters
from info import copyrights
from feedback import update_status_info


def key_press_event_cb(dialog, event):
	u"""
	Handles callback when keys on the keyboard are pressed.

	This function watches for the Escape key event. When the Escape key event is
	detected, this function closes text editor's about dialog.

	@param dialog: The text editor's about dialog.
	@type dialog: A gtk.AboutDialog object.

	@param event: An event generated by pressing the keyboard.
	@type event: A gtk.Event object

	@return: If false, propagate the generated event. Otherwise, block the
			event.
	@rtype: A boolean value.

	"""

	# Close the text editor's about dialog when the escape key is pressed.
	if event.keyval == keysyms.Escape:

		dialog.response(RESPONSE_CLOSE)

		return True

	return False


def delete_event_cb(dialog, event):
	u"""
	Handles callback when the dialog window is closed.

	Close the text editor about dialog.

	@param dialog: The text editor's about dialog.
	@type dialog: A gtk.AboutDialog object.

	@param event: An event generated by pressing the keyboard.
	@type event: A gtk.Event object

	@return: If false, propagate the generated event. Otherwise, block the
			event.
	@rtype: A boolean value.

	"""

	dialog.response(RESPONSE_CLOSE)

	return False


def create_about_dialog(editor):
	u"""
	Create and show the text editor's about dialog.

	@return: Scribes' about dialog
	@rtype: A gtk.Dialog object

	"""

	# Feedback to the statusbar area indicating the purpose of the about dialog.
	update_status_info(editor, msg0204, "about", False)

	# Create the text editor's about dialog.
	dialog = AboutDialog()
	dialog.connect(u"key-press-event", key_press_event_cb)
	dialog.connect(u"delete-event", delete_event_cb)

	# Create a URL link on the text editor's about dialog to Scribes' homepage.
	uhook(lambda dialog, url:url_show(website))

	# Set dialog widget properties.
	dialog.set_property(u"width-request", 400)

	# Set dialog window properties.
	dialog.set_role(u"Scribes About Dialog")
	dialog.set_transient_for(editor)
	dialog.set_property(u"icon-name", u"stock_about")
	dialog.set_property(u"skip-pager-hint", True)
	dialog.set_property(u"skip-taskbar-hint", True)
	dialog.set_property(u"urgency-hint", False)

	# Set dialog properties.
	dialog.set_artists(artists)
	dialog.set_authors(author)
	dialog.set_documenters(documenters)
	dialog.set_property(u"comments", msg0001)
	dialog.set_property(u"copyright", copyrights)
	dialog.set_property(u"license", license_string.strip())
	dialog.set_property(u"logo-icon-name", u"scribes")
	dialog.set_property(u"name", u"Scribes")
	dialog.set_property(u"translator-credits", translators)
	dialog.set_property(u"version", version)
	dialog.set_property(u"website", website)
	dialog.set_property(u"website-label", website)
	# Show the text editor's about dialog.
	response = dialog.run()
	if response:
		# Destroy the dialog object, send feedback to the statusbar area
		# indicating the dialog was closed, and call the garbage collector.
		dialog.hide()
		dialog.destroy()
		update_status_info(editor, msg0187, "info")
		collect()
	return dialog

