# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Accelerator module for Scribes.

Defines all the shortcut key combinations that can be used with the editor.
It also initializes and creates accelerator objects for the editor.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from pygtk import require
require("2.0")
from gtk.gdk import CONTROL_MASK, MOD1_MASK
from gtk.gdk import SHIFT_MASK
from gtk import AccelGroup

from feedback import update_status_info
from internationalization import msg0002, msg0003

################################################################################
#
#						Accelerator Callbacks
#
################################################################################


def new_file_accel_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "n" key.
	@type key: An integer representing the "n" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	# Spawn a new instance of the text editor.
	editor.actiongroup.get_action("new_file").activate()

	return True


def open_file_accel_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "o" key.
	@type key: An integer representing the "o" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	# Show the filechooser dialog.
	editor.actiongroup.get_action("open_file").activate()

	return True


def save_file_accel_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "s" key.
	@type key: An integer representing the "s" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	# Save the file to the host's file system.
	editor.actiongroup.get_action("save_file").activate()

	return True


def saveas_file_accel_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "s" key.
	@type key: An integer representing the "s" key.

	@param modifier: A representation of the <control> and <shift> key modifier.
	@type modifier: An integer representing the <control> and <shift> key.

	"""

	# Show the save dialog.
	editor.actiongroup.get_action("saveas_file").activate()

	return True


def print_file_accel_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "p" key.
	@type key: An integer representing the "p" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	# Show the print dialog.
	editor.actiongroup.get_action("print_file").activate()

	return True


def undo_edits_accel_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "z" key.
	@type key: An integer representing the "z" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	# If an action cannot be undone, send appropriate feedback to the user via
	# the statusbar and sound the system bell.
	if not editor.actiongroup.get_action("undo_edits").is_sensitive():


		update_status_info(editor, message=msg0002, mode="no")

		return True

	editor.actiongroup.get_action("undo_edits").activate()

	return True


def redo_edits_accel_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "z" key.
	@type key: An integer representing the "z" key.

	@param modifier: A representation of the <control> and <shift> key modifier.
	@type modifier: An integer representing the <control> and <shift> key.

	"""

	if not editor.actiongroup.get_action("redo_edits").is_sensitive():

		# Feedback to statusbar indicating a redo operation cannot be performed.
		update_status_info(editor, message=msg0003, mode=u"no")

		return True

	editor.actiongroup.get_action("redo_edits").activate()

	return True


def search_text_accel_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "f" key.
	@type key: An integer representing the "f" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	# Show the find bar.
	editor.actiongroup.get_action("search_text").activate()

	return True


def replace_text_accel_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "r" key.
	@type key: An integer representing the "r" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	# Show the replace bar.
	editor.actiongroup.get_action("replace_text").activate()

	return True


def goto_line_accel_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "i" key.
	@type key: An integer representing the "i" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	print key
	# Show the goto bar.
	editor.actiongroup.get_action("goto_line").activate()

	return True


def close_file_accel_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "w" key.
	@type key: An integer representing the "w" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	# Close the editor.
	editor.actiongroup.get_action("close_file").activate()

	return True


def match_bracket_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "b" key.
	@type key: An integer representing the "b" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	# Find matching brackets.
	editor.actiongroup.get_action("match_bracket").activate()

	return True


def delete_line_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "d" key.
	@type key: An integer representing the "d" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Delete current line or selected lines.
	editor.actiongroup.get_action("delete_line").activate()

	return True


def join_line_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "j" key.
	@type key: An integer representing the "j" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Append the text on the next line to the current one.
	editor.actiongroup.get_action("join_line").activate()

	return True


def select_word_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "w" key.
	@type key: An integer representing the "w" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Select a word in the text editor's buffer.
	editor.actiongroup.get_action("select_word").activate()

	return True


def select_paragraph_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "p" key.
	@type key: An integer representing the "p" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Select a paragraph in the text editor's buffer.
	editor.actiongroup.get_action("select_paragraph").activate()

	return True


def uppercase_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "up arrow" key.
	@type key: An integer representing the "up arrow" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Change selected text in the text editor's buffer from lowercase to
	# uppercase.
	editor.actiongroup.get_action("uppercase").activate()

	return True


def lowercase_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "down arrow" key.
	@type key: An integer representing the "down arrow" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Change selected text in the text editor's buffer from uppercase to
	# lowercase.
	editor.actiongroup.get_action("lowercase").activate()

	return True


def swapcase_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "down arrow" key.
	@type key: An integer representing the "down arrow" key.

	@param modifier: A representation of the <alt> and <shift> key modifier.
	@type modifier: An integer representing the <alt> and <shift> key.

	"""

	# Swap the case of selected text in the text editor's buffer.
	editor.actiongroup.get_action("swapcase").activate()

	return True


def titlecase_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "up arrow" key.
	@type key: An integer representing the "up arrow" key.

	@param modifier: A representation of the <alt> and <shift> key modifier.
	@type modifier: An integer representing the <alt> and <shift> key.

	"""

	# Capitalize selected text in the text editor's buffer.
	editor.actiongroup.get_action("titlecase").activate()

	return True


def indent_cb(accelgroup, editor, key, modifier):
	u"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "t" key.
	@type key: An integer representing the "t" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	# Indent current line or selected lines.
	editor.actiongroup.get_action("indent").activate()

	return True


def unindent_cb(accelgroup, editor, key, modifier):
	u"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "t" key.
	@type key: An integer representing the "" key.

	@param modifier: A representation of the <control> and <shift> key modifier.
	@type modifier: An integer representing the <control> and <shift> key.

	"""

	# "Unindent" current line or selected lines.
	editor.actiongroup.get_action("unindent").activate()

	return True


def select_line_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "l" key.
	@type key: An integer representing the "l" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Select all text on the current line in the text editor's buffer.
	editor.actiongroup.get_action("select_line").activate()

	return True


def select_sentence_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "s" key.
	@type key: An integer representing the "s" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Select all text in a sentence on the current line in the text editor's
	# buffer.
	editor.actiongroup.get_action("select_sentence").activate()

	return True


def remove_spaces_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "r" key.
	@type key: An integer representing the "r" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Remove all spaces at the end of the current line or selected lines in the
	# text editor's buffer.
	editor.actiongroup.get_action("remove_spaces").activate()

	return True


def remove_beginning_spaces_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "d" key.
	@type key: An integer representing the "d" key.

	@param modifier: A representation of the <alt> and <shift> key modifier.
	@type modifier: An integer representing the <alt> and <shift> key.

	"""

	# Remove all spaces at the beginning of the current line or selected lines
	# in the text editor's buffer.
	editor.actiongroup.get_action("remove_begin").activate()

	return True


def open_line_above_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "o" key.
	@type key: An integer representing the "o" key.

	@param modifier: A representation of the <alt> and <shift> key modifier.
	@type modifier: An integer representing the <alt> and <shift> key.

	"""

	# Shift all text on the current line to next one in the text editor's
	# buffer.
	editor.actiongroup.get_action("open_line_above").activate()

	return True


def open_line_below_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "o" key.
	@type key: An integer representing the "o" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Shift all text on the next line to line after it in the text editor's
	# buffer.
	editor.actiongroup.get_action("open_line_below").activate()

	return True


def delete_cursor_to_line_end_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "end" key.
	@type key: An integer representing the "end" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Delete all text from the cursor to the end of the current line in the
	# text editor's buffer.
	editor.actiongroup.get_action("delete_cursor_to_end").activate()

	return True


def delete_cursor_to_line_begin_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "d" key.
	@type key: An integer representing the "d" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Delete all text from the cursor to the beginning of the current line in
	# the text editor's buffer.
	editor.actiongroup.get_action("delete_cursor_to_begin").activate()

	return True


def bookmark_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "d" key.
	@type key: An integer representing the "d" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	# Bookmark the current line in the text editor's buffer.
	editor.actiongroup.get_action("bookmark").activate()

	return True


def remove_bookmark_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "delete" key.
	@type key: An integer representing the "delete" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Remove bookmark on the current line of the text editor.
	editor.actiongroup.get_action("remove_bookmark").activate()

	return True


def remove_all_bookmarks_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "delete" key.
	@type key: An integer representing the "delete" key.

	@param modifier: A representation of the <alt><shift> key modifier.
	@type modifier: An integer representing the <alt><shift> key.

	"""

	# Remove all bookmarks in the text editor's buffer.
	editor.actiongroup.get_action("remove_all_bookmarks").activate()

	return True


def next_bookmark_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "right" key.
	@type key: An integer representing the "right" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Move the next bookmark in the text editor's buffer.
	editor.actiongroup.get_action("next_bookmark").activate()

	return True


def previous_bookmark_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "left" key.
	@type key: An integer representing the "left" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	# Move to the previous bookmark in the text editor's buffer.
	editor.actiongroup.get_action("previous_bookmark").activate()

	return True


def convert_tabs_to_spaces_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "t" key.
	@type key: An integer representing the "t" key.

	@param modifier: A representation of the <alt><shift> key modifier.
	@type modifier: An integer representing the <alt><shift> key.

	"""

	# Convert tabs to spaces
	editor.actiongroup.get_action("tabs_to_spaces").activate()

	return True


def select_bracket_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "b" key.
	@type key: An integer representing the "b" key.

	@param modifier: A representation of the <alt> key modifier.
	@type modifier: An integer representing the <alt> key.

	"""

	editor.actiongroup.get_action("select_bracket").activate()

	return True


def show_bookmark_browser_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "b" key.
	@type key: An integer representing the "b" key.

	@param modifier: A representation of the <control> key modifier.
	@type modifier: An integer representing the <control> key.

	"""

	editor.actiongroup.get_action("show_bookmark_browser").activate()

	return True


def jump_to_last_bookmarked_line_cb(accelgroup, editor, key, modifier):
	"""
	Handles callback when a shortcut key is pressed.

	@param accelgroup: A group of shortcut keys for the text editor.
	@type accelgroup: A gtk.AccelGroup.

	@param editor: The text editor object.
	@type editor: An editor object.

	@param key: A representation of the "left arrow" key.
	@type key: An integer representing the "left arrow" key.

	@param modifier: A representation of the <alt><shift> key modifier.
	@type modifier: An integer representing the <alt><shift> key.

	"""

	editor.actiongroup.get_action("jump_to_last_bookmarked_line").activate()

	return True


################################################################################
#
#							Accelerators
#
################################################################################

def create_accel():
	"""
	Create keyboard shortcuts for the entire application

	@return: An object containing the text editor's shortcut keys.
	@rtype: A gtk.AccelGroup

	"""

	# Create an accelgroup object
	accelgroup = AccelGroup()
	from gtk import accelerator_parse
	from gtk import ACCEL_LOCKED
	locked = ACCEL_LOCKED

	# (Control - f) to launch to findbar
	accelgroup.connect_group(102,
							CONTROL_MASK,
							locked,
							search_text_accel_cb)

	# (Control - i) to launch the gotobar
	accelgroup.connect_group(accelerator_parse("i")[0],
							CONTROL_MASK,
							locked,
							goto_line_accel_cb)

	# (Control - n) to launch a new editor with an empty file
	accelgroup.connect_group(110,
					CONTROL_MASK,
					locked,
					new_file_accel_cb)

	# (Control - o) to open a file for editing or browsing
	accelgroup.connect_group(111,
						CONTROL_MASK,
						locked,
						open_file_accel_cb)

	# (Control - s) to save a file
	accelgroup.connect_group(115,
						CONTROL_MASK,
						locked,
						save_file_accel_cb)

	# (Control - Shift - s) to save a file as different name
	accelgroup.connect_group(115,
						CONTROL_MASK | SHIFT_MASK,
						locked,
						saveas_file_accel_cb)

	# (Control - w) to close a file
	accelgroup.connect_group(119,
							CONTROL_MASK,
							locked,
							close_file_accel_cb)

	# (Control - z ) to undo an action
	accelgroup.connect_group(122,
						CONTROL_MASK,
						locked,
						undo_edits_accel_cb)

	# (Control - Shift - z ) to redo an action
	accelgroup.connect_group(122,
						CONTROL_MASK | SHIFT_MASK,
						locked,
						redo_edits_accel_cb)

	# (Control - r ) to show the replace bar
	accelgroup.connect_group(114,
						CONTROL_MASK,
						locked,
						replace_text_accel_cb)

	# (Control - p ) to show the print dialog
	accelgroup.connect_group(112,
						CONTROL_MASK,
						locked,
						print_file_accel_cb)

	# (Control - b) to find matching brackets
	accelgroup.connect_group(98,
							CONTROL_MASK,
							locked,
							show_bookmark_browser_cb)
							#match_bracket_cb)

	# (Alt - d) to delete a line
	accelgroup.connect_group(100,
							MOD1_MASK,
							locked,
							delete_line_cb)

	# (Alt - j) to join a line
	accelgroup.connect_group(106,
							MOD1_MASK,
							locked,
							join_line_cb)

	# (Alt - w) to select a word
	accelgroup.connect_group(119,
							MOD1_MASK,
							locked,
							select_word_cb)

	# (Alt - p) to select a paragraph
	accelgroup.connect_group(112,
							MOD1_MASK,
							locked,
							select_paragraph_cb)

	# (Alt - Up) to uppercase a word
	accelgroup.connect_group(65362,
							MOD1_MASK,
							locked,
							uppercase_cb)

	# (Alt - Shift - Up) to capitalize of a selected string
	accelgroup.connect_group(65362,
							MOD1_MASK | SHIFT_MASK,
							locked,
							titlecase_cb)

	# (Alt - Down) to lowercase a word
	accelgroup.connect_group(65364,
							MOD1_MASK,
							locked,
							lowercase_cb)

	# (Alt - Shift- Down) to swapcase of a selected string
	accelgroup.connect_group(65364,
							MOD1_MASK | SHIFT_MASK,
							locked,
							swapcase_cb)

	# (Ctrl - t) to indent
	accelgroup.connect_group(116,
							CONTROL_MASK,
							locked,
							indent_cb)

	# (Ctrl - Shift -t) to unindent
	accelgroup.connect_group(116,
							CONTROL_MASK | SHIFT_MASK,
							locked,
							unindent_cb)

	# (Alt - l) to select line
	accelgroup.connect_group(108,
							MOD1_MASK,
							locked,
							select_line_cb)

	# (Alt - s) to select line
	accelgroup.connect_group(115,
							MOD1_MASK,
							locked,
							select_sentence_cb)

	# (Alt - r ) to show remove trailing spaces
	accelgroup.connect_group(114,
						MOD1_MASK,
						locked,
						remove_spaces_cb)

	# (Alt - Shift - r ) to show remove spaces at the beginning of line
	accelgroup.connect_group(114,
						MOD1_MASK | SHIFT_MASK,
						locked,
						remove_beginning_spaces_cb)

	# (Alt - o ) to open a line below the current line
	accelgroup.connect_group(111,
						MOD1_MASK,
						locked,
						open_line_below_cb)

	# (Alt - Shift - o ) to open a line above the current line
	accelgroup.connect_group(111,
						MOD1_MASK | SHIFT_MASK,
						locked,
						open_line_above_cb)

	# (Alt - Home ) to delete text from cursor to begining of line
	accelgroup.connect_group(65360,
						MOD1_MASK,
						locked,
						delete_cursor_to_line_begin_cb)

	# (Alt - End ) to delete text from cursor to end of line
	accelgroup.connect_group(65367,
						MOD1_MASK,
						locked,
						delete_cursor_to_line_end_cb)

	# (Control - D) to bookmark a line in the text editor's buffer.
	accelgroup.connect_group(100,
							CONTROL_MASK,
							locked,
							bookmark_cb)

	# (Alt - Delete) to remove a bookmark on a line in the text editor's buffer.
	accelgroup.connect_group(65535,
							MOD1_MASK,
							locked,
							remove_bookmark_cb)

	# (Alt - Shift - Delete) to remove all bookmarks in the text editor's
	# buffer.
	accelgroup.connect_group(65535,
							MOD1_MASK | SHIFT_MASK,
							locked,
							remove_all_bookmarks_cb)

	# (Alt - left arrow) to move to the next bookmark in the text editor's
	# buffer
	accelgroup.connect_group(65363,
							MOD1_MASK,
							locked,
							next_bookmark_cb)

	# (Alt - right arrow) to move to the previous bookmark in the text editor's
	# buffer
	accelgroup.connect_group(65361,
							MOD1_MASK,
							locked,
							previous_bookmark_cb)

	# (Alt - shift - left arrow) to jump to the last bookmarked line in the
	# text editor's buffer.
	accelgroup.connect_group(65361,
							MOD1_MASK | SHIFT_MASK,
							locked,
							jump_to_last_bookmarked_line_cb)

	# (Alt - shift - t) to convert tabs to spaces in the text editor's buffer.
	accelgroup.connect_group(116,
							MOD1_MASK | SHIFT_MASK,
							locked,
							convert_tabs_to_spaces_cb)

	# (Alt - b) to select words inside brackets, parenthesis and quotes.
	accelgroup.connect_group(98,
							MOD1_MASK,
							locked,
							select_bracket_cb)

	# Prevent accelerators from being changed during runtime.
	accelgroup.lock()

	return accelgroup

