# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


"""
Actions are GTK+ objects that perform a single predefined function. Once an
action has been created, it can be called anytime and from anywhere to perform
operations it is designed to accomplish.

User operations are performed via actions with the text editor. Actions
represent functions such as clicking toolbar buttons or activating operations
through shortcut keys. Therefore, any user operations must be implemented via
actions.

This module houses all actions supported by the text editor.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""


from pygtk import require
require("2.0")
from gtk import main as gmain, quit_add, ActionGroup, STOCK_NEW
from gtk import STOCK_OPEN, STOCK_SAVE_AS, STOCK_PRINT, STOCK_UNDO, STOCK_REDO
from gtk import STOCK_JUMP_TO, STOCK_FIND, STOCK_FIND_AND_REPLACE
from gtk import STOCK_PREFERENCES, STOCK_HELP

from savefile import SaveFile
from tooltips import new_button_tip, open_button_tip, save_button_tip
from tooltips import print_button_tip, undo_button_tip, redo_button_tip
from tooltips import find_button_tip, replace_button_tip, goto_button_tip
from tooltips import pref_button_tip, help_button_tip
from savechooser import show_save_dialog
from printing import show_print_dialog
from undo import undo_action, redo_action
from preferences import Preferences
from help import launch_user_guide
from filechooser import show_file_selector_dialog
from indent import indent, unindent, remove_trailing_spaces
from indent import remove_beginning_spaces, convert_tabs_to_spaces
from brackets import match_bracket
from close import close_window
from lines import delete_line, join_line, open_line_below, open_line_above
from lines import delete_cursor_to_line_end, delete_cursor_to_line_begin
from select import select_line, select_paragraph, select_sentence
from select import select_bracket
from case import case
from internationalization import msg0145
from feedback import update_status_info
from bookmark import create_bookmark, remove_bookmark, remove_all_bookmarks
from bookmark import next_bookmark, previous_bookmark
from gotobar import show_gotobar
from findbar import show_findbar
from replacebar import show_replacebar
from syntaxcoloreditor import ColorEditor
from templatedialog import TemplateDialog
from readonly import leave_readonly_mode, enter_readonly_mode


################################################################################
#
#						Action Callbacks
#
################################################################################


def new_file_cb(action, editor):
	"""
	Handles callback to create a new instance of the editor.

	@param action: Reference to an action that spawns a new instance of the
				text editor.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Create a new GTK+ mainloop. At any point in time, there must be at least
	# one GTK+ mainloop running. When an instance of the text editor is
	# destroyed, its GTK+ mainloop is destroyed along with it. Thus, a new
	# mainloop should be created as long as an editor object exists.
	quit_add(0, gmain)
	from main import launch_new_editor
	launch_new_editor()

	return True


def open_file_cb(action, editor):
	"""
	Handles callback to show the text editor's fileselector dialog.

	@param action: Reference to an action that shows the text editor's
				fileselector dialog.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Show the fileselector dialog.
	show_file_selector_dialog(editor)

	return True


def save_file_cb(action, editor):
	"""
	Handles callback to save the file loaded in the text editor.

	@param action: Reference to an action to save the file loaded in the current
				text editor.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the save operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Save the file loaded in the text editor.
		SaveFile(editor)

	return True


def close_file_cb(action, editor):
	"""
	Handles callback to close the text editor's window.

	@param action: Reference to an action that closes the text editor's
				window.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Close the text editor's window.
	close_window(editor)

	return True


def saveas_file_cb(action, editor):
	"""
	Handles callback to show the text editor's save dialog.

	@param action: Reference to an action that shows the text editor's
				save dialog.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Show the text editor's save dialog.
	show_save_dialog(editor)

	return True


def print_file_cb(action, editor):
	"""
	Handles callback to show the text editor's print dialog.

	@param action: Reference to an action that shows the text editor's
				print dialog.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Show the text editor's print dialog.
	show_print_dialog(editor)

	return True


def undo_edits_cb(action, editor):
	"""
	Handles callback to undo an operation in the text editor's buffer.

	@param action: Reference to an action that performs an undo operation in the
				text editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Undo the last text operation in the text editor's buffer.
		undo_action(editor)

	return True


def redo_edits_cb(action, editor):
	"""
	Handles callback to redo an undone operation in the text editor's buffer.

	@param action: Reference to an action that performs a redo operation in the
				text editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Redo the last undone text operation in the text editor's buffer.
		redo_action(editor)

	return True


def goto_line_cb(action, editor):
	"""
	Handles callback to show the text editor's goto bar.

	@param action: Reference to an action that shows the text editor's goto bar.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Show the text editor's goto bar.
	show_gotobar(editor)

	return True


def show_findbar_cb(action, editor):
	"""
	Handles callback to show the text editor's find bar.

	@param action: Reference to an action that shows the text editor's
				find bar.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Show the text editor's find bar.
	show_findbar(editor)

	return True


def show_replacebar_cb(action, editor):
	"""
	Handles callback to show the text editor's replace bar.

	@param action: Reference to an action that shows the text editor's
				replace bar.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Show the text editor's replace bar.
		show_replacebar(editor)

	return True


def preferences_cb(action, editor):
	"""
	Handles callback to show the text editor's preferences dialog.

	@param action: Reference to an action that shows the text editor's
				preferences dialog.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Show the text editor's preferences dialog.
	Preferences(editor)

	return True


def help_cb(action, editor):
	"""
	Handles callback to launch the GNOME help browser.

	@param action: Reference to an action that launches the GNOME help browser.
	@type action: A gtk.Action object.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Launch the GNOME help browser, Yelp.
	launch_user_guide(editor)

	return True


def match_bracket_cb(action, editor):
	"""
	Handles callback to find matching brackets in text editor's buffer.

	@param action: Reference to an action that finds matching brackets in the
				text editor's buffer.
	@type action: An action object

	@param editor: Reference to the editor instance
	@type editor: An editor object

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Find matching brackets.
		match_bracket(editor)

	return True


def indent_cb(action, editor):
	"""
	Handles callback to indent a line or selected lines in the text editor's
	buffer.

	@param action: Reference to an action that indents a line or selected lines
				in the text editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance
	@type editor: An editor object

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Indent lines or selected lines in the text editor's buffer.
		indent(editor)

	return True


def unindent_cb(action, editor):
	"""
	Handles callback to unindent a line or selected lines in the text editor's
	buffer.

	@param action: Reference to an action that unindents a line or selected
				lines in the text editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Unindent line or lines in the text editor's buffer.
		unindent(editor)

	return True


def delete_line_cb(action, editor):
	"""
	Handles callback to delete the contents of a line in the text editor's
	buffer.

	@param action: Reference to an action that deletes the contents of a line in
					the editor's text buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: A gtk.Action object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Delete the contents of a line in the text editor's buffer.
		delete_line(editor)

	return True


def join_line_cb(action, editor):
	"""
	Handles callback to append text from the next line to the current line in
	the text editor's buffer.

	@param action: Reference to an action that appends text from the next line
				to the current line in the text editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Remove spaces at the end of the current line in the text buffer then
		# append the text on the next line to the current one.
		remove_trailing_spaces(editor, True)
		join_line(editor)

	return True


def select_word_cb(action, editor):
	"""
	Handles callback to select a word in the text editor's buffer.

	@param action: Reference to an action to select a word in the tex editor's
				buffer.
	@type action: A gtk.Action

	@param editor: Reference to the editor instance
	@type editor: An editor object

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		from cursor import get_cursor_iter
		iterator = get_cursor_iter(editor)
		from word import select_word
		result = select_word(editor.text_buffer, iterator)

		if result:

			from internationalization import msg0007
			update_status_info(editor, msg0007, "yes")

		else:

			from internationalization import msg0008
			update_status_info(editor, msg0008, "no")

	return True


def select_paragraph_cb(action, editor):
	"""
	Handles callback to select a paragraph in the text editor's buffer.

	@param action: Reference to an action to select a paragraph in the text
				editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Select a paragraph in the text editor's buffer.
		select_paragraph(editor)

	return True


def select_line_cb(action, editor):
	"""
	Handles callback to select a line in the text editor's buffer.

	@param action: Reference to an action to select a line in the text
				editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Select a line in the text editor's buffer.
		select_line(editor)

	return True


def select_sentence_cb(action, editor):
	"""
	Handles callback to select a sentence in the text editor's buffer.

	@param action: Reference to an action to select a sentence in the text
				editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Select a sentence in the text editor's buffer.
		select_sentence(editor)

	return True


def uppercase_cb(action, editor):
	"""
	Handles callback to change selected text in the text editor's buffer from
	lowercase to uppercase.

	@param action: Reference to an action to change a selected text in the text
				editor's buffer from lowercase to uppercase.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Change selected text in the text editor's buffer from lowercase to
		# uppercase.
		case(editor, "upper")

	return True


def lowercase_cb(action, editor):
	"""
	Handles callback to change selected text in the text editor' buffer from
	uppercase to lowercase.

	@param action: Reference to an action to change a selected text in the text
				editor's buffer from uppercase to lowercase.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Change selected text in the text editor's buffer from uppercase to
		# lowercase.
		case(editor, "lower")

	return True


def swapcase_cb(action, editor):
	"""
	Handles callback to swap the case of selected text in the text editor's
	buffer.

	@param action:	An action to swap the case of selected text
	@type action:	An action object

	@param editor:	Reference to the editor instance
	@type editor:		An editor object

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Swap the cases of selected text in the text editor's buffer.
		case(editor, "swap")

	return True


def titlecase_cb(action, editor):
	u"""
	Handles callback to capitalize a selected text in the text editor's buffer.

	@param action: Reference to an action to capitalize selected text in the
				text editor's buffer.
	@type action: A gtk.Action

	@param editor: Reference to the editor's instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Capitalize selected text in the text editor's buffer.
		case(editor, "title")

	return True


def remove_spaces_cb(action, editor):
	"""
	Handles callback to remove trailing spaces at the end of a line or lines in
	the text editor's buffer.

	@param action: Reference to an action to remove trailing spaces at the end
				of a line or lines in the text editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Remove trailing spaces at the end of a line or lines in the text
		# editor's buffer.
		remove_trailing_spaces(editor)

	return True


def remove_beginning_spaces_cb(action, editor):
	u"""
	Handles callback to remove beginning spaces at the end of a line or lines in
	the text editor's buffer.

	@param action: Reference to an action to remove beginning spaces at the end
				of a line or lines in the text editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Remove spaces at the beginning of a line or lines in the text editor's
		# buffer.
		remove_beginning_spaces(editor)

	return True


def open_line_above_cb(action, editor):
	"""
	Handles callback to shift the contents of the current line to the next one
	in the text editor's buffer.

	This action creates an illusion of creating a new line above the current
	line. Hence the function is named open_line_above. In reality the function
	just shifts contents of the current line to the next one.

	@param action: An action to shift the contents of the current line to the
				next one in the text editor's buffer.
	@type action: An gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Shift the contents of the current line to the next one in the text
		# editor's buffer.
		open_line_above(editor)

	return True


def open_line_below_cb(action, editor):
	"""
	Handles callback to open a new line below the current line in the text
	editor's buffer.

	@param action: An action that opens a new line below the current line in
				the text editor's buffer.
	@type action: An gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Remove spaces at the end of the current line in the text buffer then
		# append the text on the next line to the current one.
		remove_trailing_spaces(editor, True)
		open_line_below(editor)

	return True


def delete_cursor_to_line_end_cb(action, editor):
	"""
	Handles callback to delete text from the cursor to the end of the current
	line in the text editor's buffer.

	@param action: An action that deletes text from the cursor to the end of
				line in the text editor's buffer.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Delete text from the cursor to the end of the line in the text
		# editor's buffer.
		delete_cursor_to_line_end(editor)

	return True


def delete_cursor_to_line_begin_cb(action, editor):
	"""
	Handles callback to delete text from the cursor to the beginning of the
	current line in the text editor's buffer.

	@param action: An action that deletes text from the cursor to the beginning
	of the current line in the text editor's buffer.
	@type action: A gtk.Action

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		# Delete text from the cursor to the beginning of the current line in
		# text editor's buffer.
		delete_cursor_to_line_begin(editor)

	return True


def bookmark_cb(action, editor):
	"""
	Handles callback when a line is bookmarked in the text editor's buffer.

	@param action: An action that bookmarks a line in the text editor's buffer
	@type action: A gtk.Action

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		create_bookmark(editor)

	return True


def remove_bookmark_cb(action, editor):
	"""
	Handles callback when a line is bookmarked in the text editor's buffer.

	@param action: An action that removes a bookmark on a line in the text
				editor's buffer
	@type action: A gtk.Action

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		remove_bookmark(editor)

	return True


def remove_all_bookmarks_cb(action, editor):
	"""
	Handles callback when a line is bookmarked in the text editor's buffer.

	@param action: An action that removes all bookmarks in the text editor's
				buffer.
	@type action: A gtk.Action

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		remove_all_bookmarks(editor)

	return True


def next_bookmark_cb(action, editor):
	"""
	Handles callback when a line is bookmarked in the text editor's buffer.

	@param action: An action that moves to the next bookmark in the text
				editor's buffer
	@type action: A gtk.Action

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		next_bookmark(editor)

	return True


def previous_bookmark_cb(action, editor):
	"""
	Handles callback when a line is bookmarked in the text editor's buffer.

	@param action: An action that moves to the previous bookmark in the text
				editor's buffer
	@type action: A gtk.Action

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		previous_bookmark(editor)

	return True

def show_bookmark_browser(action, editor):
	"""
	Handles callback to show a bookmark browser.

	@param action: An action that moves to the previous bookmark in the text
				editor's buffer
	@type action: A gtk.Action

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	try:

		editor.bookmark_browser.show()

	except AttributeError:

		from bookmarkbrowser import ScribesBookmarkBrowser
		editor.bookmark_browser = ScribesBookmarkBrowser(editor)
		editor.bookmark_browser.show()

	return True


def convert_tabs_to_spaces_cb(action, editor):
	"""
	Handles callback to convert tabs to spaces in the text editor's buffer.

	@param action: An action to convert tabs to spaces.
	@type action: A gtk.Action

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Prevent this action from occurring when the text editor is in readonly
	# mode.
	if editor.readonly:

		# Feedback to the status bar indicating the operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

	else:

		convert_tabs_to_spaces(editor)

	return True


def select_bracket_cb(action, editor):
	"""
	Handles callback to select characters with brackets or quotes.

	@param action: An action to select characters with brackets or quotes.
	@type action: A gtk.Action

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	if editor.readonly:

		update_status_info(editor, msg0145, "no")

	else:

		select_bracket(editor)

	return True


def toggle_readonly_cb(action, editor):
	"""
	Handles callback to toggle readonly mode on or off.

	@param action: An action to toggle readonly mode.
	@type action: A gtk.Action

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	if editor.readonly:

		leave_readonly_mode(editor)

	else:

		enter_readonly_mode(editor)

	return True


def show_syntax_editor_cb(action, editor):
	"""
	Handles callback to display the color editor.

	@param action: An action to display the color editor.
	@type action: A gtk.Action

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	ColorEditor(editor)

	return True


def show_template_editor_cb(action, editor):
	"""
	Handles callback to display the template editor.

	@param action: An action to display the template editor.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	TemplateDialog(editor)

	return True


def jump_to_last_bookmarked_line_cb(action, editor):
	"""
	Handles callback to display the template editor.

	@param action: An action to display the template editor.
	@type action: A gtk.Action object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	from bookmark import jump_to_last_bookmarked_line
	jump_to_last_bookmarked_line(editor)

	return True


################################################################################
#
#								Actions
#
################################################################################


def create_actions(editor):
	u"""
	Create an action group.

	Actions are unique function that the editor can perform.

	@param editor:	Reference to the editor instance.
	@type editor:	An editor object.

	@return:		An action group containing editor actions.
	@rtype:			An action group object.

	"""

	actiongroup = ActionGroup(u"EditorActions")

	new_file_action = ("new_file", STOCK_NEW, "_N", "<Control>n",
						new_button_tip, new_file_cb)

	open_file_action = ("open_file", STOCK_OPEN, None, "<Control>o",
							open_button_tip, open_file_cb)

	saveas_file_action = ("saveas_file", STOCK_SAVE_AS, None,
								"<Control><Shift>s", save_button_tip,
								saveas_file_cb)

	print_file_action = ("print_file", STOCK_PRINT, None, "<Control>p",
							print_button_tip, print_file_cb)

	undo_edits_action = ("undo_edits", STOCK_UNDO, None, None,
							undo_button_tip, undo_edits_cb)

	redo_edits_action = ("redo_edits", STOCK_REDO, None,
							None, redo_button_tip,
							redo_edits_cb)

	goto_line_action = ("goto_line", STOCK_JUMP_TO, None, "<Control>i",
							goto_button_tip, goto_line_cb)
	search_text_action = ("search_text", STOCK_FIND, None, "<Contro>f",
								find_button_tip, show_findbar_cb)
	replace_text_action = ("replace_text", STOCK_FIND_AND_REPLACE, None,
								"<Contro>r", replace_button_tip,
								show_replacebar_cb)
	preferences_action = ("preferences", STOCK_PREFERENCES, None, None,
								pref_button_tip, preferences_cb)
	help_action = ("help", STOCK_HELP, None, None, help_button_tip,
					help_cb)
	save_file_action = ("save_file", None, "_save_file", None, None,
							save_file_cb)
	close_file_action = ("close_file", None, "_close_file", None, None,
							close_file_cb)
	match_bracket_action = ("match_bracket", None, "_mb" , None, None,
							match_bracket_cb)
	indent_action = ("indent", None, "ind", None, None, indent_cb)
	unindent_action = ("unindent", None, "unind", None, None, unindent_cb)
	delete_line_action = ("delete_line", None, "del", None, None,
							delete_line_cb)
	join_line_action = ("join_line", None, "join", None, None, join_line_cb)
	select_word_action = ("select_word", None, "word", None, None,
							select_word_cb)
	select_paragraph_action = ("select_paragraph", None, "para", None, None,
								select_paragraph_cb)
	uppercase_action = ("uppercase", None, "uw", None, None, uppercase_cb)
	lowercase_action = ("lowercase", None, "lw", None, None, lowercase_cb)
	swapcase_action = ("swapcase", None, "sw", None, None, swapcase_cb)
	titlecase_action = ("titlecase", None, "tw", None, None, titlecase_cb)
	select_line_action = ("select_line", None, "sl", None, None,
							select_line_cb)
	select_sentence_action = ("select_sentence", None, "st", None, None,
								select_sentence_cb)
	remove_spaces_action = ("remove_spaces", None, "rs", None, None,
								remove_spaces_cb)
	open_line_below_action = ("open_line_below", None, "olb", None, None,
								open_line_below_cb)
	open_line_above_action = ("open_line_above", None, "ola", None, None,
								open_line_above_cb)
	delete_cursor_to_end_action = ("delete_cursor_to_end", None, "dcte", None,
									None, delete_cursor_to_line_end_cb)
	delete_cusrsor_to_begin_action = ("delete_cursor_to_begin", None, "dctb",
										None, None,
										delete_cursor_to_line_begin_cb)
	remove_beginning_spaces_action = ("remove_begin", None, "rb", None, None,
										remove_beginning_spaces_cb)
	bookmark_action = ("bookmark", None, "bk", None, None, bookmark_cb)
	remove_bookmark_action = ("remove_bookmark", None, "rk", None, None,
							remove_bookmark_cb)
	remove_all_bookmarks_action = ("remove_all_bookmarks", None, "rab", None,
									None, remove_all_bookmarks_cb)
	next_bookmark_action = ("next_bookmark", None, "nb", None, None,
							next_bookmark_cb)
	previous_bookmark_action = ("previous_bookmark", None, "pb", None, None,
								previous_bookmark_cb)
	show_bookmark_browser_action = ("show_bookmark_browser", None, "sbb", None,
									None, show_bookmark_browser)
	convert_tabs_to_spaces_action = ("tabs_to_spaces", None, "tts", None, None,
									convert_tabs_to_spaces_cb)
	select_brackets_action = ("select_bracket", None, "sb", None, None,
							select_bracket_cb)
	readonly_action = ("toggle_readonly", None, "tr", None, None,
						toggle_readonly_cb)
	syntax_editor_action = ("show_syntax_editor", None, "cea", None, None,
							show_syntax_editor_cb)
	template_editor_action = ("show_template_editor", None, "ste", None, None,
								show_template_editor_cb)

	toggle_readonly_action = ("toggle_readonly", None, "tr", None, None,
								toggle_readonly_cb)

	jump_to_last_bookmarked_line_action = ("jump_to_last_bookmarked_line", None,
										"jtlbl", None, None,
										jump_to_last_bookmarked_line_cb)

	# Action Entries.
	entries = [new_file_action, open_file_action, saveas_file_action,
			print_file_action, undo_edits_action, redo_edits_action,
			goto_line_action, search_text_action, replace_text_action,
			preferences_action, help_action, save_file_action,
			close_file_action, match_bracket_action, indent_action,
			unindent_action, delete_line_action, join_line_action,
			select_word_action, select_paragraph_action, uppercase_action,
			lowercase_action, select_line_action, select_sentence_action,
			remove_spaces_action, open_line_below_action,
			open_line_above_action, delete_cursor_to_end_action,
			delete_cusrsor_to_begin_action, remove_beginning_spaces_action,
			titlecase_action, swapcase_action, bookmark_action,
			remove_bookmark_action, remove_all_bookmarks_action,
			next_bookmark_action, previous_bookmark_action,
			convert_tabs_to_spaces_action, select_brackets_action,
			readonly_action, syntax_editor_action, template_editor_action,
			toggle_readonly_action, show_bookmark_browser_action,
			jump_to_last_bookmarked_line_action]

	# Create Actions for actiongroup
	actiongroup.add_actions(entries, user_data=editor)

	return actiongroup

