# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module implements functions that provide automatic word completion for the
text editor.

Automatic word completion works by scanning the document for strings that
begin with the first three or more characters after the cursor. The characters
must be alphanumeric.

When matches are found, a completion window appears displaying a list of the
found matches. Users can then use either the up and down arrow keys or the
mouse pointer to select and activate the desired string for word completion.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
FIXME: Refactor!!!
"""

from collections import deque

from pygtk import require
require("2.0")
from gobject import timeout_add
from gtk.gdk import WINDOW_TYPE_HINT_MENU
from gtk import TreeView, ListStore, CellRendererText
from gtk import TreeViewColumn, Window, WINDOW_POPUP
from gtk import POLICY_NEVER, POLICY_ALWAYS
from gtk import ScrolledWindow, SHADOW_IN, TEXT_WINDOW_TEXT

from cursor import get_string_to_cursor
from cursor import get_cursor_window_coordinates, get_cursor_size
from feedback import update_status_info
from internationalization import msg0155
from utility import response
from timeout import unblock_insert_signal_timeout


def insert_word_completion(path, editor):
	"""
	Insert items selected in the completion window into the text editor's
	buffer.

	@param path: The selected row in the completion window.
	@type path: A gtk.TreeRow object.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	"""

	# Begin recording for undo operation.
	#editor.text_buffer.begin_user_action()
	editor.text_buffer.handler_block(editor.tb_ins_id)

	# Get the database containing potential completion string matches.
	model = editor.view.get_model()

	# Get the selected completion string.
	completion_string = model[path[0]][0].decode("utf8")

	# Index to split completion string for insertion into the text editor's
	# buffer. Encode to utf8 before insertion.
	index = len(editor.valid_string.decode("utf8"))
	string = completion_string[index:]

	# Split completion_string at the right index and insert into the editor's
	# buffer.
	editor.text_buffer.insert_at_cursor(string)

	editor.text_buffer.handler_unblock(editor.tb_ins_id)
	# End recording for undo operation.
	# editor.text_buffer.end_user_action()

	# Feedback to the status bar indicating word completion occurred.
	update_status_info(editor, msg0155, "yes")

	return


def row_activated_cb(treeview, path, column, editor):
	"""
	Handles callback when item in completion window is selected and activated.

	@param path: The selected row in the completion window.
	@type path: A gtk.TreeRow object.

	@param column: The column holding the completion list.
	@type column: A gtk.TreeViewColumn

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Insert selected word into the text editor's buffer and hide there
	# completion window.
	insert_word_completion(path, editor)
	hide_completion_window(editor)

	return


def button_press_event_cb(view, event):
	"""
	Handles callback when an item in the completion	window is clicked on.

	@param view: The completion window's treeview.
	@type view: A gtk.TreeView object.

	@param event: An event generated by clicking on the mouse button.
	@type event: A gtk.Event object.

	"""

	# Get the selected item on the completion window.
	selection = view.get_selection()

	# Get the model and iterator of the selected item.
	model, iterator = selection.get_selected()

	# Get the path and column of the selected item.
	path = model.get_path(iterator)
	column = view.get_column(0)

	# Activate the selected item to trigger word completion.
	view.row_activated(path, column)

	return


def create_completion_renderer():
	"""
	Create a text cell renderer for the text editor's word completion window.

	@return: An object that renders text into the text editor's word completion
			window.
	@rtype: A gtk.CellRendererText object.

	"""

	renderer = CellRendererText()

	return renderer


def create_completion_column(editor):
	"""
	Create a column for the text editor's word completion window.

	@return: A visible column in the text editor's word completion window.
	@rtype: A gtk.TreeViewColumn object.

	"""

	column = TreeViewColumn(u"", create_completion_renderer(), text=0)
	editor.column = column
	column.set_expand(False)

	return column


def create_completion_model():
	"""
	Create the database to contain potential matches for word completion.

	@return: A list model to use for the text editor's completion window
			treeview.
	@rtype: A gtk.ListStore object.

	"""

	model = ListStore(str)

	return model


def create_completion_scrollwindow():
	"""
	Create a scroll window widget for the text editor's word completion window.

	@return: A widget for scrolling through potential matches for word completion
			in the text editor word completion window.
	@rtype: A gtk.ScrolledWindow object.

	"""

	scrollwin = ScrolledWindow()
	scrollwin.set_shadow_type(SHADOW_IN)

	return scrollwin


def create_completion_treeview(editor):
	"""
	Create the treeview for the text editor's word completion window.

	@return: A wigdet for displaying a list containing potential matches for
			word completion.
	@rtype: A gtk.TreeView object.

	"""

	scrollwindow = create_completion_scrollwindow()
	editor.scrollwindow = scrollwindow
	view = TreeView()
	scrollwindow.add(view)
	view.connect("row-activated", row_activated_cb, editor)
	view.connect("button-press-event", button_press_event_cb)
	view.append_column(create_completion_column(editor))
	view.set_headers_visible(False)
	view.set_property("rules-hint", True)
	view.set_property("hover-selection", True)

	return view


def create_completion_window():
	"""
	Create the text editor's word completion window.

	@return: A completion window to house potential matches for word completion.
	@rtype: A gtk.Window object.

	"""

	window = Window(WINDOW_POPUP)
	window.set_property("accept-focus", True)
	window.set_property("can-focus", True)
	window.set_property("allow-grow", False)
	window.set_property("allow-shrink", False)
	window.set_property("decorated", False)
	window.set_property("resizable", False)
	window.set_property("skip-pager-hint", True)
	window.set_property("skip-taskbar-hint", True)
	window.set_property("type-hint", WINDOW_TYPE_HINT_MENU)
	window.set_property("urgency-hint", False)

	return window


def update_completion_model(editor, completion_list):
	"""
	Populate the text editor's word completion database with potential matches
	for word completion.

	@return: A database populated with potential matches for word completion.
	@rtype: A gtk.ListStore object.

	"""

	# Remove all entries in the text editor's word completion database. If a
	# database does not exist, create one.
	try:

		editor.model.clear()

	except AttributeError:

		editor.model = create_completion_model()
		editor.model.clear()

	# Populate the database with potential matches for word completion.
	for matches in completion_list:

		response()
		editor.model.append([matches])

	return editor.model


def calculate_completion_window_position(editor, width, height):
	"""
	Calculate the position to place the word completion window in the text
	editor's buffer.

	@param editor: Reference to the editor object.
	@type editor: An editor object

	@param width: The width of the text editor's completion window.
	@type width: An integer object.

	@param height: The height of the text editor's completion window.
	@type height: An integer object.

	"""

	# The flag is true when the position of the word completion window needs to
	# adjusted accross the y-axis.
	editor.y_coordinate_flag = False

	# Get the cursor's coordinate and size.
	cursor_x, cursor_y = get_cursor_window_coordinates(editor)
	cursor_height = get_cursor_size(editor)[1]

	# Get the text editor's textview coordinate and size.
	window = editor.text_view.get_window(TEXT_WINDOW_TEXT)
	rectangle = editor.text_view.get_visible_rect()
	window_x, window_y = window.get_origin()
	window_width, window_height = rectangle.width, rectangle.height

	# Determine where to position the completion window.
	position_x = window_x + cursor_x
	position_y = window_y + cursor_y + cursor_height

	# If the completion window extends past the text editor's buffer,
	# reposition the completion window inside the text editor's buffer area.
	if (position_x + width) > (window_x + window_width):

		position_x = (window_x + window_width) - width

	if (position_y + height) > (window_y + window_height):

		position_y = (window_y + cursor_y) - height
		editor.y_coordinate_flag = True

	return position_x, position_y


def calculate_completion_window_size(editor):
	"""
	Determine the size of the text editor's completion window.

	The completion window should be as wide as the longest word but no less than
	200 pixel wide. Its height should be as long as its list but no more than
	200 pixels high.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	"""

	# Get the size of the text editor's completion window treeview. Use this
	# information to determine the size of the text editor's completion
	# window.

	#editor.view.columns_autosize()
	width, height = editor.view.size_request()

	if width <= 200 and height <= 200:

		editor.scrollwindow.set_policy(POLICY_NEVER, POLICY_NEVER)
		width = 200

	elif width >= 200 and height <= 200:

		editor.scrollwindow.set_policy(POLICY_NEVER, POLICY_NEVER)

	elif width <= 200 and height >= 200:

		editor.scrollwindow.set_policy(POLICY_NEVER, POLICY_ALWAYS)
		width = 228
		height = 200

	else:

		editor.scrollwindow.set_policy(POLICY_NEVER, POLICY_ALWAYS)
		width += 28
		height = 200

	return width, height


def update_completion_window(editor, completion_list):
	"""
	Update the text editor's word completion window.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	@param completion_list: A list containing potential matches for word
							completion.
	@type completion_list: A deque object (A faster version of a list)

	"""

	# Update the completion window's view with potential matches for word
	# completion. Create a treeview if none does not exist.
	try:

		editor.view.set_model(update_completion_model(editor, completion_list))

	except AttributeError:

		editor.view = create_completion_treeview(editor)
		editor.view.set_model(update_completion_model(editor, completion_list))

	column = editor.view.get_column(0)
	editor.view.set_cursor((0,), column, start_editing=False)
	editor.view.queue_draw()
	editor.view.queue_resize()

	# Create a scroll window if one has not been created already.
	if not editor.created_flag:

		try:

			editor.completion_window.add(editor.scrollwindow)

		except AttributeError:

			editor.completion_window = create_completion_window()
			editor.completion_window.add(editor.scrollwindow)

	# Set the width and height of the text editor's completion window.
	width, height = calculate_completion_window_size(editor)
	editor.completion_window.set_property("width-request", width)
	editor.completion_window.set_property("height-request", height)

	# Set the position of the text editor's completion window.
	position_x, position_y = calculate_completion_window_position(editor, width,
																height)

	# If the completion window is already visible there is no need to position
	# it. The only exception is when the completion window needs to be adjusted
	# accross the y-axis to prevent it from crossing the text editor's buffer
	# boundaries.
	if editor.completion_flag:

	#	if editor.y_coordinate_flag:
		x_coordinate = editor.completion_window.get_position()[0]
		editor.completion_window.move(x_coordinate, position_y)

	else:

		editor.completion_window.move(position_x, position_y)

	editor.view.columns_autosize()

	return


def show_completion_window(editor, completion_list):
	"""
	Show the text editor's word completion window.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	@param completion_list: A list containing potential matches for word
							completion.
	@type completion_list: A deque object (A faster version of a list)

	"""

	# Populate the completion window with a list of potential matches for
	# word completion. Determine the size of the text editor's word completion
	# window in the text editor's buffer. Determine where to place the word
	# completion window.
	update_completion_window(editor, completion_list)

	# Show the completion_window.
	editor.completion_window.show_all()
	editor.completion_window.window.process_updates(True)

	# A flag indicating the visibility of the completion window. True if the
	# window is visible. False otherwise.
	editor.completion_flag = True

	return


def hide_completion_window(editor):
	"""
	Hide the text editor's completion window.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	try:

		editor.completion_window.hide_all()
		editor.completion_flag = False
		editor.previous_list = deque([])

	except AttributeError:

		pass

	return


def update_view(editor, match_list):
	"""
	Determine whether or not to refresh the text editor's word completion
	window.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	@param match_list: A list containing potential matches for word completion.
	@type match_list: A deque object.

	@return: True if the word completion window should be updated, false
		otherwise.
	@rtype: A boolean object.

	"""

	result = False

	# Do not update the text editor's word completion window if the previous
	# list is the same as the current one.
	try:

		if editor.previous_list == match_list:

			result = False

		else:

			result = True

		editor.previous_list = match_list

	except AttributeError:

		editor.previous_list = match_list

	return result


def move_completion_window_selection(editor, keyval, keys):
	"""
	Move the selection on the completion window up or down one row.

	@param editor: Reference to the text editor.
	@type editor: An editor object.

	@param keyval: An event generated by pressing a character on the keyboard.
	@type keyval: A gtk.Event.

	"""

	# Get the selected item on the completion window.
	selection = editor.view.get_selection()

	# Get the model and iterator of the selected item.
	model, iterator = selection.get_selected()

	# If for whatever reason the selection is lost, select the first row
	# automactically when the up or down arrow key is pressed.
	if not iterator:

		selection.select_path((0,))
		model, iterator = selection.get_selected()

	path = model.get_path(iterator)

	if keyval == keys["up"]:

		# If the up key is pressed check to see if the first row is selected.
		# If it is, select the last row. Otherwise, get the path to the row
		# above and select it.
		if not path[0]:

			number_of_rows = len(model)
			selection.select_path(number_of_rows-1)
			editor.view.scroll_to_cell(number_of_rows-1)

		else:

			selection.select_path((path[0]-1,))
			editor.view.scroll_to_cell((path[0]-1,))

	elif keyval == keys["down"]:

		# Get the iterator of the next row.
		next_iterator = model.iter_next(iterator)

		# If the next row exists, select it, if not select the first row.
		if next_iterator:

			selection.select_iter(next_iterator)
			path = model.get_path(next_iterator)
			editor.view.scroll_to_cell(path)

		else:

			selection.select_path(0)
			editor.view.scroll_to_cell(0)

	elif keyval == keys["enter"]:

		# Insert the selected item into the editor's buffer when the enter key
		# event is detected.
		editor.view.row_activated(path, editor.view.get_column(0))

	else:

		hide_completion_window(editor)

	return


def manipulate_completion_window(editor, keyval, keys):
	"""
	Move selections in the text editor's word completion or hide the window.

	If the completion window is visible, the up or down arrow keys are blocked.
	The keys will instead be used to move selections in the completion window's
	treeview.

	If the completion window is visible, we validate the characters for
	possible automatic completion. See module autocomplete.py for the function
	definition.

	If the completion window is visible, and the key pressed is not an
	alphanumeric key, we hide the completion window. Also hide the window
	if the key pressed are left of right.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	@param keyval: Key code representing a keyboard character pressed.
	@type keyval: An integer object.

	@param keys: A dictionary containing selected key codes.
	@type keys: A dictionary object.

	@return: A value indicating whether or not to propagate the key event
			signal. False to propagate the signal, an True otherwise.
	@rtype: A boolean value.

	"""


	if editor.completion_flag:

		if keyval in (keys["up"], keys["down"], keys["right"],
					keys["left"], keys["enter"] ):

			move_completion_window_selection(editor, keyval, keys)
			value = True

		elif keyval == keys["backspace"]:

			hide_completion_window(editor)
			value = False

		elif keyval in (keys["control"], keys["shift"], keys["alt"],
						keys["underscore"]):

			value = False

		elif not unichr(keyval).isalnum():

			hide_completion_window(editor)
			value = False

		else:

			value = False

	else:

		value = False

	return value


def search_for_matches(cursor_string, dictionary):
	"""
	Search the contents of the text editor's buffer for potential word
	completion matches.

	@param cursor_string: A string after a space character preceding the cursor.
	@type cursor_string: A string object.

	@param unicode_string: The contents of the text editor's buffer in unicode.
	@type unicode_string: A unicode string object.

	@return: A list containing potential words for word completion.
	@rtype: A list object.

	"""

	# Fixme: Add comment.
	match_list = [list(items) for items in dictionary.items() \
			if items[0].startswith(cursor_string) and items[0] != cursor_string]

	if match_list:

		wank_list = []
		from utility import response

		for items in match_list:

			response()
			items.reverse()
			wank_list.append(items)

		wank_list.sort()
		wank_list.reverse()
		word_completion_list = [items[1] for items in wank_list]

	else:

		word_completion_list = None

	return word_completion_list


def run_word_completion_check(editor, cursor_string):
	"""
	Scan the text editor's buffer to determine whether a string is eligible for
	word completion.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	@param cursor_string: A string 3 characters long before the cursor
	@type cursor_string: A string.

	@return: A list containing potential matches for word completion or nothing.
	@rtype: A list object or None

	"""

	found_matches = search_for_matches(cursor_string,
										editor.word_completion_dictionary)

	if found_matches:

		# Show or update the completion window with a list of possible word
		# completion candidates.
		if editor.completion_flag:

			# Check whether or not to update the editor's completion window
			# view.
			result = update_view(editor, found_matches)

			if result:

				update_completion_window(editor, found_matches)

		else:

			show_completion_window(editor, found_matches)
			editor.created_flag = True

	else:

		hide_completion_window(editor)

	return


def validate_cursor_string(editor):
	"""
	Check if the characters before the cursor meet the criteria for automatic
	word completion.

	Automatic word completion is activated only after three alphanumeric
	characters	can be retrieved before the cursor.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	"""

	# Get the string or word before the cursor.
	string = get_string_to_cursor(editor)

	# If the string meets the criteria for automatic completion, scan the buffer
	# for possible completion strings.
	if string:

		string.decode("utf8")
		editor.valid_string = string
		timeout_add(3, run_word_completion_check, editor, string)

	else:

		timeout_add(3, hide_completion_window, editor)

	return


def block_word_completion(editor):
	"""
	Disable word completion in the text editor's buffer.

	@param editor: Reference to an editor instance.
	@type editor: An editor object

	"""

	# Block the "insert-text" signal to disable automatic word completion.
	editor.text_buffer.handler_block(editor.tb_ins_id)
	editor.blocked_word_completion = True

	return


def unblock_word_completion(editor):
	"""
	Enable word completion in the text editor's buffer.

	@param editor: Reference to an editor instance.
	@type editor: An editor object

	"""

	if editor.blocked_word_completion:

		# Unblock the "insert-text" signal to enable automatic word completion
		# after 100 msecs.
		timeout_add(100, unblock_insert_signal_timeout, editor)

	return

