# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This modules contains functions to create, remove and traverse bookmarks.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from sys import argv
from os import path
from gc import collect

from pygtk import require
require("2.0")
from gtk import Image

from cursor import get_current_line, get_cursor_iter
from feedback import update_status_info
from internationalization import msg0164, msg0165, msg0166, msg0167, msg0168
from internationalization import msg0169, msg0170, msg0171, msg0172


def find_bookmark_image():
	"""
	Search for the bookmark image folder on the host's file system.

	@return: Folder path to the bookmark image
	@rtype: A string

	"""

	head, tail = path.split(path.split(path.abspath(argv[0]))[0])

	if tail ==  "bin":

		data_dir = path.join(head, "share")

		if path.exists(data_dir):

			bookmark_folder = path.join(data_dir, "scribes")

			if path.exists(bookmark_folder):

				return bookmark_folder

			else:

				print "Cannot find bookmark folder"
		else:

			print "Cannot find bookmark folder"

	else:

		print "Cannot find bookmark folder"

	return


def create_bookmark_pixbuf(editor):
	"""
	Create the bookmark pixbuff.

	The little image that is show in line marker area.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	bookmark_folder = find_bookmark_image()
	image_file = path.join(bookmark_folder, "bookmarks.png")
	editor.bookmark_image_flag = True
	image = Image()
	image.set_from_file(image_file)
	editor.bookmark_pixbuf = image.get_pixbuf()

	return editor.bookmark_pixbuf


def update_bookmark_list(editor, flag="", marker=None):
	"""
	Maintain list of bookmarks in the text editor's buffer.

	The flag argument can either be create or remove. If the argument is add,
	the list is updated with the marker of the new bookmark that has just
	been created in the text editor's buffer. And if the argument is remove, the
	marker associated to the bookmark is removed from the list.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	@param flag: Flag can either be nothing, "add" or "remove"
	@type flag: A string.

	@param marker: A marker placed on a bookmarked line in the text editor's
				buffer.
	@type marker: A gtk.SourceMarker object.

	"""

	if not flag:

		return editor.bookmark_list

	elif flag == "remove":

		editor.bookmark_list.remove(marker)

	elif flag == "add":

		editor.bookmark_list.append(marker)

	if not editor.bookmark_list:

		editor.text_view.set_show_line_markers(False)
		editor.bookmark_image_flag = False
		editor.bookmark_flag = False
		collect()

	return editor.bookmark_list


def get_bookmarked_lines(editor):
	u"""
	Return a tuple of all bookmarked lines in the text editor's buffer.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	@return: All bookmarked lines
	@rtype: A tuple object

	"""

	bookmarked_lines = [marker.get_line() for marker in \
						update_bookmark_list(editor)]

	return tuple(bookmarked_lines)


def check_for_line_bookmark(editor):
	u"""
	Check if the line in the text editor's buffer is already bookmarked and
	return the marker if it is.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	@return: Return nothing if the line is not bookmarked or the a list
			containing the marker if it is.
	@rtype: None or a list object

	"""

	# Get the beginning and ending points in the current line of the text
	# editor

	cursor_line = get_current_line(editor)
	start_iter = editor.text_buffer.get_iter_at_line(cursor_line)
	end_iter = start_iter.copy()
	end_iter.forward_to_line_end()

	# Get the bookmark marker on the current line of the editor if any.

	marker = editor.text_buffer.get_markers_in_region(start_iter, end_iter)

	return marker


def bookmark_line(editor, line):
	"""
	Bookmark a specific line in the text editor's buffer.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	@param line: The line number to be bookmarked.
	@type line: An integer object.

	"""

	# Check if the line to be bookmarked actually exists.

	if line > editor.text_buffer.get_line_count() - 1:

		pass

	else:

		# Get the iterator at the beginning of line and create a marker.

		line_iter = editor.text_buffer.get_iter_at_line(line)
		editor.bookmark_flag = True
		marker = editor.text_buffer.create_marker(None, "bookmark", line_iter)

		# Show the line margin if it is not already shown.

		if not editor.text_view.get_show_line_markers():

			editor.text_view.set_show_line_markers(True)

		# Show the bookmark icon in the line margin.

		if editor.bookmark_image_flag:

			editor.text_view.set_marker_pixbuf("bookmark",
												editor.bookmark_pixbuf)

		else:

			editor.bookmark_pixbuf = create_bookmark_pixbuf(editor)
			editor.text_view.set_marker_pixbuf("bookmark",
											editor.bookmark_pixbuf)

		# Update the bookmark list.

		update_bookmark_list(editor, "add", marker)

	return


def create_bookmark(editor):
	"""
	Bookmark a line in the text editor's buffer.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# If the current line is already bookmarked, send feedback to the status bar
	# indicating the line is already bookmarked and exit the function.
	bookmark = check_for_line_bookmark(editor)
	cursor_line = get_current_line(editor)

	if bookmark:

		message = msg0164 % (cursor_line + 1)
		update_status_info(editor, message, "no")

	else:

		bookmark_line(editor, cursor_line)

		# Feedback to the status bar indicating the current line in the text
		# editor's buffer has been bookmarked.
		message = msg0165 % (cursor_line + 1)
		update_status_info(editor, message, u"yes")

	return


def remove_bookmark(editor):
	"""
	Remove bookmark on a line in the text editor's buffer.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# If the current line in the text editor's buffer is not bookmarked, send
	# feedback to the status bar indicating that.
	marker = check_for_line_bookmark(editor)
	cursor_line = get_current_line(editor)

	if not marker:

		message = msg0166 % (cursor_line + 1)
		update_status_info(editor, message, "no")

	else:

		# Delete the bookmark on the current line of the text editor's buffer.
		editor.text_buffer.delete_marker(marker[0])

		# Feedback to the status bar indicating the bookmark in current line in
		# the text editor's buffer has been removed.
		message = msg0167 % (cursor_line + 1)
		update_status_info(editor, message, "yes")
		collect()

	return


def remove_all_bookmarks(editor):
	"""
	Remove all bookmarks in the text editor's buffer.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# If there are no bookmarks in the text editor, send feedback to the status
	# bar indicating that.
	if not editor.bookmark_list:

		update_status_info(editor, msg0168, "no")

	else:

		# Get all the bookmarks in the text editor.
		start_iter, end_iter = editor.text_buffer.get_bounds()
		bookmark_list = editor.text_buffer.get_markers_in_region(start_iter,
																end_iter)

		# Delete all the bookmarks in the text editor.
		for markers in bookmark_list:

			editor.text_buffer.delete_marker(markers)


		editor.text_view.set_show_line_markers(False)
		editor.bookmark_list = []

		# Feedback to the status bar indicating the all bookmarks in the text
		# editor's buffer have been removed.
		update_status_info(editor, msg0169, u"yes")
		collect()

	return


def next_bookmark(editor):
	"""
	Move to the next bookmark in the text editor's buffer.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# If there are no bookmarks in the text editor, send feedback to the status
	# bar indicating that.
	if not editor.bookmark_list:

		update_status_info(editor, msg0168, "no")

		return

	# Search for the next bookmark marker starting from the cursor position in
	# the text editor's buffer.
	cursor_line = get_current_line(editor)
	cursor_iter = get_cursor_iter(editor)
	marker = editor.text_buffer.get_next_marker(cursor_iter)

	# If no bookmark marker is found, send feedback to the status bar indicating
	# no bookmark was found.
	if not marker:

		update_status_info(editor, msg0170, "no")

		return

	# Retrieve the line number the next bookmark was found on.
	marker_line = marker.get_line()

	# If the current line is already bookmarked, ignore the line when searching
	# for the next bookmark.
	if cursor_line == marker.get_line() and cursor_iter.starts_line():

		bookmarked_line_list = []

		for marker in editor.bookmark_list:

			line = marker.get_line()

			if line > marker_line:

				bookmarked_line_list.append(line)

		if bookmarked_line_list:

			bookmarked_line_list.sort()
			marker_line = bookmarked_line_list[0]

		else:

			update_status_info(editor, msg0170, "no")

			return

	# Place the cursor at the position of the next found bookmark.
	line_iter = editor.text_buffer.get_iter_at_line(marker_line)
	editor.text_buffer.place_cursor(line_iter)
	cursor_mark = editor.text_buffer.get_insert()
	editor.text_view.scroll_to_mark(cursor_mark, 0.3,
								use_align=False, xalign=0.5,
								yalign=0.5)

	# Feedback to the status bar indicating the cursor was moved to the found
	# bookmark.
	message = msg0171 % (marker_line + 1)
	update_status_info(editor, message, u"yes")

	return


def previous_bookmark(editor):
	"""
	Move to the previous bookmark in the text editor's buffer.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# If there are no bookmarks in the text editor, send feedback to the status
	# bar indicating that.
	if not editor.bookmark_list:

		update_status_info(editor, msg0168, "no")

		return

	# Search for the previous bookmark marker starting from the cursor position
	# in the text editor's buffer.
	cursor_line = get_current_line(editor)
	cursor_iter = get_cursor_iter(editor)
	marker = editor.text_buffer.get_prev_marker(cursor_iter)

	# If no bookmark marker is found, send feedback to the status bar indicating
	# no bookmark was found.
	if not marker:

		update_status_info(editor, msg0172, "no")

		return

	# Retrieve the line number the next bookmark was found on.
	marker_line = marker.get_line()

	# If the current line is already bookmarked, ignore the line when searching
	# for the previous bookmark.
	if cursor_line == marker.get_line() and cursor_iter.starts_line():

		bookmarked_line_list = []

		for marker in editor.bookmark_list:

			line = marker.get_line()

			if line < marker_line:

				bookmarked_line_list.append(line)

		if bookmarked_line_list:

			bookmarked_line_list.sort()
			marker_line = bookmarked_line_list[-1]

		else:

			update_status_info(editor, msg0172, "no")

			return

	# Place the cursor at the position of the previous found bookmark.
	line_iter = editor.text_buffer.get_iter_at_line(marker_line)
	editor.text_buffer.place_cursor(line_iter)
	cursor_mark = editor.text_buffer.get_insert()
	editor.text_view.scroll_to_mark(cursor_mark, 0.3,
								use_align=False, xalign=0.5,
								yalign=0.5)

	# Feedback to the status bar indicating the cursor was moved to the found
	# bookmark.

	message = msg0171 % (marker_line + 1)
	update_status_info(editor, message, "yes")

	return


def get_last_bookmarked_line(editor):
	# FIXME: Documentation
	lines = get_bookmarked_lines(editor)
	line = lines[-1]

	return line


def jump_to_last_bookmarked_line(editor):
	# FIXME: Documentation
	# If there are no bookmarks in the text editor, send feedback to the status
	# bar indicating that.
	if not editor.bookmark_list:

		from internationalization import msg0342
		update_status_info(editor, msg0342, "no")

		return

	line = get_last_bookmarked_line(editor)
	cursor_line = get_current_line(editor)

	if line == cursor_line:

		from internationalization import msg0341
		update_status_info(editor, msg0341, "no")

		return

	# Move cursor to the selected line in the dialog's treeview.
	iterator = editor.text_buffer.get_iter_at_line(int(line))
	editor.text_buffer.place_cursor(iterator)
	from cursor import move_view_to_cursor_position
	move_view_to_cursor_position(editor)
	from internationalization import msg0340
	update_status_info(editor, msg0340, "yes")

	return

