# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


"""
This module defines a class that creates an instance of a bookmark browser.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""


from gtk import Dialog


class ScribesBookmarkBrowser(Dialog):
	"""
	This class creates instances of a dialog that displays a window showing
	bookmarked lines in a text editor. It inherits from gtk.Dialog. See the
	GTK+ manual for more information on gtk.Dialog.

	"""


	def __init__(self, editor):
		"""
		Initialize and create the dialog's attributes and widgets.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		"""

		Dialog.__init__(self)
		self.__init_attributes(editor)
		self.__set_dialog_properties()
		self.__create_dialog_widgets()
		self.__arrange_dialog_widgets()
		self.connect("key-press-event", self.key_press_event_cb)
		self.connect("delete-event", self.delete_event_cb)
		self.connect("map-event", self.dialog_map_event_cb)

	def __init_attributes(self, editor):
		"""
		Initialize the dialog's attributes.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		@param editor: Reference to an instance of the text editor.
		@type editor: An Editor object.

		"""

		self.editor = editor
		from bookmark import get_bookmarked_lines, get_last_bookmarked_line
		self.bookmarked_lines = get_bookmarked_lines
		self.last_bookmarked_line = get_last_bookmarked_line
		self.bookmark_list = []
		self.__first_time_flag = True

		return


	def __set_dialog_properties(self):
		"""
		Customize the dialog properties.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		"""

		# Calculate resolution independent width and height of the preferences
		# dialog window.
		width, height = \
self.editor.calculate_resolution_independence(1.679790026, 1.865209472)
		from internationalization import msg0336
		self.set_property("title", msg0336)
		self.set_property("role", "bookmark_browser")
		from gtk.gdk import WINDOW_TYPE_HINT_DIALOG
		self.set_property("type-hint", WINDOW_TYPE_HINT_DIALOG)
		self.set_property("modal", False)
		self.set_property("width-request", width)
		self.set_property("height-request", height)
		self.set_property("skip-taskbar-hint", True)
		self.set_property("skip-pager-hint", True)
		from bookmark import create_bookmark_pixbuf
		self.set_property("icon", create_bookmark_pixbuf(self.editor))
		from gtk import WIN_POS_CENTER_ON_PARENT
		self.set_property("window-position", WIN_POS_CENTER_ON_PARENT)
		self.set_property("resizable", True)
		self.set_property("has-separator", False)
		from gtk import STOCK_CLOSE, RESPONSE_CLOSE
		self.add_button(STOCK_CLOSE, RESPONSE_CLOSE)
		self.set_transient_for(self.editor)
		self.vbox.set_property("spacing", 10)
		self.set_property("border-width", 10)

		return


	def __create_dialog_widgets(self):
		"""
		Create key dialog widgets.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		"""

		self.liststore = self.__create_liststore()
		self.treeview = self.__create_treeview()

		return


	def __arrange_dialog_widgets(self):
		"""
		Prepare the dialog for display by packing widgets appropriately.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		"""

		from scrollwindow import create_scroll_window
		scrollwin = create_scroll_window()
		scrollwin.add(self.treeview)
		self.vbox.pack_start(scrollwin, True, True, 0)

		return


	def show(self):
		"""
		Show the dialog.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		"""

		if self.bookmarked_lines(self.editor):

			self.__populate_list_store()
			self.show_all()
			response = self.run()

		else:

			response = None
			from feedback import update_status_info
			from internationalization import msg0339
			update_status_info(self.editor, msg0339, "no")

		if response:

			self.hide_dialog()

		return


	def hide_dialog(self):
		"""
		Hide the dialog.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		"""

		self.hide()

		return


################################################################################
#
#								Dialog  Callbacks
#
################################################################################


	def key_press_event_cb(self, dialog, event):
		u"""
		Handles callback when keys on the keyboard are pressed.

		This function watches for the Escape key event. When the Escape key
		event is detected, this function closes text editor's about dialog.

		@param dialog: The text editor's about dialog.
		@type dialog: A gtk.AboutDialog object.

		@param event: An event generated by pressing the keyboard.
		@type event: A gtk.Event object

		@return: If false, propagate the generated event. Otherwise, block the
				event.
		@rtype: A boolean value.

		"""

		from gtk import keysyms

		# Close the text editor's about dialog when the escape key is pressed.
		if event.keyval == keysyms.Escape:

			from gtk import RESPONSE_CLOSE
			dialog.response(RESPONSE_CLOSE)

		return False


	def delete_event_cb(self, dialog, event):
		u"""
		Handles callback when keys on the keyboard are pressed.

		This function watches for the Escape key event. When the Escape key
		event is detected, this function closes text editor's about dialog.

		@param dialog: The text editor's about dialog.
		@type dialog: A gtk.AboutDialog object.

		@param event: An event generated by pressing the keyboard.
		@type event: A gtk.Event object

		@return: If false, propagate the generated event. Otherwise, block the
				event.
		@rtype: A boolean value.

		"""

		# Hide the dialog.
		from gtk import RESPONSE_CLOSE
		dialog.response(RESPONSE_CLOSE)

		return True


	def dialog_map_event_cb(self, dialog, event):
		"""
		Handles callback when the dialog is displayed.

		This function focuses the dialog's treeview widget everytime the dialog
		is displayed. It also shows the sort indicator when the dialog is
		initially displayed.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		@param dialog: The bookmark browser dialog.
		@type dialog: A gtk.Dialog object.

		@param event: An event that maps widgets to a screen.
		@type event: A gtk.Event object.

		@return: True to prevent propagation of the map event to parent widgets.
		@rtype: A Boolean object.

		"""

		# Focus the dialog's treeview widget.
		self.treeview.grab_focus()

		return True


################################################################################
#
#							Dialog ListStore
#
################################################################################


	def __create_liststore(self):
		"""
		Create a liststore for the dialog and set its properties.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		@return: A liststore object for the dialog
		@rtype: A gtk.ListStore object.

		"""

		from gtk import ListStore
		liststore = ListStore(int, str, "gboolean")

		return liststore


################################################################################
#
#							Dialog TreeView
#
################################################################################


	def __create_treeview(self):
		"""
		Create a treeview for the dialog and set its properties.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		@return: A treeview for the dialog that contains the position of
			bookmarks in the text editor's buffer.
		@rtype: A gtk.TreeView object.

		"""

		from gtk import TreeView
		treeview = TreeView()

		from gtk import CellRendererText
		renderer = CellRendererText()
		renderer.set_property('cell-background', 'yellow')
		renderer.set_property("xalign", 1.0)
		from gtk import TreeViewColumn
		from internationalization import msg0337
		column = TreeViewColumn(msg0337, renderer, text=0)
		treeview.append_column(column)
		column.set_expand(False)
		from gtk import TREE_VIEW_COLUMN_AUTOSIZE, SORT_DESCENDING
		column.set_sizing(TREE_VIEW_COLUMN_AUTOSIZE)
		column.set_attributes(renderer, text=0, cell_background_set=2)
		column.set_sort_indicator(True)
		column.set_sort_order(SORT_DESCENDING)
		column.set_sort_column_id(0)

		renderer = CellRendererText()
		renderer.set_property('cell-background', 'yellow')
		from internationalization import msg0338
		column = TreeViewColumn(msg0338, renderer, text=1)
		treeview.append_column(column)
		column.set_expand(True)
		column.set_sizing(TREE_VIEW_COLUMN_AUTOSIZE)
		column.set_attributes(renderer, text=1, cell_background_set=2)
		column.set_sort_order(SORT_DESCENDING)
		column.set_sort_column_id(1)


		treeview.set_model(self.liststore)
		treeview.columns_autosize()
		treeview.set_enable_search(True)
		treeview.set_search_column(0)
		treeview.set_property("rules-hint", True)
		treeview.connect("row-activated", self.row_activated_cb)
		treeview.connect("map-event", self.treeview_map_event_cb)

		return treeview


	def row_activated_cb(self, treeview, path, column):
		"""
		Handles callback when a row in the treeview is activated.

		This function moves the cursor to the position in the buffer specified
		by the bookmark in the row activated.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		@param treeview: The treeview that contains bookmarked lines and their
			associated text.
		@type treeview: A gtk.TreeView object.

		@param path: The position of a row in treeview.
		@type path: A Path object.

		@param column: Columns in a treeview.
		@type column: A gtk.TreeViewColumn object.

		@return: Return True to prevent the propagation of signals to parent
			widgets.
		@rtype: A Boolean object.

		"""

		# Get the selected line in the dialog's treeview.
		selection = treeview.get_selection()
		model, iterator = selection.get_selected()
		line = model.get_value(iterator, 0)

		# Move cursor to the selected line in the dialog's treeview.
		iterator = self.editor.text_buffer.get_iter_at_line(int(line)-1)
		self.editor.text_buffer.place_cursor(iterator)
		from cursor import move_view_to_cursor_position
		move_view_to_cursor_position(self.editor)
		from gtk import RESPONSE_CLOSE
		self.response(RESPONSE_CLOSE)
		from internationalization import msg0171
		message = msg0171 % int(line)
		from feedback import update_status_info
		update_status_info(self.editor, message, "yes")

		return True


	def treeview_map_event_cb(self, treeview, event):
		"""
		Handles callback when the dialog is displayed.

		This function focuses the dialog's treeview widget everytime the dialog
		is displayed. It also shows the sort indicator when the dialog is
		initially displayed.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		@param treeview: The dialog's treeview widget.
		@type treeview: A gtk.TreeView object.

		@param event: An event that maps widgets to a screen.
		@type event: A gtk.Event object.

		@return: True to prevent propagation of the map event to parent widgets.
		@rtype: A Boolean object.

		"""

		# Focus the dialog's treeview widget.
		treeview.grab_focus()

		if self.__first_time_flag:

			# Show the sort indicator when the dialog is initially displayed.
			column = treeview.get_columns()
			column[0].clicked()
			self.__first_time_flag = False

		return True


################################################################################
#
#								Helper Methods
#
################################################################################


	def __get_line_and_text_at_bookmarks(self, bookmarks):
		"""
		Get the strings on a bookmarked line.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		@param bookmarks: A list of bookmarked lines
		@type bookmarks: A List object.

		@return: A list containing a tuple of bookmarked lines and strings.
		@rtype: A List object.

		"""

		bookmarks.sort()
		bookmarked_lines_and_text = []
		from utility import get_text_at_line

		for line in bookmarks:

			text = get_text_at_line(self.editor.text_buffer, line).strip(" \t")
			bookmarked_lines_and_text.append((line + 1, text))

		return bookmarked_lines_and_text


	def __populate_list_store(self):
		"""
		Populate the dialog's liststore with bookmarked lines.

		@param self: Reference to the ScribesBookmarkBrowser instance.
		@type self: A ScribesBookmarkBrowser object.

		"""

		bookmarks = self.bookmarked_lines(self.editor)
		bookmark_list = self.__get_line_and_text_at_bookmarks(list(bookmarks))

		if self.bookmark_list == bookmark_list:

			return

		self.bookmark_list = bookmark_list
		self.liststore.clear()

		# Get last bookmarked line in the text editor's buffer.
		line = self.last_bookmarked_line(self.editor) + 1

		# True to highlight a treeview row.
		value = False

		for item in bookmark_list:

			if item[0] in (line,):

				value = True

			self.liststore.append([item[0], item[1], value])
			value = False

		return

