# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
The module contains functions that define all bracket operations of the text
editor. They include finding matching brackets, removing matching brackets,
completing matching brackets to mention a few.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from pygtk import require
require("2.0")

try:

	from gtksourceview import source_iter_find_matching_bracket as match

except ImportError:

	from gtksourceview import gtk_source_iter_find_matching_bracket as match

from gtk import TEXT_SEARCH_TEXT_ONLY

from feedback import update_status_info
from internationalization import msg0020, msg0021, msg0154, msg0156, msg0207
from cursor import get_cursor_iter, get_current_line


def match_bracket(editor):
	u"""
	Find matching brackets

	@param editor: an editor instance
	@type editor: an editor object
	"""

	# Get cursor position
	cursor_iter = get_cursor_iter(editor)

	# Find matching bracket.
	if match(cursor_iter):

		cursor_iter.forward_char()
		editor.text_buffer.place_cursor(cursor_iter)
		current_line = get_current_line(editor)
		cursor_mark = editor.text_buffer.get_insert()
		editor.text_view.scroll_to_mark(cursor_mark, 0.3,
								use_align=False, xalign=0.5,
								yalign=0.5)
		update_status_info(editor, msg0020 % (current_line+1), u"yes")

	else:

		update_status_info(editor, msg0021, u"no")

	return


def update_bracket_dictionary(editor, character, marker_one, marker_two):
	u"""
	Update the editor's bracket dictionary with information about a bracket
	region.

	The information updated are the bracket type either ", [, ( or { and the
	position of the opening bracket.

	@param editor: A reference to the editor object.
	@type editor: An editor object.

	@param character: A bracket type.
	@type character: A string object.

	@param marker: A mark at the beginning of a bracket region.
	@type marker: A gtk.TextMark object.

	"""

	editor.bracket_dictionary[editor.bracket_count] = (character, marker_one,
													marker_two)

	return


def insert_completion(editor, character):
	u"""
	Inserts opposite bracket characters into the editor's buffer.

	@param editor: A reference to the editor object.
	@type editor: An editor object.

	@param character: ], ), }, or "
	@type character: A string.

	"""

	# Begin recording for undo operation.
	#editor.text_buffer.begin_user_action()

	# Insert character into the editor's buffer.
	editor.text_buffer.insert_at_cursor(character)

	# Move the cursor between the brackets.
	cursor_iter = get_cursor_iter(editor)
	cursor_iter.backward_char()
	editor.text_buffer.place_cursor(cursor_iter)

	# Place a mark before the opening bracket.
	marker_iter = cursor_iter.copy()
	marker_iter.backward_char()
	marker_one = editor.text_buffer.create_mark(None, marker_iter)

	# Place a mark after the closing bracket.
	marker_iter = cursor_iter.copy()
	marker_iter.forward_char()
	marker_two = editor.text_buffer.create_mark(None, marker_iter)

	# Set the region flag to keep track on the cursor within the bracket region
	editor.bracket_region_flag = True
	editor.bracket_count += 1
	update_bracket_dictionary(editor, character, marker_one, marker_two)

	# End recording for undo operation.
	# editor.text_buffer.end_user_action()

	return


def activate_bracket_completion(editor, value):
	u"""
	Provide completion for {, [, " and ( characters.

	@param editor: A reference to the editor object.
	@type editor: An editor object.

	@param value: An integer value representing either {, [, " or (.
	@type value: An integer.

	"""

	if value == 123:

		insert_completion(editor, "{}")

	elif value == 91:

		insert_completion(editor, "[]")

	elif value == 34:

		insert_completion(editor, '""')

	elif value == 60:

		insert_completion(editor, "<>")

	elif value == ord("»".decode("utf-8")):

		insert_completion(editor, "»«")

	elif value == ord("«".decode("utf-8")):

		insert_completion(editor, "«»")

	elif value == ord("「".decode("utf-8")):

		insert_completion(editor, "「」")

	else:

		insert_completion(editor, "()")

	# If the flag is true, when a backspace character is entered in the text
	# editor's buffer, matching brackets are removed.
	editor.backspace_flag = True

	# Feedback to the statusbar indicating a bracket completion operation
	# occurred.
	update_status_info(editor, msg0154, "yes")

	return


def remove_matching_brackets(editor):
	u"""
	Remove matching brackets in the text editor's buffer.

	Sometimes users accidentally type brackets into the text editor's buffer.
	On such occassions pressing the backspace key activates this function to
	remove the matching brackets.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Begin recording for undo operation.

	#editor.text_buffer.begin_user_action()

	# Move iterators before and after matching brackets.

	cursor_iter = get_cursor_iter(editor)
	move_iter = cursor_iter.copy()
	move_iter.forward_char()
	cursor_iter.backward_char()

	# Remove matching brackets.

	editor.text_buffer.delete(cursor_iter, move_iter)

	# End recording for undo operation.

	#editor.text_buffer.end_user_action()

	# Feedback to the statusbar indicating matching brackets have been removed.

	update_status_info(editor, msg0156, "yes")

	return


def update_bracket_region_info(editor):
	u"""
	Update the text editor's bracket dictionary.

	The function is called to delete information about a bracket region that
	no longer exists.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Delete markers assiociated with a particular bracket region.

	marker = editor.bracket_dictionary[editor.bracket_count][1]
	editor.text_buffer.delete_mark(marker)
	marker = editor.bracket_dictionary[editor.bracket_count][2]
	editor.text_buffer.delete_mark(marker)

	# Delete the entry in the text editor's bracket dictionary associated with
	# the bracket region in question.

	del editor.bracket_dictionary[editor.bracket_count]
	editor.bracket_count -= 1

	# Reset bracket flags when the bracket region count is zero.

	if not editor.bracket_count:

		editor.bracket_region_flag = False

	return


def search_for_bracket_region_positions(editor):
	u"""
	Determine the range to search for opening and closing brackets.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Get the beginning search point.

	marker_one = editor.bracket_dictionary[editor.bracket_count][1]
	start_iter = editor.text_buffer.get_iter_at_mark(marker_one)
	marker_two = editor.bracket_dictionary[editor.bracket_count][2]
	end_iter = editor.text_buffer.get_iter_at_mark(marker_two)
	end_iter.forward_char()

	return start_iter, end_iter


def check_bracket_region(editor):
	u"""
	Check if the text editor's cursor is within a bracket region.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	start_iter, end_iter = search_for_bracket_region_positions(editor)

	# Get the type of brackets to search for.

	opening_bracket = editor.bracket_dictionary[editor.bracket_count][0][0]
	closing_bracket = editor.bracket_dictionary[editor.bracket_count][0][1]

	# Search for the beginning brackets.

	cursor_iter = get_cursor_iter(editor)
	result = cursor_iter.backward_search(opening_bracket,
										TEXT_SEARCH_TEXT_ONLY,
										start_iter)

	# If beginning bracket is found, search for closing brackets. If neither
	# beginning nor closing brackets are found, the cursor is not within a
	# bracket region. Thus update the editor's bracket region information.

	if result:

		result = cursor_iter.forward_search(closing_bracket,
										TEXT_SEARCH_TEXT_ONLY,
										end_iter)

		if not result:

			update_bracket_region_info(editor)

	else:

		update_bracket_region_info(editor)

	return


def move_cursor_out_of_bracket_end(editor):
	u"""
	Move the text editor's cursor out of a bracket region to the right.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Search for the closing bracket and move the text editor's cursor after
	# it.

	end = search_for_bracket_region_positions(editor)
	cursor_iter = get_cursor_iter(editor)
	closing_bracket = editor.bracket_dictionary[editor.bracket_count][0][1]
	result = cursor_iter.forward_search(closing_bracket,
										TEXT_SEARCH_TEXT_ONLY,
										end[1])

	if result:

		editor.text_buffer.place_cursor(result[1])

	return


def move_cursor_out_of_bracket_begin(editor):
	u"""
	Move the text editor's cursor out of a bracket region to the left.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	begin = search_for_bracket_region_positions(editor)
	cursor_iter = get_cursor_iter(editor)
	opening_bracket = editor.bracket_dictionary[editor.bracket_count][0][0]
	result = cursor_iter.backward_search(opening_bracket, TEXT_SEARCH_TEXT_ONLY,
										begin[0])

	if result:

		editor.text_buffer.place_cursor(result[0])

	return


def enclose_selected_text(editor, opening_character, closing_character):
	u"""
	Enclose selected text in the buffer in quotes, braces, brackets or
	parenthesis.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	@param opening_character: The opening quote, brace, bracket or parenthesis.
	@type opening_character: A string object.

	@param closing_character: The closing quote, brace, bracket or parenthesis.
	@type closing_character: A string object.

	"""

	# Begin recording for undo operation.

	# editor.text_buffer.begin_user_action()

	# Get selection from the text editor's buffer.

	selection = editor.text_buffer.get_selection_bounds()
	marker = editor.text_buffer.create_mark(None, selection[1])

	# Insert the opening character at the beginning of the selection boundary.

	editor.text_buffer.insert(selection[0], opening_character)
	iterator = editor.text_buffer.get_iter_at_mark(marker)

	# Insert the closing character at the end of the selection boundary.

	editor.text_buffer.insert(iterator, closing_character)
	iterator = editor.text_buffer.get_iter_at_mark(marker)

	# Move the cursor after the closing character.

	editor.text_buffer.place_cursor(iterator)
	editor.text_buffer.delete_mark(marker)

	# End recording for undo operation.

	#editor.text_buffer.end_user_action()

	# Feedback to the statusbar indicating the selected has been enclosed.

	update_status_info(editor, msg0207, "yes")

	return


def enclose_selection(editor, keyval):
	u"""
	Enclose selected text in brackets, parenthesis or quotes.

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	@param keyval: A keyboard code representing a key character.
	@type keyval: An integer object.

	"""

	if keyval == 91:

		opening_character = "["
		closing_character = "]"
		enclose_selected_text(editor, opening_character, closing_character)

	elif keyval == 123:

		opening_character = "{"
		closing_character = "}"
		enclose_selected_text(editor, opening_character, closing_character)

	elif keyval == 40:

		opening_character = "("
		closing_character = ")"
		enclose_selected_text(editor, opening_character, closing_character)

	elif keyval == 34:

		opening_character = closing_character = '"'
		enclose_selected_text(editor, opening_character, closing_character)

	elif keyval == 39:

		opening_character = closing_character = "'"
		enclose_selected_text(editor, opening_character, closing_character)

	elif keyval == ord("»".decode("utf-8")):

		opening_character = "»"
		closing_character = "«"
		enclose_selected_text(editor, opening_character, closing_character)

	elif keyval == ord("«".decode("utf-8")):

		opening_character = "«"
		closing_character = "»"
		enclose_selected_text(editor, opening_character, closing_character)

	elif keyval == ord("「".decode("utf-8")):

		opening_character = "「"
		closing_character = "」"
		enclose_selected_text(editor, opening_character, closing_character)

	else:

		opening_character = "<"
		closing_character = ">"
		enclose_selected_text(editor, opening_character, closing_character)

	return

