# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""

This module contains the function necessary for case operations and
manipulations. Any operations that involves changing cases from upper to lower
or mixing cases should be found here.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from pygtk import require
require("2.0")

from feedback import update_status_info
from internationalization import msg0015, msg0016, msg0017, msg0018, msg0019
from internationalization import msg0149, msg0150, msg0151
from error import error_dialog
from autocomplete import block_word_completion, unblock_word_completion


def case(editor, option="upper"):
	"""
	Change a selected text from lowercase to uppercase, uppercase to lowercase,
	titled case or mixed case depending on the argument option

	@param editor: an editor instance
	@type editor: an editor object

	@param option: The case to change selected text to. It may either be
				"upper", "lower", "swap" or "title"
	@type option: a string object

	"""

	block_word_completion(editor)

	# Begin recording for undo operation.
	# editor.text_buffer.begin_user_action()

	# Get selection boundaries.
	selection = editor.text_buffer.get_selection_bounds()

	# If there is no selection, send feedback to the statusbar indicating the
	# problem.
	if not selection:

		# editor.text_buffer.end_user_action()
		unblock_word_completion(editor)
		update_status_info(editor, msg0015, u"no")

		return

	# Get offsets at boundaries
	begin_offset = selection[0].get_offset()
	end_offset = selection[1].get_offset()

	# Get selected strings from the editor's buffer
	string = editor.text_buffer.get_text(selection[0], selection[1])

	# Convert the string to UTF-8 unicode
	try:

		text = unicode(string, "utf8")

	except UnicodeError:

		unblock_word_completion(editor)
		error_dialog(editor, u"Unicode error, see case.py")

		return

	if option == "upper":

		check_string = text.isupper
		change_string = text.upper
		message = msg0017
		error_message = msg0016

	elif option == "lower":

		check_string = text.islower
		change_string = text.lower
		message = msg0019
		error_message = msg0018

	elif option == "title":

		check_string = text.istitle
		change_string = text.title
		message = msg0150
		error_message = msg0151

	else:

		change_string = text.swapcase
		message = msg0149

	# Check if the text is already in the case the user wants to change to. If
	# it is, send a message to the user indicating the case.
	try:

		if check_string():

			unblock_word_completion(editor)
			# editor.text_buffer.end_user_action()
			update_status_info(editor, error_message, u"no")

			return

	except UnboundLocalError:

		pass

	# Delete the selected string.
	editor.text_buffer.delete(selection[0], selection[1])

	# Convert the selected text to case in question and insert it into the
	# editor's buffer.
	new_iter = editor.text_buffer.get_iter_at_offset(begin_offset)
	editor.text_buffer.insert(new_iter, change_string())

	# Select the converted text again.
	begin_iter = editor.text_buffer.get_iter_at_offset(begin_offset)
	end_iter = editor.text_buffer.get_iter_at_offset(end_offset)
	editor.text_buffer.select_range(begin_iter, end_iter)

	# End recording for undo operation.
	# editor.text_buffer.end_user_action()
	unblock_word_completion(editor)

	# Send feedback to the statusbar indicating the status of the operation.
	update_status_info(editor, message, u"yes")

	return

