# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


u"""
Main window and graphic user interface for the editor.

This modules contains code responsible for creating the editor's main window.
The editor's main window consists of a toolbar object containing 11 tool items,
a scroll window containing a textview and a status area comprising of three
statusbars.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from re import compile as recompile, UNICODE
from os import environ
from locale import getdefaultlocale, getpreferredencoding

from pygtk import require
require("2.0")
from gobject import timeout_add, source_remove, GError, PRIORITY_LOW
from gtk import Window, keysyms, HBox
from gtk import window_set_auto_startup_notification
from gtk import STOCK_LEAVE_FULLSCREEN, Button, WINDOW_TOPLEVEL
from gtk.gdk import WINDOW_TYPE_HINT_NORMAL, WINDOW_STATE_MAXIMIZED
from gtk.gdk import KEY_PRESS_MASK, CONTROL_MASK, MOD1_MASK
from gnome import init, PARAM_APP_DATADIR, PARAM_ENABLE_SOUND

from gnome.ui import libgnomeui_module_info_get
from gtkspell import Spell

from timeout import fullscreen_timeout
from timeout import garbage, hide_cursor_timeout
from feedback import update_status_info
from utility import find_app_data_dir, create_button
from scribes_gconf import scribes_init_gconf, get_gconf_client
from accelerators import create_accel
from actions import create_actions
from maincontainer import create_main_container
from toolbar import create_toolbar
from scrollwindow import create_scroll_window
from textbuffer import create_text_buffer
from textview import create_text_view
from statusbar import create_statusbar
from cursor import cursor_position
from gotobar import hide_gotobar
from findbar import hide_findbar
from replacebar import hide_replacebar
from internationalization import msg0022, msg0023, msg0024, msg0025
from info import version
from templates import generate_generic_templates
from tooltips import create_tooltips


################################################################################
#
#							Editor Window
#
################################################################################


class Editor(Window):
	u"""
	Main class and graphic user interface properties of the editor.

	The Editor class defines the main graphic user interface of the application.
	The interface consists of a toolbar, a scroll window containing a textview
	and a status bar.

	"""

	HOMEDIR = environ[u"HOME"]
	DESKTOPDIR = HOMEDIR + u"/Desktop"
	SCRIBESINFODIR = HOMEDIR + u"/.gnome2/scribes"
	# Regular expression pattern used to match characters before the cursor
	# position.
	pattern = recompile(r"[^-\w]", UNICODE)
	preferredencoding = getpreferredencoding(True)


	def __init__(self):
		u"""Initialize and create Editor's main graphic user interface."""

		self.templates = {}
		self.template_mode_count = 0
		self.template_mode_flag = False
		self.template_list = []
		self.language = None
		self.word_completion_dictionary = {}

		self.indexing = False
		# Initialize the default encoding used decode and encode text files.
		self.encoding = u"UTF-8"

		# Default name for all unsaved documents. See internationalization.py for
		# value of msg0025.
		self.filename = msg0025

		# List containing bookmarks in the text editor's buffer.
		self.bookmark_list = []


		self.bracket_dictionary = {}

		self.bracket_count = 0

		self.search_range = 0

		self.filechooser_encoding = None
		self.initialize_flags()

		# Activate start up notification
		window_set_auto_startup_notification(True)

		# Initialize the GNOME libraries.
		props = {PARAM_APP_DATADIR: find_app_data_dir(),
				PARAM_ENABLE_SOUND: True}
		init(u"scribes", version, properties=props)

		# Initialize the gconf monitoring deamons and get user configuration
		# values
		scribes_init_gconf(self)


		# Create editor's window.
		Window.__init__(self, WINDOW_TOPLEVEL)
		self.connect(u"delete-event", self.close_window_cb)
		self.connect(u"key-press-event", self.key_press_event_cb)
		self.connect(u"window-state-event", self.window_state_event_cb)
		self.connect("map-event", self.map_event_cb)
		self.add_events(KEY_PRESS_MASK)

		# Set widget properties
		self.set_name(u"scribes_window")

		# Set container properties
		self.set_border_width(1)

		# msg0025, "Unsaved Document," is a string eligible for
		# internationalization and localization. See internationalization.py for
		# more details
		self.set_title(msg0025)
		self.set_role(u"main_window")
		self.set_type_hint(WINDOW_TYPE_HINT_NORMAL)
		self.set_icon_name(u"text-editor")

		# Set the width and height of editor's window to one that is resolution
		# independent.
		width, height = self.calculate_resolution_independence(1.462857143,
															1.536)
		self.set_default_size(width, height)

		# Create accelerators for the editor
		self.add_accel_group(create_accel())

		# Create and initialize editor actions
		self.actiongroup = create_actions(self)

		# Create editor's main container
		self.main_container = create_main_container()
		self.add(self.main_container)

		self.tips = create_tooltips()
		# Toolbar area
		toolbar_area = HBox(homogeneous=False, spacing=0)

		# Create editor's toolbar
		toolbar = create_toolbar(self)
		toolbar_area.pack_start(toolbar, True, True, 0)
		self.main_container.pack_start(toolbar_area, False, False, 0)
		self.toolbar = toolbar
		toolbar.connect(u"enter-notify-event", self.enter_notify_event_cb)
		self.toolbar_area = toolbar_area

		# Create editor's scroll window
		scroll_window = create_scroll_window()
		self.main_container.pack_start(scroll_window, True, True, 0)
		self.scroll_window = scroll_window

		# Create editor's text buffer
		self.text_buffer = create_text_buffer(self)

		# Create editor's text view
		text_view = create_text_view(self)
		scroll_window.add(text_view)
		text_view.grab_focus()

		# Create editor's status area container
		status_area = HBox(homogeneous=False, spacing=0)

		#self.main_container.pack_start(status_area, False, False, 0)
		self.main_container.pack_start(status_area, False, False, 0)
		self.status_area = status_area

		# Create editor's statusbar
		status_structure = create_statusbar()
		status_container = status_structure[0]
		status_area.pack_start(status_container, expand=True, fill=True,
								padding=0)
		self.message_id = u""
		self.status_one = status_structure[1]
		self.status_one_id = self.status_one.get_context_id(u"id-1")
		self.status_one.push(self.status_one_id, self.message_id)
		self.status_two = status_structure[2]
		self.status_two_id = self.status_two.get_context_id(u"id-2")
		self.status_two.push(self.status_two_id, u"")
		self.status_three = status_structure[3]

		# Insert mode is the default editor mode. Let the status bar reflect
		# this mode by default.
		default_mode = u"INS"
		self.status_three_id = self.status_three.get_context_id(default_mode)
		self.status_three.push(self.status_three_id, default_mode.ljust(0))

		# Activate the statusbar's cursor position label
		cursor_position(self)

		# Call the garbage collector every 30 minutes.
		garbage_id = timeout_add(1800000, garbage, self)

		# Hide the cursor after 5 seconds of cursor inactivity
		cursor_id = timeout_add(5000, hide_cursor_timeout, self)
		template_id = timeout_add(2000, self.template_timeout)

		# Spell Checking
		if self.spell_check:

			try:

				self.spell_checker = Spell(text_view, getdefaultlocale()[0])

			except GError:

				pass


	def map_event_cb(self, window, event):

		if self.first_time_flag:

			timeout_add(7000, self.update_word_completion_dictionary,
						priority=PRIORITY_LOW)
			self.first_time_flag = False

		return True


	def update_word_completion_dictionary(self):
		"""
		Periodically index words found in the text editor's buffer that are
		eligible for word completion.

		This function is a callback to allow priority GObject timeout_add
		function. It is called periodically, approximately every 7 seconds, to
		scan the text buffer for words eligible for word completion. Eligible
		words are indexed in a dictionary. The dictionary is a key:value pair.
		Keys are represented by eligible words, while pairs are denoted by
		integers representing the number of occurence of eligible words in the
		text editor's buffer. The key:value pair of the dictionary format is
		represented as follows:

			word: number_of_words

		For the purpose of this project, a word is a sequence of alphanumeric
		characters that may, optionally, contain underscore ("_") or dash ("-")
		characters. A word is eligible for word completion when it is more than
		3 characters long.

		@param self: Reference to the Editor instance.
		@type self: An Editor object.

		@return: Returns True to ensure that this function is called again,
			approximately 7 seconds later.
		@rtype: A Boolean object.

		"""

		if self.indexing or self.dirty is False or self.readonly \
			or self.is_indexed or self.loading_flag:

			# Indexing does not occur if any of these conditions are true.
			#
			# 1). The text editor is in the process of indexing a document.
			# 2). The text editor is in readonly mode.
			# 3). The text editor has just been indexed and its buffer has not
			# been modified.
			# 4). The text editor is in the process of loading a document.

			return True

		# Ensure the text editor's graphic user interface does not lock during
		# indexing operations, especially, when indexing very large documents.
		from utility import response
		response()

		# True to denote the commencement of the indexing operation. This
		# prevents this function from being called when an indexing operation is
		# already in progress.
		self.indexing = True

		# Retrieve all the contents of the text editor's buffer.
		begin, end = self.text_buffer.get_bounds()
		text = self.text_buffer.get_text(begin, end)

		if text:

			# Search the buffer for words that are more than 3 characters in
			# length, count the number of each word found, and return a
			# dictionary with each word as a key, and their occurence as value
			# pairs.
			from utility import split_string
			response()
			result = split_string(self.pattern, 3, text)

		else:

			# If the text editor's buffer is empty reset the word completion
			# dictionary.
			self.word_completion_dictionary  = {}
			result = None

		if result:

			response()
			self.word_completion_dictionary = result

		# False means the indexing operation is complete and thus the function
		# can perform indexing operations in the future if need be.
		self.indexing = False

		# True if the editor's word completion dictionary has recently been indexed
		# and does not currently need to be indexed.
		self.is_indexed = True

		return True


	def test(self):

		from utility import response
		response()
		from thread import start_new_thread
		start_new_thread(self.update_word_completion_dictionary, ())

		return True


	def calculate_resolution_independence(self, width, height):
		u"""
		Calculate the width and height of the text editor's window based on the
		monitor's resolution.

		@param self: A reference to the editor's instance.
		@type self: An editor object.

		@return: The width and height to set the text editor's window.
		@rtype: A tuple object

		"""

		screen = self.get_screen()
		width = int(screen.get_width()/width)
		height = int(screen.get_height()/height)

		return width, height


	def activate_fullscreen(self, state=True):
		u"""
		Activate or deactivate fullscreen mode depending on the value of state.

		@param self: Reference to the editor instance.
		@type self: An editor object.

		@param state: True to activate fullscreen, false to deactivate it.
		@type state: Boolean object.

		@note: See flags.py for more information on fullscreen_flag and
			toolbar_flag.
		@note: See internationalization.py for more information on msg0024

		"""

		if state:

			self.fullscreen_flag = True

			# Create the leave fullscreen button.

			button = create_button(STOCK_LEAVE_FULLSCREEN, msg0024)
			button.connect(u"enter-notify-event", self.enter_notify_event_cb)
			self.button = Button()
			self.button.connect(u"clicked", self.unfullscreen_cb)
			self.button.add(button)

			# Add the button to the toolbar area and hide it after 5 seconds of
			# mouse inactivity.

			self.toolbar_area.pack_start(self.button, False, False, 0)
			self.toolbar_area.show_all()
			self.fullscreen()
			self.ft_id = timeout_add(5000, fullscreen_timeout, self)
			self.status_area.hide_all()

		else:

			# Show the text editor's toolbar.

			self.fullscreen_flag = False
			self.toolbar_flag = True
			source_remove(self.ft_id)
			self.button.hide_all()
			self.button.destroy()
			self.unfullscreen()
			self.toolbar_area.show_all()
			self.status_area.show_all()

		return


	def key_press_event_cb(self, entry, key_press):
		u"""
		Handles callback when key strokes are detected when the editor is
		focused.

		The "key-press-event" signal is emitted when the user presses a key on
		the	keyboard.

		@param key_press: A key press event.
		@type key_press: GTK Signal

		@param self: editor
		@type self: editor object

		"""

		# In template mode shift - tab is used to navigate the previous variable
		# or placeholder in the template.
		if key_press.keyval == keysyms.ISO_Left_Tab and self.template_mode_flag:

			self.template_list[-1].jump_to_previous_variable()

			return True

		# In template mode control - tab inserts the Tab character into the
		# text editor's buffer. The tab key is used for navigation in
		# template mode.
		if key_press.keyval == keysyms.Tab and self.template_mode_flag:

			if key_press.state & CONTROL_MASK:

				self.text_buffer.insert_at_cursor(u"\t")

				return True

		# Listen for F1 key press to launch the Scribes manual
		if key_press.keyval == keysyms.F1:

			self.actiongroup.get_action(u"help").activate()

		# Listen for F12 key press to launch the preferences dialog
		if key_press.keyval == keysyms.F12:

			if key_press.state & CONTROL_MASK:

				self.actiongroup.get_action(u"show_syntax_editor").activate()

			elif key_press.state & MOD1_MASK:

				self.actiongroup.get_action(u"show_template_editor").activate()

			else:

				self.actiongroup.get_action(u"preferences").activate()

		# Listen for F11 to enter fullscreen mode
		if key_press.keyval == keysyms.F11:

			if not self.fullscreen_flag:

				self.activate_fullscreen()

			else:

				self.activate_fullscreen(False)

		# Hide and destroy the gotobar when ESC is pressed
		if key_press.keyval == keysyms.Escape and self.gotobar_flag:

			hide_gotobar(self)

		# Hide and destroy the findbar when ESC is pressed
		if key_press.keyval == keysyms.Escape and self.findbar_flag:

			hide_findbar(self)

		# Hide and destroy the replacebar when ESC is pressed
		if key_press.keyval == keysyms.Escape and self.replacebar_flag:

			hide_replacebar(self)

		# Toggle readonly mode.
		if key_press.keyval == keysyms.F3:

			self.actiongroup.get_action(u"toggle_readonly").activate()

		# Spell Checking
		if key_press.keyval == keysyms.F6:

			client = get_gconf_client()

			# We use this flag to fool and prevent the editor from switching to
			# edit mode when the user types into the buffer.
			self.mode_edit_flag = True

			if client.get_bool(u"/apps/scribes/spell_check"):

				client.set_bool(u"/apps/scribes/spell_check", False)

				# msg0022 is a string eligible for internationalization and
				# localization. See internationalization.py for me information.
				# Send feedback to user via statusbar
				update_status_info(self, msg0022, u"spell")

			else:

				client.set_bool(u"/apps/scribes/spell_check", True)

				# msg0023 is a string eligible for internationalization and
				# localization. See internationalization.py for me information.
				# Send feedback to user via statusbar
				update_status_info(self, msg0023, u"spell")

		return False


################################################################################
#
#						Fullscreen Callback
#
################################################################################


	def unfullscreen_cb(self, widget):

		self.activate_fullscreen(False)

		return


	def enter_notify_event_cb(self, widget, event):


		if self.fullscreen_flag:

			source_remove(self.ft_id)
			self.toolbar_area.show_all()
			self.toolbar_flag = True

		return False


################################################################################
#
#						Close Window Callback
#
################################################################################


	def close_window_cb(self, widget, event):
		u"""
		Delete the editor's window.

		@param widget: the widget that received the signal
		@type widget: GTK Window
		@param event: the event that triggered the signal
		@type event: GTK Event
		@param data: the optional first user parameter specified with the
			connect method
		@type data: None

		@return: Returns False to propagate the signal to all widgets
		@rtype: boolean
		"""

		self.actiongroup.get_action(u"close_file").activate()

		return True


################################################################################
#
#						Window State Callback
#
################################################################################


	def window_state_event_cb(self, widget, event):

		if event.new_window_state == WINDOW_STATE_MAXIMIZED:

			self.maximize_flag = True

		else:

			self.maximize_flag = False

		return True


	def initialize_flags(self):
		u"""
		Contains flags that indicate the global state of elements within the
		editor.

		Flags indicate the particular state of an element within the editor.
		Generally, flags default to the false boolean value. This value
		indicates that the state of an element within the editor is inactive or
		off.

		The purpose of flags is to synchronize events or actions, or to
		determine whether or not an action should or can be performed. For
		example, when the editor's readonly flag is true, the text editor's
		buffer should not be editable. Consequently, when the editor is
		performing a saving operation the garbage collector should not be run.
		The editor's saving flag is used to determine whether or not a saving
		operation is occurring.

		"""

		# The flag is true when the file in the text editor's buffer has been
		# saved.
		self.save_flag = True

		# The flag is true when the gotobar is visible.
		self.gotobar_flag = False

		# The flag is true when the findbar is visible.
		self.findbar_flag = False

		# The flag is true when the replacebar is visible.
		self.replacebar_flag = False

		# FIXME: The loading_flag is used to determine whether or not the
		# program should load a file into the current buffer or should load the
		# file in a new instance of the editor.
		self.loading_flag = False

		# The flag is true when the file loaded into the text editor's buffer is
		# in readonly mode.
		self.readonly = False

		self.is_indexed = True
		# The flag is true if the file is a new file.
		self.isNewFile = True

		# The flag is true if the buffer has been modified at least once.
		self.dirty = False

		# The flag is true when an icon exist in the text editor's status bar
		# area.
		self.mode_flag = False

		# The flag is true when a file is being edited.
		self.mode_edit_flag = False

		# The flag is true when the progress bar that appears in the text
		# editor's statusbar area is active.
		self.progress_flag = False

		# The flag is true when the text editor is in the process of saving a
		# file.
		self.saving_flag = False

		# The flag is true when the text editor is in fullscreen mode.
		self.fullscreen_flag = False

		# The flag is true when the text editor's toolbar is visible.
		self.toolbar_flag = True

		# FIXME:
		self.status_busy_flag = False

		# The flag is true if the word completion window list is visible.
		self.completion_flag = False

		# The flag is true if a bookmark image has already been created.
		self.bookmark_image_flag = False

		# The flag is true if the text editor's buffer has a bookmark.
		self.bookmark_flag = False

		self.created_flag = False

		self.bracket_region_flag = False

		self.backspace_flag = False

		self.first_time_flag = True

		return


	def template_timeout(self):
		u"""
		Handles timeout callback to generate generic templates.

		@param editor: Reference to an editor instance.
		@type editor: A Editor object.

		"""

		generate_generic_templates(self)

		return False

