# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


u"""
This modules defines the file selector dialog for Scribes.

The file selector dialog is used to select and arbitrary number of files for
loading by the editor.
"""


from collections import deque
from os import path, environ
from gc import collect
from locale import getpreferredencoding

from pygtk import require
require(u"2.0")
from gtk import FileFilter, FileChooserDialog, WIN_POS_CENTER_ON_PARENT
from gtk import STOCK_CANCEL, STOCK_OPEN, RESPONSE_CANCEL, RESPONSE_OK
from gtk import Alignment, Expander
from gtk.gdk import WINDOW_TYPE_HINT_DIALOG, GRAVITY_CENTER

from fileloader import FileLoader
from internationalization import msg0025, msg0026, msg0027, msg0028, msg0029
from internationalization import msg0173, msg0188, msg0187
from feedback import update_status_info

from encoding import create_encoding_box


def create_filter(name=u"", mime=u"", pattern=u""):
	u"""
	Create a filter for the text editor's fileselector dialog.

	The filter is used to filter text files based on file type and source code.

	@param name: A human readable name for the text editor's fileselector
				filter.
	@type name: A string.

	@param mime: The mime type to filter.
	@type mime: A string.

	@param pattern: A regular expression pattern.
	@type pattern: A string.

	@return: A filter.
	@rtype: A gtk.FileFilter object.

	"""

	filefilter = FileFilter()
	filefilter.set_name(name)
	filefilter.add_mime_type(mime)

	if pattern:

		filefilter.add_pattern(pattern)

	return filefilter


def determine_open_folder(editor):
	u"""
	Find the last visited folder.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	@return: Path to the last visited folder if available.
	@rtype: A string.

	"""

	if editor.filename != msg0025:

		file_path = path.expanduser(editor.filename)
		folder_path = path.dirname(file_path)

		return folder_path

	elif path.exists(environ[u'HOME'] + u"/Desktop"):

		folder_path = environ[u'HOME'] + u"/Desktop"

		return folder_path

	else:

		folder_path = environ[u'HOME']

		return folder_path


def show_file_selector_dialog(editor):
	u"""
	Show the file selector dialog

	The file selector dialog allows users to select an arbritrary number of
	files to be loaded into the editor.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	width, height = editor.calculate_resolution_independence(1.6, 1.929648241)
	update_status_info(editor, msg0188, u"open", False)

	# Create Filters
	txt_filter = create_filter(msg0029, "text/plain")
	py_filter = create_filter(msg0028, "text/x-python")
	no_filter = create_filter(msg0027, "", "*")

	# Create file selector dialog
	dialog = FileChooserDialog()

	# Folder to show
	folder_path = determine_open_folder(editor)

	# Set window properties
	dialog.set_title(msg0026)
	dialog.set_role("open_document_dialog")
	dialog.activate_default()
	dialog.set_transient_for(editor)
	dialog.set_type_hint(WINDOW_TYPE_HINT_DIALOG)
	dialog.set_modal(True)
	dialog.set_default_size(width, height)
	dialog.set_skip_taskbar_hint(True)
	dialog.set_icon_name(u"stock_open")
	dialog.set_position(WIN_POS_CENTER_ON_PARENT)
	dialog.set_gravity(GRAVITY_CENTER)

	if editor.filename != msg0025:

		dialog.select_filename(editor.filename)

	# Set dialog properties
	dialog.add_button(STOCK_CANCEL, RESPONSE_CANCEL)
	dialog.add_button(STOCK_OPEN, RESPONSE_OK)
	dialog.set_default_response(RESPONSE_OK)

	# Set file chooser properties
	dialog.set_select_multiple(True)
	dialog.set_current_folder(folder_path)
	dialog.add_filter(txt_filter)
	dialog.add_filter(py_filter)
	dialog.add_filter(no_filter)

	# Create an Expander
	expander = Expander()
	dialog.vbox.pack_start(expander, False, True, 0)
	expander.set_label(msg0173)
	expander.set_use_underline(True)

	# Container that holds encoding selection combobox and label
	encoding_box = create_encoding_box(editor)
	alignment = Alignment()
	alignment.add(encoding_box)
	alignment.set(0.3, 0.0, 0.7, 0.0)
	expander.add(alignment)
	encoding_box.show_all()
	dialog.show_all()
	response = dialog.run()

	if response == RESPONSE_OK:

		file_list = dialog.get_uris()
		file_list = deque(file_list)
		dialog.hide_all()
		dialog.destroy()
		from main import load_uris
		load_uris(editor, file_list)

		if not editor.loading_flag:

			update_status_info(editor, msg0187, u"info")

	else:

		dialog.hide_all()
		dialog.destroy()
		update_status_info(editor, msg0187, u"info")
		collect()

	return

