# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Gotobar for the text editor.

The gotobar is a widget that appears just above the status bar that allows a
user move the cursor to a specific line.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from gc import collect

from pygtk import require
require("2.0")
from gtk import Label, Adjustment, HBox, SpinButton
from gtk import UPDATE_IF_VALID, keysyms, Image, STOCK_JUMP_TO, ICON_SIZE_BUTTON

from internationalization import msg0055, msg0056, msg0180, msg0181, msg0182
from feedback import update_status_info
from cursor import get_current_line


def value_changed_cb(spin_button, editor):
	u"""
	Handles callback when the entry in the spinbutton object changes.

	@param spin_button: Reference to the spinbutton object on the gotobar.
	@type spin_button: A gtk.SpinButton object

	@param editor: Reference to an editor instance
	@type editor: An editor object

	"""

	# Get the line number.

	line = spin_button.get_value() - 1
	message = msg0181 % (line + 1)
	update_status_info(editor, message, "yes", False)

	# Adjust the cursor and text view as per changes made to the spinbutton.

	line_iter = editor.text_buffer.get_iter_at_line(int(line))
	editor.text_buffer.place_cursor(line_iter)
	cursor_mark = editor.text_buffer.get_insert()
	editor.text_view.scroll_to_mark(cursor_mark, 0.3,
								use_align=False, xalign=0.5,
								yalign=0.5)

	return


def key_press_event_cb(spin_button, key_press, editor):
	u"""
	Handles callback when the return key is hit.

	Hitting the return or enter key has the side effect of hiding the gotobar.

	@param spin_button: Reference to the spinbutton object on the gotobar.
	@type spin_button: A gtk.SpinButton object

	@param key_press: Any keypress event
	@type key_press: A gtk.gdk.Event object

	@param editor: Reference to an editor instance
	@type editor: An editor object

	"""

	# Hides the gotobar when the return key is pressed

	if key_press.keyval == keysyms.Return:

		hide_gotobar(editor)

	return


def create_gotobar(editor):
	u"""
	Create the editor's gotobar.

	The gotobar bar is widget that appears just above the statusbar. It contains
	a spinbutton to move the cursor to a specific line.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	@return: A gotobar containing labels and spinbutton entry widget.
	@rtype: A gtk.Hbox object.

	@note: See internationalization.py for more information on msg0055 and
		msg0056.

	"""

	gotobar = HBox(False, 0)

	# Add the GTK JUMP TO stock icon to the gotobar widget

	image = Image()
	image.set_from_stock(STOCK_JUMP_TO, ICON_SIZE_BUTTON)
	gotobar.pack_start(image, False, False, 3)

	# Add a label to the gotobar widget
	# See internationalization.py for full message of msg0055

	label = Label(msg0055)
	gotobar.pack_start(label, False, False, 3)

	# Get the current line number

	cursor_line = get_current_line(editor)

	# Create an adjustment for the spin button below

	adjustment = Adjustment()

	value = cursor_line + 1
	lower = 1
	upper =  editor.text_buffer.get_line_count()
	step_increment = 1
	page_increment = 5
	page_size = 11

	adjustment.set_all(value, lower, upper, step_increment, page_increment,
					page_size)

	# Add a spin button indicating the current line number to the gotobar
	# widget. The user can enter line numbers manually, or can use the spin
	# button to jump them to the specified line.

	spin_button = SpinButton(adjustment)
	editor.spin_button = spin_button
	spin_button.connect("value-changed", value_changed_cb, editor)
	spin_button.connect("key-press-event", key_press_event_cb, editor)
	gotobar.pack_start(spin_button, False, False, 3)
	spin_button.set_numeric(True)
	spin_button.set_update_policy(UPDATE_IF_VALID)
	spin_button.set_wrap(False)
	spin_button.set_snap_to_ticks(True)
	spin_button.set_digits(0)
	spin_button.set_property("xalign", 1)

	# Add a label indicating the total number of lines in the buffer

	gotobar_label = Label(msg0056 % upper)
	gotobar.pack_start(gotobar_label, False, False, 3)

	return gotobar


def show_gotobar(editor):
	u"""
	Show the gotobar and give focus to the elements within it.

	@param editor: Reference to the text editor's instance.
	@type editor: An editor object.

	@note: See flags.py for more information on gotobar_flag.
	@note: See internationalization.py for more information on msg0180.

	"""

	editor.gotobar_flag = True

	# Make the goto, search and replace buttons on the toolbar insensitive.

	editor.actiongroup.get_action(u"goto_line").set_sensitive(False)
	editor.actiongroup.get_action(u"search_text").set_sensitive(False)
	editor.actiongroup.get_action(u"replace_text").set_sensitive(False)

	# Create, show and focus the gotobar right above the statusbar area.

	editor.gotobar = create_gotobar(editor)
	editor.main_container.pack_start(editor.gotobar, False, False, 0)
	editor.main_container.reorder_child(editor.gotobar, 2)
	editor.spin_button.grab_focus()
	editor.gotobar.show_all()

	# Feedback to the statusbar indicating the editor is in goto mode.

	update_status_info(editor, msg0180, "goto", False)

	return


def hide_gotobar(editor):
	u"""
	Hide the gotobar and give focus back to the text editor's buffer.

	@param editor: Reference to the text editor's instance.
	@type editor: An editor object.

	@note: See flags.py for more information on gotobar_flag.
	@note: See internationalization for more information on msg0182.

	"""

	editor.gotobar_flag = False

	# Destroy the gotobar and focus the text's editor's buffer.

	editor.gotobar.hide_all()
	editor.gotobar.destroy()
	editor.text_view.grab_focus()

	# Make the goto and search toolbar buttons sensitive. In readonly mode
	# make the replace toolbar button insensitive.

	if editor.readonly:

		flag = False

	else:

		flag = True

	editor.actiongroup.get_action(u"goto_line").set_sensitive(True)
	editor.actiongroup.get_action(u"search_text").set_sensitive(True)
	editor.actiongroup.get_action(u"replace_text").set_sensitive(flag)

	# Feedback to statusbar indicating the gotobar has been closed.

	update_status_info(editor, msg0182, "info")

	collect()

	return

