# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module contains functions that monitor or filter keyboard events triggered
from the text editor's buffer.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from pygtk import require
require("2.0")
from brackets import remove_matching_brackets, activate_bracket_completion
from brackets import move_cursor_out_of_bracket_end, enclose_selection
from autocomplete import manipulate_completion_window
from templates import check_for_template_insertion


def create_keys_dictionary():
	"""
	Create a dictionary containing keyboard codes.

	@return: A dictionary containing keyboard codes.
	@rtype: A dictionary object.

	"""

	keys = {
		"quote": 34,
		"up": 65362,
		"down": 65364,
		"backspace": 65288,
		"tab":  65289,
		"left": 65363,
		"right": 65361,
		"enter": 65293,
		"control": 65507,
		"shift": 65505,
		"alt": 65513,
		"underscore": 95,
		"escape": 65307,
	}

	return keys


def exit_bracket_region(editor, keyval, keys):
	"""
	Move the text editor's cursor out of a bracket region when the escape key is
	pressed.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	@param keyval: Key code representing a keyboard character pressed.
	@type keyval: An integer object.

	@param keys: A dictionary containing selected key codes.
	@type keys: A dictionary object.

	"""

	if editor.bracket_region_flag:

		if keyval == keys["escape"]:

			move_cursor_out_of_bracket_end(editor)

	return


def remove_bracket_completion(editor, keyval, keys):
	"""
	Remove an automatic bracket completion operation or move the text editor's
	cursor out of a bracket region.

	Immediately after a bracket completion operation occurs pressing the
	backspace key will remove the opening and closing bracket. Immediately
	after a bracket completion operation occurs pressing the key of closing
	bracket key will move the cursor out of the bracket region.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	@param keyval: Key code representing a keyboard character pressed.
	@type keyval: An integer object.

	@param keys: A dictionary containing selected key codes.
	@type keys: A dictionary object.

	@return: A value indicating whether or not to propagate the key event
			signal. False to propagate the signal, an True otherwise.
	@rtype: A boolean value.

	"""

	if editor.backspace_flag and keyval == keys["backspace"]:

		# remove matching bracket
		remove_matching_brackets(editor)
		value = True

	elif editor.backspace_flag and editor.bracket_region_flag:

		closing_bracket = editor.bracket_dictionary[editor.bracket_count][0][1]

		if keyval == ord(closing_bracket):

			# Exit bracket region.
			move_cursor_out_of_bracket_end(editor)
			value = True

		else:

			value = False

	else:

		value = False

	editor.backspace_flag = False

	return value


def bracket_completion(editor, keyval):
	"""
	Complete [, (, " and { with their closing counterparts.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	@param keyval: Key code representing a keyboard character pressed.
	@type keyval: An integer object.

	@return: A value indicating whether or not to propagate the key event
			signal. False to propagate the signal, an True otherwise.
	@rtype: A boolean value.

	"""

	from gnomevfs import get_uri_from_local_path, get_mime_type
	from internationalization import msg0025

	if editor.filename == msg0025:

		mimetype = None

	else:

		uri = get_uri_from_local_path(editor.filename)
		mimetype = get_mime_type(uri)

	if mimetype in ("text/xml", "application/xml", "text/html"):

		keys = (123, 91, 34, 40, 60, ord("»".decode("utf-8")),
			ord("«".decode("utf-8")), ord("「".decode("utf-8")))

	else:

		keys = (123, 91, 34, 40, ord("»".decode("utf-8")),
			ord("«".decode("utf-8")), ord("「".decode("utf-8")))

	# If the key pressed is the quote key, activate bracket completion.
	if keyval in keys:

		activate_bracket_completion(editor, keyval)
		value = True

	else:

		value = False

	return value


def monitor_opening_bracket_characters(editor, keyval):
	"""
	Listen for (, [, ", {, ' and < characters.

	@param editor: Reference to the text editor.
	@type editor: An editor object.

	@param keyval: An event generated by pressing a character on the keyboard.
	@type keyval: A gtk.Event.

	@return: True if characters in question where detected, false otherwise.
	@rtype: A boolean object.

	"""

	if keyval in (123, 91, 34, 40, 60, 39, ord("»".decode("utf-8")),
				ord("«".decode("utf-8")), ord("「".decode("utf-8"))):

		enclose_selection(editor, keyval)
		result = True

	else:

		result = False

	return result


def search_for_selection(editor):
	"""
	Search for selections in the text editor's buffer.

	@param editor: Reference to the text editor.
	@type editor: An editor object.

	"""

	result = editor.text_buffer.get_selection_bounds()

	return result


def filter_key_events(editor, keyval):
	"""
	Filter key events and perform certain actions depending on the key events
	propagated.

	@param editor: Reference to the text editor.
	@type editor: An editor object.

	@param keyval: An event generated by pressing a character on the keyboard.
	@type keyval: A gtk.Event.

	@return: A value indicating whether or not to propagate the key event
			signal. False to propagate the signal, an True otherwise.
	@rtype: A boolean value.

	"""

	keys = create_keys_dictionary()

	if keyval == keys["tab"]:

		result = check_for_template_insertion(editor)

		if result:

			return result

	result = search_for_selection(editor)

	if result:

		result = monitor_opening_bracket_characters(editor, keyval)

		if result:

			return result

	exit_bracket_region(editor, keyval, keys)
	result = remove_bracket_completion(editor, keyval, keys)

	if result:

		return result

	result = bracket_completion(editor, keyval)

	if result:

		return result

	result = manipulate_completion_window(editor, keyval, keys)

	if result:

		return result

	return False

