# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
This module exposes an object that allows developers to log the activities of
the text editor.

The module exposes a logger object called log. log allows developers to place
debugging or informational messages at crucial or arbitrary locations in a
module to be debugged or analyzed. As opposed to print statements, log outputs
debugging information to the console as well as to a file named
~/Desktop/scribes.log.

The output of log contains the name and line number of the module that triggered
the log, the time the log was triggered as well as the information that was
logged. Technically, log is just a wrapper object of Python's logging class.
Thus, log exposes all the methods exposed by a logging.getLogger object. See
the Python logging module manual for more information.

Examples:

	log.debug("some message I think I need to know", filename)

	log.info("Initialized sexy_scribes function")

	log.exception("Hmmm, what could be wrong?", filename)

Use log instead of print statements for debugging.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from info import name, DESKTOPDIR
from logging import getLogger, Formatter, StreamHandler, FileHandler


def create_formatter(place="file"):
	u"""
	Create the format of the message to be logged.

	Messages logged to the console are less verbose than those logged to the
	log file.

	@param place: Whether to log messages to console or to a file
	@type place: A string object. It can either be "file" or "console"

	@return: Returns a format for logging messages
	@rtype: A logging.Formatter object

	"""

	if place == "file":

		formatter = Formatter("%(module)s %(lineno)d: %(levelname)s \
%(asctime)s: %(message)s")

	else:

		formatter = Formatter("%(module)s %(lineno)d: %(message)s")

	return formatter


def create_handler(place="file"):
	u"""
	Create a handler to determine where logged information is placed.

	The handler created only determines whether to log messages to a console or
	to a file.

	@param place: Where to log messages to. It can be the console or to a file
	@type place: A string object. It can either be "file" or "console"

	@return: Returns the destination where messages are logged.
	@rtype: A logging.Handler object

	"""

	if place == "file":

		desktop = DESKTOPDIR + "/scribes.log"
		handler = FileHandler(desktop, "a")
		handler.setFormatter(create_formatter())

	else:

		handler = StreamHandler()
		handler.setFormatter(create_formatter("console"))

	return handler


def create_logger():
	u"""
	Create a logger for logging debugging or informational messages.

	@return: Returns an object for logging messages to the console.
	@rtype: A logging.Logger object

	"""

	# Create the logger.

	log = getLogger(name)

	# Log messages all messages.

	log.setLevel(10)

	# Log to both console and file.

	log.addHandler(create_handler())
	log.addHandler(create_handler("console"))

	return log

# Logger object for exportation to other modules.

log = create_logger()

