# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


u"""
This module contains the function to the main entry point of the program. All
functions necessary to spawn new instances of the text editor can be found in
this module.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""


def fork_scribes():
	u"""
	Fork text editor's process to free the terminal.

	"""

	from os import fork
	pid = fork()

	if pid != 0:

		from sys import exit
		exit(0)

	return


def test(FileLoader, editor, uri, readonly):

	FileLoader(editor, uri, readonly)
	editor.show_all()
	editor.window.process_updates(True)

	return False


def load_uris(editor, uri_list=None, readonly=False):
	u"""
	Load one or more files into the text editor's buffer.

	This function should be called when there is already an instance of the
	text editor running. If the text editor already holds a document, a new
	instance of the editor is spawned and the file is loaded into the new
	instance.

	@param self: Reference to the text editor instance.
	@type self: An Editor object.

	@param uri_list: A list of files(URIs) to be loaded into text editor
		instances
	@type uri_list: A List object.

	@param readonly: Whether or not files should be loaded in readonly mode.
	@type readonly: A Boolean object.

	"""

	from fileloader import FileLoader
	from gobject import timeout_add

	for uri in uri_list:

		timeout_add(3, test, FileLoader, editor, uri, readonly)

	return


def launch_new_editor(uri_list=None, readonly=False):
	u"""
	Create a new instance of a text editor and, optionally, load a text file
	into it.

	@param uri_list: A list of files(URIs) to be loaded into the text editor
		instances.
	@type uri_list: A List object.

	@param readonly: Whether or not files should be loaded in readonly mode.
	@type readonly: A Boolean object.

	"""

	from editor import Editor
	editor = Editor()

	if uri_list:

		load_uris(editor, uri_list, readonly)

	else:

		editor.show_all()
		editor.window.process_updates(True)

	return editor


def main(argv=None):
	u"""
	Entry point to the main program.

	This function initializes the text editor. It also processes command line
	arguments or options passed to the text editor via the shell terminal. Valid
	arguments are filenames. Valid options include --version(-v),
	--readonly(-r), --help(-h), --newfile(-n). If no arguments or options are
	passed to the text editor via the shell terminal, a single instance of the
	text editor is displayed.

	@param argv: Arguments passed to the text editor from the shell terminal.
	@type argv:	A List object.

	"""

	if not argv:

		# The text editor is called from the shell terminal without arguments.
		# Thus, launch an empty instance of the editor.
		fork_scribes()
		editor = launch_new_editor()

	else:

		# The text editor is called from the shell terminal with arguments.
		# Process the arguments and if necessary, launch a new instance of the
		# editor with files loaded.
		from commandline import CommandLineProcessor
		get_uri_list = CommandLineProcessor(argv)
		uri_list, readonly = get_uri_list()

		if uri_list:

			fork_scribes()

			# Create a new text editor instance.
			editor = launch_new_editor(uri_list, readonly)

		del get_uri_list

	try:

		if editor:

			# Connect editor to the GTK+ mainloop.
			from gtk import main
			main()

	except AttributeError:

		pass

	return


if __name__ == "__main__":

	main()

