# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


"""
Displays the editor's current mode in the status area.

The editor modes serve primarily as feedback services for the user. Scribes
strives to avoid showing dialog messages to convey information to the user.
Thus, icons representing useful information appear at the leftmost corner of the
status area. The icons get updated depending on what the user is doing, or what
mode the editor is in.

@comment: This feature is largely experimental. We need to brainstorm on a large
	list of possible user actions and modes that can be represented with icons
	that can be conveyed to the user.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from pygtk import require
require("2.0")
from gobject import timeout_add, source_remove
from gtk import Frame, STOCK_SPELL_CHECK, Image, STOCK_OPEN, STOCK_EDIT
from gtk import STOCK_SAVE, STOCK_SAVE_AS, STOCK_NEW, STOCK_PRINT, STOCK_UNDO
from gtk import STOCK_REDO, STOCK_JUMP_TO, STOCK_FIND, STOCK_FIND_AND_REPLACE
from gtk import STOCK_PREFERENCES, STOCK_DIALOG_WARNING
from gtk import STOCK_DIALOG_INFO, STOCK_DIALOG_AUTHENTICATION
from gtk import STOCK_DIALOG_ERROR, ICON_SIZE_MENU, STOCK_PASTE, STOCK_CUT
from gtk import STOCK_COPY, SHADOW_IN, ProgressBar
from gtk import STOCK_HELP, STOCK_SELECT_COLOR
from gtk import STOCK_YES, STOCK_NO
from gtk import STOCK_EXECUTE, STOCK_ABOUT

from internationalization import msg0062, msg0095, msg0096


def calculate_progress(editor, progress):
	"""
	Calculate the progress bar fraction

	@param editor: Reference to the editor instance
	@type editor: An editor object
	@param progress: Progress bar widget
	@type progress: GTK progressbar

	"""

	try:

		amount_loaded = float(editor.opt_count)/float(editor.file_size)
		amount_left = float(editor.read_size)/float(editor.file_size)
		value = amount_loaded + amount_left
		string = msg0095 % (int(editor.opt_count / editor.file_size * 100))
		progress.set_text(string)

	except ZeroDivisionError:

		value = 0.0

	if not editor.progress_flag or value > 1.0:

		progress.set_text(msg0096)
		source_remove(editor.timer)
		source_remove(editor.gtimer)

		return False

	if value >= 0.0 and value <= 1.0:

		progress.set_fraction(value)

	return True


def create_progressbar(editor):
	"""
	Create and show a progress bar.

	The progress bar is shown in the statusbar area when loading files that take
	more than one second to load.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	@return: A progress bar.
	@rtype: A gtk.ProgressBar object.

	@note: See flags.py for more information on progress_flag.
	@note: See internationalization.py for more information on msg0062.

	"""

	editor.progress_flag = True
	progress = ProgressBar()
	progress.set_text(msg0062)
	progress.set_ellipsize(True)
	progress.show()
	editor.timer = timeout_add(500, calculate_progress, editor, progress)

	return progress


def create_mode(icon):
	"""
	Create a frame containing a GTK+ stock icon depicting a user operation.

	The frame is shown in the statusbar area when a user performs an action or
	when the mode/state of the text editor changes.

	@param icon: A GTK+ stock icon to display in the statusbar area.
	@type icon: A string object

	@return: A frame containing a stock icon representing a user operation.
	@rtype: A gtk.Frame object.

	"""

	# Create a frame to hold stock icons
	mode = Frame()
	mode.set_no_show_all(True)
	mode.set_shadow_type(SHADOW_IN)

	# Create image from stock icon
	image = Image()
	mode.add(image)
	image.set_from_stock(icon, ICON_SIZE_MENU)
	image.show()

	return mode


def update_editor_mode(editor, icon="", progress=False):
	"""
	Display an icon, or progress bar, in the status area representing the
	current mode the editor is in.

	@param editor: Reference to an editor instance
	@type editor: An editor object

	@param icon: A stock icon that represents a mode or operation.
	@type icon: A string object

	@param progress: Whether or not to activate the progress bar
	@type progress: A boolean object

	@return: A mode indicating an editor operation
	@rtype: A gtk.Frame object containing a Stock icon

	"""

	modes = {
		"open": STOCK_OPEN,
		"edit": STOCK_EDIT,
		"save": STOCK_SAVE,
		"saveas": STOCK_SAVE_AS,
		"new": STOCK_NEW,
		"print": STOCK_PRINT,
		"undo": STOCK_UNDO,
		"redo": STOCK_REDO,
		"goto": STOCK_JUMP_TO,
		"find": STOCK_FIND,
		"replace": STOCK_FIND_AND_REPLACE,
		"preferences": STOCK_PREFERENCES,
		"warning": STOCK_DIALOG_WARNING,
		"readonly": STOCK_DIALOG_AUTHENTICATION,
		"error": STOCK_DIALOG_ERROR,
		"copy" : STOCK_COPY,
		"cut": STOCK_CUT,
		"paste": STOCK_PASTE,
		"spell": STOCK_SPELL_CHECK,
		"help": STOCK_HELP,
		"yes": STOCK_YES,
		"no": STOCK_NO,
		"run": STOCK_EXECUTE,
		"info": STOCK_DIALOG_INFO,
		"about": STOCK_ABOUT,
		"color": STOCK_SELECT_COLOR,
		"" : None
	}

	if progress:

		return create_progressbar(editor)

	else:

		return create_mode(modes[icon])


def remove_mode(editor):
	"""
	Remove icon depicting user operations or editor's mode from the statusbar
	area.

	@param editor: editor
	@type editor: editor object

	@note: I don't envision this functions being called directly. Instead the
		convenience function update_status_info should be used to send
		feedback to the statusbar area. See utility.py for the definition of
		the function.

	"""

	if editor.mode_flag:

		editor.mode.hide()
		editor.mode_flag = False
		editor.mode.destroy()
		del editor.mode

	return


def add_mode(editor, icon="", progress=False):
	"""
	Show an icon depicting user operations in the text editor's statusbar area.

	@param editor: editor
	@type editor: editor object

	@param icon: stock icon
	@typ icon: string

	@param progress: show/hide a progress bar
	@type progress: boolean

	@note: I don't envision this functions being called directly. Instead the
		convenience function update_status_info should be used to send
		feedback to the statusbar area. See utility.py for the definition of
		the function.

	"""

	# Do not show icon if the progress bar is active.
	if editor.progress_flag:

		return

	# If an icon already exist in the status area, remove it first.
	if editor.mode_flag:

		remove_mode(editor)

	# The flag indicates an icon exists in the status area
	editor.mode_flag = True
	mode = update_editor_mode(editor, icon, progress)

	# Update the status area and show icon mode
	editor.status_area.pack_start(mode, expand=False, fill=False,
								padding=0)
	editor.status_area.reorder_child(mode, 0)
	editor.mode = mode
	mode.show()

	return


def reset_mode(editor):
	"""
	Reset the icon in the statusbar area to the correct mode the editor is in.

	The editor operates at two or three important modes throughout its lifetime.
	They are edit, saved and readonly modes. Edit mode indicates the editor is
	currently being edited and has not been saved. Saved mode indicates the
	editor has been saved and is not being edited. Readonly mode as the name
	suggests indicates the editor is open in readonly mode. After temporary
	messages have been displayed in the statusbar area, the area has to be
	reset to a state that indicates the current mode the editor is in. This
	function handles this.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	@note: I don't envision this functions being called directly. Instead the
		convenience function update_status_info should be used to send
		feedback to the statusbar area. See utility.py for the definition of
		the function.

	"""

	# Do nothing if the progress bar is active.
	if editor.progress_flag:

		return

	if editor.readonly:

		add_mode(editor, "readonly")

		return

	# If the text editor's buffer is empty and has not been modified show no
	# icon.
	if editor.isNewFile and not editor.dirty:

		remove_mode(editor)

		return

	# If the file has been edited but not yet saved, show the edit icon. Show
	# the new file icon if the file has been saved.
	if not editor.save_flag or editor.mode_edit_flag:

		editor.mode_edit_flag = False
		add_mode(editor, "edit")

		return

	else:

		add_mode(editor, "new")

	return

