# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


u"""
This module implements an authentication dialog that allows users access files
at remote locations.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""


class PasswordDialog(object):
	u"""
	This class defines a dialog that allows users to authenticate themselves to
	access files on remote location. It exposes a simple text entry field where
	users can insert their password.

	"""

	def __init__(self, fileloader):
		u"""
		Initialize the PasswordDialog object.

		@param self: Reference to the PasswordDialog instance.
		@type self: A PasswordDialog object.

		@param fileloader: An object that loads URIs into the text editor's
			buffer.
		@type fileloader: A FileLoader object.

		"""

		self._init_attributes(fileloader)
		self._set_dialog_window_properties()
		self._create_dialog_widgets()
		self._arrange_dialog_widgets()
		self.show_template_editor()


	def _init_attributes(self, fileloader):
		u"""
		Initialize attributes the PasswordDialog instance that will be used
		repeatedly accross the class structure.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		@param fileloader: An object that loads URIs into the text editor's
			buffer.
		@type fileloader: A FileLoader object.

		"""

		from gtk import Dialog
		from gtk.gdk import KEY_PRESS_MASK
		self.editor = fileloader.editor
		self.loader = fileloader
		self.dialog = Dialog()
		self.dialog.connect("key-press-event", self.key_press_event_cb)
		self.dialog.add_events(KEY_PRESS_MASK)

		return


	def _create_dialog_widgets(self):
		"""
		Create all widgets displayed in the authentication dialog.

		@param self: Reference to the PasswordDialog instance.
		@type self: A PasswordDialog object.

		"""

		self.label = self._create_dialog_labels()
		self.image = self._create_image()
		self.entry = self._create_text_entry()

		return


	def _set_dialog_window_properties(self):
		"""
		Define the dialog window properties.

		@param self: Reference to the PasswordDialog instance.
		@type self: A PasswordDialog object.

		"""

		from internationalization import msg0330
		from gtk.gdk import WINDOW_TYPE_HINT_DIALOG, GRAVITY_CENTER
		from gtk import WIN_POS_CENTER_ON_PARENT, STOCK_OK, RESPONSE_OK
		from gtk import STOCK_CANCEL, RESPONSE_CANCEL
		width, height = self.editor.calculate_resolution_independence(1.0, 1.0)
		dialog = self.dialog
		dialog.set_property("title", msg0330)
		dialog.set_property("role", "template_dialog")
		dialog.set_property("type-hint", WINDOW_TYPE_HINT_DIALOG)
		dialog.set_property("modal", True)
		dialog.set_property("skip-taskbar-hint", True)
		dialog.set_property("skip-pager-hint", True)
		dialog.set_property("icon-name", "gnome-settings")
		dialog.set_property("window-position", WIN_POS_CENTER_ON_PARENT)
		dialog.set_property("gravity", GRAVITY_CENTER)
		dialog.set_property("resizable", True)
		dialog.set_property("has-separator", False)
		dialog.set_property("border-width", 12)
		dialog.vbox.set_spacing(12)
		dialog.action_area.set_spacing(6)
		dialog.add_button(STOCK_CANCEL, RESPONSE_CANCEL)
		dialog.add_button(STOCK_OK, RESPONSE_OK)
		dialog.set_transient_for(self.editor)

		return


	def _arrange_dialog_widgets(self):
		"""
		Pack widgets appropriately into the TemplateAdd dialog.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		"""

		from gtk import HBox, VBox, Alignment
		hbox = HBox(False, 12)
		self.dialog.vbox.pack_start(hbox, False, False, 0)
		hbox.pack_start(self.image, False, False, 0)

		vbox = VBox(True, 12)
		hbox.pack_start(vbox, True, True, 0)
		alignment = Alignment(xalign=0.0, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(self.label[0])
		vbox.pack_start(alignment, False, False, 0)

		hbox = HBox(False, 12)
		vbox.pack_start(hbox, True, True, 0)
		self.label[1].set_mnemonic_widget(self.entry)
		hbox.pack_start(self.label[1], False, False, 0)
		hbox.pack_start(self.entry, True, True, 0)

		return


	def show_template_editor(self):
		"""
		Display the TemplateAdd dialog.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		"""

		from gtk import RESPONSE_OK
		from gc import collect
		self.dialog.show_all()
		response = self.dialog.run()
		value = self.entry.get_text().strip()

		if not value:

			value = ""

		if response == RESPONSE_OK:

			self.loader.uri_object.password = value
			from gobject import timeout_add
			timeout_add(10, self._process_uri)
			self.dialog.hide_all()
			self.dialog.destroy()

		else:

			self.dialog.hide_all()
			self.dialog.destroy()
			self.loader._error_recovery()
			del self
			collect()

		return


################################################################################
#
#						Callback to for the dialog
#
################################################################################


	def key_press_event_cb(self, dialog, event):
		"""
		Callback to the dialog's "key-press-event"

		The "key-press-event" signal is emitted when the user presses a key on
		the	keyboard. This function closes the authentication dialog when the
		Escape key is pressed.

		@param self: Reference to the PasswordDialog instance.
		@type self: A PasswordDialog object.

		@param dialog: The authentication dialog.
		@type dialog: A Dialog object.

		@param event: Event when the Escape key is pressed.
		@type event: An Event object.

		"""

		from gtk import keysyms, RESPONSE_CANCEL

		if event.keyval == keysyms.Escape:

			self.dialog.response(RESPONSE_CANCEL)

		return False


################################################################################
#
#								Labels
#
################################################################################


	def _create_dialog_labels(self):
		"""
		Create labels for the authentication dialog.

		@param self: Reference to the PasswordDialog instance.
		@type self: A PasswordDialog object.

		"""

		from internationalization import msg0331, msg0332, msg0333, msg0334
		from gtk import Label
		from urlparse import urlparse
		strings = [msg0331 % "Mystilleef@sourceforge.net", msg0332, msg0333, msg0334]
		label_list = []

		# Convert strings to labels.
		for item in strings:

			label = Label(item)
			label.set_use_underline(True)
			label_list.append(label)

		return label_list


################################################################################
#
#							Authentication Image
#
################################################################################


	def _create_image(self):
		u"""
		Create an image to add to the authentication dialog.

		@param self: Reference to the PasswordDialog instance.
		@type self: A PasswordDialog object.

		"""

		from gtk import Image
		from gtk import STOCK_DIALOG_AUTHENTICATION, ICON_SIZE_DIALOG
		image = Image()
		image.set_from_stock(STOCK_DIALOG_AUTHENTICATION, ICON_SIZE_DIALOG)

		return image


################################################################################
#
#							Password Entry
#
################################################################################


	def _create_text_entry(self):
		"""
		Create an entry to hold the password.

		@param self: Reference to the PasswordDialog instance.
		@type self: A PasswordDialog object.

		@return: An entry to hold a password key.
		@rtype: A gtk.Entry object.

		"""

		from gtk import Entry
		entry = Entry()
		entry.set_visibility(False)
		entry.set_invisible_char("*")

		return entry


	def _process_uri(self):
		u"""
		Try reloading the URI with the new information (password) retrieved.

		@param self: Reference to the PasswordDialog instance.
		@type self: A PasswordDialog object.

		"""

		self.loader._process_uri()
		from gc import collect
		del self
		collect()

		return False

