# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Manage cursor position and window states.

This module contains code that keeps track of the editor's cursor position and
window state. The only function exported in this module is window_position. It
is called before a file is opened or closed.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from os import path, makedirs
from shelve import open as shelve_open
from bookmark import get_bookmarked_lines


def open_metadata_database(editor):
	"""
	Open the editor's metadata database.

	The metadata database contains miscellaneous information about the state of
	of the text editor and its contents before it is closed. The information
	is used when restore the editor and its contents to their last state.

	The database stores information about a file's window position (i.e window
	cordinates), state (i.e maximize or unmaximize) and the position of the
	cursor in file before it was is closed.

	@param editor: the editor.
	@type editor: an editor object

	"""

	# If the database folder does not exist, create one
	if not path.exists(editor.SCRIBESINFODIR):

		makedirs(editor.SCRIBESINFODIR)

	metadata = shelve_open(editor.SCRIBESINFODIR + "/scribesinfo.gdb", "c")

	return metadata


def close_metadata_database(metadata):
	"""
	Close the text editor's metadata database.

	@param metadata:
	@type metadata:

	"""

	metadata.close()

	return


def update_metadata_database(editor):
	"""
	Update the text editor's metadata database with new information.

	@param editor:
	@type editor:

	"""

	metadata = open_metadata_database(editor)

	if editor.fullscreen_flag:

		editor.maximize_flag = True

	# Check for bookmarks in the text editor's buffer.
	if editor.bookmark_flag:

		bookmarked_lines = get_bookmarked_lines(editor)

	else:

		bookmarked_lines = None

	rect = (editor.x, editor.y, editor.width, editor.height)
	metadata[str(editor.filename)] = (editor.cursor, editor.maximize_flag, rect,
							editor.encoding, bookmarked_lines)
	close_metadata_database(metadata)

	return


def get_metadata_info(editor):
	"""
	Retrieve information from the text editor's metadata database.

	"""

	value = False
	metadata = open_metadata_database(editor)

	# If the file has a key in the database continue processing. Otherwise
	# close the database and exit.
	if metadata.has_key(editor.filename):

		value = True

		# The flag indicates that information about the file exists in the
		# database.
		editor.position_flag = True
		editor.pos = metadata[str(editor.filename)]

		# Get encoding used to save the file if any.
		try:

			if editor.pos[3]:

				editor.encoding = editor.pos[3]

		except IndexError:

			pass

		# Check for bookmarks.
		try:

			if editor.pos[4]:

				editor.bookmark_flag = True
				editor.bookmarked_lines = editor.pos[4]

			else:

				editor.bookmark_flag = False

		except IndexError:

			editor.bookmark_flag = False

		# Check if the text editor's window needs to be maximized or
		# repositioned.
		if editor.pos[1]:

			editor.maximize()

		elif editor.pos[2]:

			editor.move(editor.pos[2][0], editor.pos[2][1])
			editor.resize(editor.pos[2][2], editor.pos[2][3])

	else:

		# The flag indicates that information about the file does not exist
		# in the database.
		editor.position_flag = False
		editor.bookmark_flag = False

	close_metadata_database(metadata)

	return value

