# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module defines user customization options for Scribes

"""

from gc import collect

from gobject import timeout_add, source_remove
from gtk import Dialog, WIN_POS_CENTER_ON_PARENT, STOCK_CLOSE, RESPONSE_CLOSE
from gtk import keysyms, Label, Alignment, HBox, FontButton, UPDATE_ALWAYS
from gtk import SpinButton, CheckButton
from gtk.gdk import WINDOW_TYPE_HINT_DIALOG, GRAVITY_CENTER

from scribes_gconf import get_gconf_client
from internationalization import msg0132, msg0187, msg0063, msg0064, msg0065
from internationalization import msg0066, msg0067, msg0068, msg0069, msg0070
from internationalization import msg0071, msg0072, msg0073, msg0196, msg0197
from internationalization import msg0198, msg0199, msg0200, msg0201, msg0202
from internationalization import msg0203, msg0023, msg0022
from tooltips import create_tooltips, font_button_tip, tab_spin_button_tip
from tooltips import tw_check_button_tip, margin_check_button_tip
from tooltips import margin_spin_button_tip, spell_check_button_tip
from feedback import update_status_info


class Preferences(object):
	"""
	The Preferences editor is a dialog window. It provides all the methods
	needed to customize the properties of the editor. Technically, the
	preferences dialog is a frontend to GConf, the GNOME configuration
	database and deamon.

	When changes are made to the dialog, those modifications are registered in
	the database. A GConf deamon then informs the editor of the changes, the
	editor consequently reflects these changes instantaneously.

	"""

	def __init__(self, editor):
		"""
		Initialize the Preferences class.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@param editor: Reference to an editor instance.
		@type editor: An Editor object.

		"""

		self._initialize_resuable_components(editor)
		self._set_dialog_window_properties()
		self._create_dialog_widgets()
		self._arrange_dialog_widgets()
		self.show_preference_dialog()


	def _initialize_resuable_components(self, editor):
		"""
		Initialize reusable components only once.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@param editor: Reference to an editor instance.
		@type editor: An Editor object.

		"""

		self.editor = editor
		self.dialog = Dialog()
		self.dialog.connect("key-press-event", self.key_press_event_cb)
		self.client = get_gconf_client()
		self.timer = 60
		self.preferences_flag = True
		self.tips = create_tooltips()

		return


	def _set_dialog_window_properties(self):
		"""
		Define the dialog window properties.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		"""

		# Calculate resolution independent width and height of the preferences
		# dialog window.
		width, height = \
self.editor.calculate_resolution_independence(2.56, 1.861818182)
		dialog = self.dialog
		dialog.set_property("title", msg0132)
		dialog.set_property("role", "preferences_dialog")
		dialog.set_property("type-hint", WINDOW_TYPE_HINT_DIALOG)
		dialog.set_property("modal", True)
		dialog.set_property("width-request", width)
		dialog.set_property("height-request", height)
		dialog.set_property("skip-taskbar-hint", True)
		dialog.set_property("skip-pager-hint", True)
		dialog.set_property("icon-name", "gnome-settings")
		dialog.set_property("window-position", WIN_POS_CENTER_ON_PARENT)
		dialog.set_property("gravity", GRAVITY_CENTER)
		dialog.set_property("resizable", True)
		dialog.set_property("has-separator", False)
		dialog.set_property("border-width", 10)
		dialog.vbox.set_spacing(10)
		dialog.vbox.set_homogeneous(False)
		dialog.add_button(STOCK_CLOSE, RESPONSE_CLOSE)
		dialog.set_transient_for(self.editor)

		return


	def _create_dialog_widgets(self):
		"""
		Create all widgets displayed in the preferences dialog.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		"""

		self.labels = self._create_labels()
		self.font_button = self._create_font_button()
		self.tab_spin_button = self._create_tab_spin_button()
		self.txtwrap_check_button = self._create_txtwrap_check_button()
		self.margin_check_button = self._create_margin_check_button()
		self.margin_spin_button = self._create_margin_spin_button()
		self.spell_check_button = self._create_spell_check_button()

		return


	def _arrange_dialog_widgets(self):
		"""
		Pack widgets appropriately into the preferences dialog.

		This function controls the alignment and position of buttons and labels
		within the preferences dialog.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		"""

		alignment = Alignment(xalign=0.0, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(self.labels[0])
		self.dialog.vbox.pack_start(alignment, True, False, 0)

		hbox = HBox(False, 12)
		alignment = Alignment(xalign=0.4, yalign=0.0, xscale=0.7, yscale=0.0)
		alignment.add(hbox)
		self.dialog.vbox.pack_start(alignment, False, False, 0)
		hbox.pack_start(self.labels[1], False, False, 0)
		hbox.pack_start(self.font_button, True, True, 0)
		self.labels[1].set_mnemonic_widget(self.font_button)

		alignment = Alignment(xalign=0.0, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(self.labels[2])
		self.dialog.vbox.pack_start(alignment, True, False, 0)

		hbox = HBox(False, 12)
		alignment = Alignment(xalign=0.1, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(hbox)
		self.dialog.vbox.pack_start(alignment, False, False, 0)
		hbox.pack_start(self.labels[3], False, False, 0)
		hbox.pack_start(self.tab_spin_button, False, False, 0)
		self.labels[3].set_mnemonic_widget(self.tab_spin_button)

		alignment = Alignment(xalign=0.0, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(self.labels[4])
		self.dialog.vbox.pack_start(alignment, True, False, 0)

		hbox = HBox(False, 12)
		alignment = Alignment(xalign=0.08, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(hbox)
		self.dialog.vbox.pack_start(alignment, False, False, 0)
		hbox.pack_start(self.txtwrap_check_button, False, False, 0)

		alignment = Alignment(xalign=0.0, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(self.labels[5])
		self.dialog.vbox.pack_start(alignment, True, False, 0)

		hbox = HBox(False, 12)
		alignment = Alignment(xalign=0.08, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(hbox)
		self.dialog.vbox.pack_start(alignment, False, False, 0)
		hbox.pack_start(self.margin_check_button, False, False, 0)

		hbox = HBox(False, 12)
		alignment = Alignment(xalign=0.1, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(hbox)
		self.dialog.vbox.pack_start(alignment, False, False, 0)
		hbox.pack_start(self.labels[6], False, False, 0)
		hbox.pack_start(self.margin_spin_button, False, False, 0)
		self.labels[6].set_mnemonic_widget(self.margin_spin_button)

		alignment = Alignment(xalign=0.0, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(self.labels[7])
		self.dialog.vbox.pack_start(alignment, True, False, 0)

		hbox = HBox(False, 12)
		alignment = Alignment(xalign=0.08, yalign=0.0, xscale=0.0, yscale=0.0)
		alignment.add(hbox)
		self.dialog.vbox.pack_start(alignment, False, False, 0)
		hbox.pack_start(self.spell_check_button, False, False, 0)

		return


	def show_preference_dialog(self):
		"""
		Display the preferences dialog.

		@param self: Reference to the Preference instance.
		@type self: A Preference object.

		"""

		update_status_info(self.editor, msg0196, "preferences", False)
		self.dialog.show_all()
		response = self.dialog.run()

		if response:

			source_remove(self.timer)
			self.dialog.hide_all()
			self.dialog.destroy()
			update_status_info(self.editor, msg0187, "info")
			del self
			collect()

		return


################################################################################
#
#						Close Dialog Callback
#
################################################################################


	def key_press_event_cb(self, dialog, event):
		"""
		Handles callback when the Escape key is pressed.

		This function destroys the preference dialog.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@param dialog: The Preferences dialog.
		@type dialog: A Preferences object.

		@param event: Event when the Escape key is pressed.
		@type event: An Event object.

		"""

		if event.keyval == keysyms.Escape:

			self.dialog.response(RESPONSE_CLOSE)

		return


################################################################################
#
#								Labels
#
################################################################################


	def _create_labels(self):
		"""
		Create labels for the preferences dialog.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		"""

		string_list = [msg0063, msg0064, msg0065, msg0066, msg0067, msg0069,
						msg0071, msg0072]
		label_list = []

		# Convert strings to labels.
		for string in string_list:

			label = Label(string)
			label.set_use_underline(True)
			label.set_use_markup(True)
			label_list.append(label)

		return label_list


################################################################################
#
#								Font Button
#
################################################################################


	def _create_font_button(self):
		"""
		Create the font button for the preferences dialog.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@return: A button that allows users to modify the font properties of
			the text editor.
		@rtype: A FontButton object.

		"""

		font = self.client.get_string("/apps/scribes/font")
		button = FontButton(font)
		button.connect("font-set", self.font_button_cb)
		self.tips.set_tip(button, font_button_tip)

		return button


	def font_button_cb(self, button):
		"""
		Handles callback when the font properties have been modified.

		This function updates the GConf database with the changes made in the
		font dialog.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@param button: The font button on the preferences dialog.
		@type button: A preferences object.

		"""

		font = button.get_font_name()
		self.client.set_string("/apps/scribes/font", font)
		message = msg0197 % font
		source_remove(self.timer)
		update_status_info(self.editor, message, "yes", False)
		self.timer = timeout_add(7000, self.timeout)

		return True


################################################################################
#
#							Tab Spin Button
#
################################################################################


	def _create_tab_spin_button(self):
		"""
		Create the tab spin button for the preferences dialog.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@return: The tab spin button for the preferences dialog.
		@rtype: A SpinButton object.

		"""

		value = self.client.get_int("/apps/scribes/tab")
		button = SpinButton()
		button.set_max_length(3)
		button.set_width_chars(3)
		button.set_digits(0)
		button.set_increments(1, 5)
		button.set_range(1, 24)
		button.set_update_policy(UPDATE_ALWAYS)
		button.set_numeric(True)
		button.set_snap_to_ticks(True)
		button.set_value(value)
		self.tips.set_tip(button, tab_spin_button_tip)
		button.connect("value-changed", self.tab_spin_button_cb)

		return button


	def tab_spin_button_cb(self, button):
		"""
		Handles callback when the value in the tab spin button changes.

		This function gets the value of the tab spin button and registers it
		in the GConf database.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@param button: The tab spin button in the preferences dialog.
		@type button: A SpinButton object.

		"""

		value = button.get_value()
		self.client.set_int("/apps/scribes/tab", int(value))
		message = msg0198 % value
		source_remove(self.timer)
		update_status_info(self.editor, message, "yes", False)
		self.timer = timeout_add(7000, self.timeout)

		return True


################################################################################
#
#						Text Wrap Check Button
#
################################################################################


	def _create_txtwrap_check_button(self):
		"""
		Create the text wrap check button for the preferences dialog.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@return: The text wrap check button for the preferences dialog.
		@rtype: A CheckButton object.

		"""

		value = self.client.get_bool("/apps/scribes/text_wrapping")
		button = CheckButton(msg0068, True)
		button.set_active(value)
		button.connect("toggled", self.txtwrap_check_button_cb)
		self.tips.set_tip(button, tw_check_button_tip)

		return button


	def txtwrap_check_button_cb(self, button):
		"""
		Handles callback when the state of the text wrap check button changes.

		This function gets the state of the text wrap check button and registers
		it to the GConf database.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@param button: The text wrap check button on the preferences dialog.
		@type button: A CheckButton object.

		"""

		value = button.get_active()
		self.client.set_bool("/apps/scribes/text_wrapping", value)
		source_remove(self.timer)

		if value:

			update_status_info(self.editor, msg0199, "yes", False)

		else:

			update_status_info(self.editor, msg0200, "yes", False)

		self.timer = timeout_add(7000, self.timeout)

		return True


################################################################################
#
#						Right Margin Check Button
#
################################################################################


	def _create_margin_check_button(self):
		"""
		Create the margin check button for the preferences dialog.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@return: The margin check button for the preferences dialog.
		@rtype: A CheckButton object.

		"""

		value = self.client.get_bool("/apps/scribes/margin")
		button = CheckButton(msg0070, True)
		button.set_active(value)
		button.connect("toggled", self.margin_check_button_cb)
		self.tips.set_tip(button, margin_check_button_tip)

		return button


	def margin_check_button_cb(self, button):
		"""
		Handles callback when the state of the margin check button changes.

		This function gets the state of the margin check button and registers it
		to the GConf database.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@param button: The margin check button on the preferences dialog.
		@type button: A CheckButton object.

		"""

		value = button.get_active()
		self.client.set_bool("/apps/scribes/margin", value)
		source_remove(self.timer)

		if value:

			self.margin_spin_button.set_sensitive(True)
			update_status_info(self.editor, msg0201, "yes", False)

		else:

			self.margin_spin_button.set_sensitive(False)
			update_status_info(self.editor, msg0202, "yes", False)

		self.timer = timeout_add(7000, self.timeout)

		return True


################################################################################
#
#					Right Margin Spin Button
#
################################################################################


	def _create_margin_spin_button(self):
		"""
		Create the margin spin button for the preferences dialog.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@return: The margin spin button for the preferences dialog.
		@rtype: A SpinButton object.

		"""

		value = self.client.get_int("/apps/scribes/margin_position")
		button = SpinButton()
		button.set_max_length(3)
		button.set_width_chars(3)
		button.set_digits(0)
		button.set_increments(1, 5)
		button.set_range(1, 160)
		button.set_update_policy(UPDATE_ALWAYS)
		button.set_numeric(True)
		button.set_snap_to_ticks(True)
		button.set_value(value)
		button.connect("value-changed", self.margin_spin_button_cb)
		self.tips.set_tip(button, margin_spin_button_tip)
		value = self.client.get_bool("/apps/scribes/margin")

		if value:

			button.set_sensitive(True)

		else:

			button.set_sensitive(False)

		return button


	def margin_spin_button_cb(self, button):
		"""
		Handles callback when the value of the margin spin button changes.

		This function gets the value of the margin spin button and registers it
		in the GConf database.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@param button: The margin spin button on the preferences dialog.
		@type button: A SpinButton object.

		"""

		value = button.get_value()
		self.client.set_int("/apps/scribes/margin_position", int(value))
		message = msg0203 % value
		source_remove(self.timer)
		update_status_info(self.editor, message, "yes", False)
		self.timer = timeout_add(7000, self.timeout)

		return True


################################################################################
#
#							Spell Check Buton
#
################################################################################


	def _create_spell_check_button(self):
		"""
		Create the spell check button for the preferences dialog.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@return: The spell check button for the preferences dialog.
		@rtype: A CheckButton object.

		"""

		value = self.client.get_bool("/apps/scribes/spell_check")
		button = CheckButton(msg0073, True)
		button.set_active(value)
		button.connect("toggled", self.spell_check_button_cb)
		self.tips.set_tip(button, spell_check_button_tip)

		return button


	def spell_check_button_cb(self, button):
		"""
		Handles callback when the state of the spell check button changes.

		This function gets the state of the spell check button and registers
		it to the GConf database.

		@param self: Reference to the Preferences instance.
		@type self: A Preferences object.

		@param button: The spell check button on the preferences dialog.
		@type button: A CheckButton object.

		"""

		value = button.get_active()
		self.client.set_bool("/apps/scribes/spell_check", value)
		source_remove(self.timer)

		if value:

			update_status_info(self.editor, msg0023, "yes", False)

		else:

			update_status_info(self.editor, msg0022, "yes", False)

		self.timer = timeout_add(7000, self.timeout)

		return True


################################################################################
#
#						Preferences Timeout
#
################################################################################


	def timeout(self):

		source_remove(self.timer)
		update_status_info(self.editor, msg0196, "preferences", False)

		return False

