# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Printing module for Scribes.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from pygtk import require
require("2.0")
from gtk import WIN_POS_CENTER_ON_PARENT
from gtk import WRAP_WORD
from gtk.gdk import WINDOW_TYPE_HINT_DIALOG
from gtk.gdk import GRAVITY_CENTER
from gtksourceview import SourcePrintJob
from gnomeprint import config_default
from gnomeprint.ui import JobPreview
from gnomeprint.ui import DIALOG_RESPONSE_PRINT
from gnomeprint.ui import DIALOG_RESPONSE_PREVIEW
from gnomeprint.ui import DIALOG_COPIES
from gnomeprint.ui import Dialog

from internationalization import msg0074, msg0075, msg0076, msg0194, msg0187
from internationalization import msg0195
from feedback import update_status_info


def print_doc(job):
	"""
	Print the content of the text editor's buffer.

	@param job:
	@type job:

	"""

	job.print_()

	return


def show_print_preview(dialog, job):
	"""
	Show the print preview window.

	The print preview window is used to preview the contents of the editor's
	buffer before it is sent to a printer, or file, for printing.

	@param dialog: a print dialog object
	@param job: a print job object

	"""

	preview_window = JobPreview(job, msg0074)

	# Set window properties
	preview_window.set_position(WIN_POS_CENTER_ON_PARENT)
	preview_window.set_gravity(GRAVITY_CENTER)
	preview_window.set_modal(True)
	preview_window.set_icon_name("stock_print")
	preview_window.set_transient_for(dialog)
	preview_window.show_all()

	return


def create_print_job(editor):
	"""
	Create a print job object.

	The print job object contains all the properties necessary to print a file,
	such as the content of the editor's buffer, margin properties, print
	configuration options defined by the user, or a default configuration if
	none is defined by the user, among others.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	@return: A job to be printed.
	@rtype:

	"""

	job = SourcePrintJob(config_default(), editor.text_buffer)
	file_header = msg0075 + editor.filename

	# Set job properties
	job.set_highlight(True)
	job.set_print_header(True)
	job.set_print_numbers(False)
	job.set_tabs_width(4)
	job.set_wrap_mode(WRAP_WORD)
	job.set_header_format(file_header, "", "", True)
	job = job.print_()

	return job


def response_cb(dialog, response, job, editor):
	"""
	Handles callback when buttons on the print dialog are clicked.

	The "response" signal is emitted when an action_area widget is activated
	(button "clicked"), the dialog receives a delete_event or the application
	calls the response() method. When a delete_event triggers the "response"
	signal the response_id will be gtk.RESPONSE_NONE.

	@param dialog: The text editor's print dialog object.
	@type dialog: A gnome.ui.Dialog object

	@param response: The response entered by the user.
	@type response: gnomeprint.ui.DIALOG_RESPONSE_PRINT or
					gnomeprint.ui.DIALOG_RESPONSE_PREVIEW or
					gnomeprint.ui.DIALOG_RESPONSE_CANCEL

	@param job: A job for printing.
	@type job:

	"""

	if response == DIALOG_RESPONSE_PRINT:

		print_doc(job)
		dialog.hide_all()
		dialog.destroy()
		message = msg0195 % editor.filename
		update_status_info(editor, message, "print")

	elif response == DIALOG_RESPONSE_PREVIEW:

		show_print_preview(dialog, job)

	else:

		dialog.hide_all()
		dialog.destroy()
		update_status_info(editor, msg0187, "info")

	return


def show_print_dialog(editor):
	"""
	Show the print dialog.

	@param editor: Reference to editor instance.
	@type editor: An editor object.

	@return: The text editor's print dialog.
	@rtype: gnome.ui.print object.

	"""

	width, height = editor.calculate_resolution_independence(1.6, 1.929648241)
	update_status_info(editor, msg0194, "print", False)
	job = create_print_job(editor)
	dialog = Dialog(job, msg0076, DIALOG_COPIES)
	dialog.connect(u"response", response_cb, job, editor)

	# Set window properties
	dialog.set_default_size(width, height)
	dialog.set_skip_taskbar_hint(True)
	dialog.set_modal(True)
	dialog.set_icon_name(u"stock_print")
	dialog.set_gravity(GRAVITY_CENTER)
	dialog.set_type_hint(WINDOW_TYPE_HINT_DIALOG)
	dialog.set_position(WIN_POS_CENTER_ON_PARENT)
	dialog.set_transient_for(editor)
	dialog.run()

	return dialog
