# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Defines all the functions necessary to toggle readonly mode on or off.
FIXME: Better documentation for this module.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from os import access, W_OK, path

from cursor import cursor_position
from internationalization import msg0034, msg0322, msg0323, msg0324, msg0325
from feedback import update_status_info


def activate_readonly_mode(editor):
	"""
	Switch the text editor to readonly mode.

	This function changes several visual elements of the text editor and its
	buffer. It hides the line number margin, removes the cursor porsition and
	mode indicators in the statusbar area, appends a yellow "<READONLY>" tag to
	the text editor's window title and changes the color of the message in the
	statusbar area to blue.

	@param editor: Reference to the text editor's buffer.
	@type editor: An editor object.

	"""

	# The editor is in readonly mode when the flag is true.
	editor.readonly = True
	editor.text_view.set_editable(False)

	# Append "<READONLY>" to the text editor's window title.
	#editor.set_title(path.basename(editor.filename)+ msg0034)
	title = editor.uri_object.short_name
	editor.set_title(title + msg0034)

	# Hide the line number margin.
	editor.text_view.set_show_line_numbers(False)
	editor.text_view.set_cursor_visible(False)
	editor.text_view.set_highlight_current_line(False)

	string = u" "

	# Blank the second statusbar.
	editor.status_three.pop(editor.status_three_id)
	editor.status_three_id = editor.status_three.get_context_id(string)
	editor.status_three.push(editor.status_three_id, string)
	editor.text_view.handler_block(editor.ovrid)

	# Blank the third statusbar.
	editor.status_two.pop(editor.status_two_id)
	editor.status_two_id = editor.status_two.get_context_id(string)
	editor.status_two.push(editor.status_two_id, string)
	editor.text_buffer.handler_block(editor.markid)
	editor.text_buffer.handler_block(editor.marksetid)
	editor.actiongroup.get_action(u"replace_text").set_sensitive(False)
	editor.actiongroup.get_action(u"undo_edits").set_sensitive(False)
	editor.actiongroup.get_action(u"redo_edits").set_sensitive(False)

	return


def deactivate_readonly_mode(editor):
	"""
	Switch the text editor to readonly mode.

	This function changes several visual elements of the text editor and its
	buffer. It hides the line number margin, removes the cursor porsition and
	mode indicators in the statusbar area, appends a yellow "<READONLY>" tag to
	the text editor's window title and changes the color of the message in the
	statusbar area to blue.

	@param editor: Reference to the text editor's buffer.
	@type editor: An editor object.

	"""

	# The editor is in readonly mode when the flag is true.
	editor.readonly = False
	editor.set_title(editor.uri_object.short_name)
	editor.text_view.set_show_line_numbers(True)
	editor.text_view.set_editable(True)
	editor.text_view.set_cursor_visible(True)
	editor.text_view.set_highlight_current_line(True)

	# Insert mode is the default editor mode. Let the status bar reflect
	# this mode by default.
	default_mode = u"INS"
	editor.status_three.pop(editor.status_three_id)
	editor.status_three_id = editor.status_three.get_context_id(default_mode)
	editor.status_three.push(editor.status_three_id, default_mode.ljust(0))
	editor.text_view.handler_unblock(editor.ovrid)

	# FIXME:
	editor.text_buffer.handler_unblock(editor.markid)
	editor.text_buffer.handler_unblock(editor.marksetid)
	cursor_position(editor)
	editor.actiongroup.get_action(u"replace_text").set_sensitive(True)

	if editor.text_buffer.can_undo():

		editor.actiongroup.get_action(u"undo_edits").set_sensitive(True)

	if editor.text_buffer.can_redo():

		editor.actiongroup.get_action(u"redo_edits").set_sensitive(True)

	return


def check_file_readonly_permission(editor):
	"""
	Check if the file can be modified.

	The enter_readonly_mode and leave_readonly_mode functions should only
	complete successfully only if the file can be modified.

	@param editor: Reference to the text editor's buffer.
	@type editor: An editor object.

	"""

	if not access(editor.filename, W_OK):

		permission = False

	else:

		permission = True

	return permission


def enter_readonly_mode(editor):
	"""
	Enter readonly mode.

	@param editor: Reference to the text editor's buffer.
	@type editor: An editor object.

	"""

	result = check_file_readonly_permission(editor)

	if result:

		editor.actiongroup.get_action(u"save_file").activate()
		update_status_info(editor, msg0322, "yes")
		activate_readonly_mode(editor)

	else:

		update_status_info(editor, msg0323, "no")

	return


def leave_readonly_mode(editor):
	"""
	Exit readonly mode.

	@param editor: Reference to the text editor's buffer.
	@type editor: An editor object.

	"""

	result = check_file_readonly_permission(editor)

	if result:


		update_status_info(editor, msg0324, "yes")
		deactivate_readonly_mode(editor)

	else:

		update_status_info(editor, msg0325, "no")

	return

