# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
This modules defines a class responsible for saving files in Scribes.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gc import collect
from os import path, chdir, getcwd, listdir, environ

from gnomevfs import get_uri_from_local_path
from gnomevfs.async import create

from error import error_dialog
from feedback import update_status_info
from internationalization import msg0025, msg0083, msg0084, msg0085, msg0152


class SaveFile(object):
	u"""
	The SaveFile class defines attributes and methods for saving files.
	"""

	def __init__(self, editor):
		u"""
		Save the contents of the editor's buffer to a file on the host's
		filesystem.

		@param self: SaveFile
		@type self: SaveFile Object
		@param editor: editor
		@type editor: editor Object

		"""

		# General setup up for internationalization and localization using
		# gettext.
		self.ed = editor

		# Get the encoding used to open the current file.

		self.encoding = self.ed.encoding

		# This flag indicates that the saving process is just beginning. It is
		# important to ensure that the file is completely saved before the
		# editor exists.

		self.ed.saving_flag = True

		# Prepare the editor for saving
		self.process()


	def process(self):
		u"""
		Prepare the editor for saving, determine where to save the file, and
		define file name semantics.

		@param self: Reference to the SaveFile object
		@type self: A SaveFile object

		"""

		# If the file is a new file and the editor's buffer has not been
		# modified, show the saveas dialog when Control-S is pressed
		if self.ed.isNewFile and not self.ed.dirty:

			self.ed.actiongroup.get_action(u"saveas_file").activate()
			del self
			collect()

			return

		# Save all unsaved documents to the desktop folder. If the desktop
		# folder does not exist, save it the user's home folder. If the file is
		# an existing file, save the folder the file resides in. For all other
		# situations, save the file to the user's home folder.
		if path.exists(self.ed.DESKTOPDIR) and self.ed.isNewFile:

			chdir(self.ed.DESKTOPDIR)
			temp_filename = self.file_counter()
			self.ed.filename = self.ed.DESKTOPDIR + u"/" + temp_filename

		elif self.ed.isNewFile:

			chdir(self.ed.HOMEDIR)
			temp_filename = self.file_counter()
			self.ed.filename = self.ed.HOMEDIR + u"/" + temp_filename

		elif self.ed.filename:

			chdir(path.dirname(self.ed.filename))

		else:

			chdir(self.ed.HOMEDIR)

		# Redefine filename semantics
		self.ed.file_uri = get_uri_from_local_path(self.ed.filename)
		from gnomevfs import URI
		self.ed.uri_object = URI(self.ed.file_uri)

		if self.ed.filename.startswith(environ[u'HOME']):

			self.ed.prettyname = self.ed.filename.replace(environ[u'HOME'], u"~")

		else:

			self.ed.prettyname = self.ed.filename

		# Open file for writing.
		try:

			create(self.ed.file_uri, self.create_file_cb)

		except:

			error_dialog(self.ed, msg0083)

			return

		# Set the Window border title
		self.ed.set_title(self.ed.uri_object.short_name)

		# The file is no longer new
		self.ed.isNewFile = False

		return


	def create_file_cb(self, handle, exc_type):
		u"""
		Handles callback to the asynchronous save operation

		@param self: Reference to the SaveFile object
		@type self: A SaveFile object

		@param handle:
		@type handle:

		@param exc_type:
		@type exc_type:

		"""

		start, end = self.ed.text_buffer.get_bounds()
		text = self.ed.text_buffer.get_text(start, end)

		# Convert text in buffer from UTF-8 to Unicode
		try:

			unicode_txt = text.decode(u"utf8")

		except UnicodeDecodeError:

			error_dialog(self.ed, msg0084)

			return

		# Convert Unicode text to encoding used to open the file
		try:


			output = unicode_txt.encode(self.encoding, u"replace")

		except UnicodeEncodeError:

			error_dialog(self.ed, msg0152)

			return

		handle.write(output, self.write_file_cb)

		return


	def write_file_cb(self, handle, bytes, exc_type, bytes_requested):
		u"""
		Handles callback to asynchronously read files.

		@param self: Reference to the SaveFile object
		@type self: A SaveFile object

		@param handle:
		@type handle:

		@param bytes:
		@type bytes:

		@param exc_type:
		@type exc_type:

		@param bytes_requested
		@type bytes_requested

		"""

		self.ed.text_buffer.set_modified(False)

		# This flag indicates that the saving process is over.
		self.ed.saving_flag = False
		handle.close(self.close_file_cb)

		return


	def close_file_cb(self, *args):
		u"""
		Handles callback to asynchronously close file handles.

		@param self: Reference to the SaveFile object
		@type self: A SaveFile object

		@param handle:
		@type handle:

		@param bytes:
		@type bytes:

		@param exc_type:
		@type exc_type:

		@param bytes_requested
		@type bytes_requested

		"""

		# Call the collector only if the file is not in the saving process.
		if not self.ed.saving_flag:

			collect()

		# This flag shows the file has already been saved and doesn't need
		# further saving. Used by the automatic saving function of the
		# editor. When the flag is true, the file is saved. Otherwise, the
		# file is unsaved.
		self.ed.save_flag = True

		if not self.ed.status_busy_flag:

			self.ed.mode_edit_flag = True
			message = msg0085 % self.ed.prettyname
			update_status_info(self.ed, message, u"save")

		return



	def file_counter(self):
		u"""
		Calculate a count to append to the end of an unsaved document

		@param self: An object for saving files
		@param type: SaveFile object

		@return: the name of an unsaved document with a count value appended
		@rtype: string
		"""

		count = 0

		for files in listdir(getcwd()):

			files = str(files)

			if files.startswith(msg0025):

				count = count + 1

		temp_filename = self.ed.filename + u" " + str(count + 1)

		return temp_filename.strip()

