# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""

This module contains the function necessary for selection operations and
manipulations. Any operations that involves selecting words, lines, sentences,
paragraphs and blocks of code should be found here.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
@todo: Write a function to open a new line above or below the current line.
"""

from pygtk import require
require("2.0")
from gtk import TEXT_SEARCH_TEXT_ONLY

from feedback import update_status_info
from internationalization import msg0007, msg0008, msg0009, msg0010, msg0011
from internationalization import msg0012, msg0013, msg0014, msg0205, msg0206
from cursor import get_cursor_iter, get_current_line


def select_word(editor):
	"""
	Select a word, if any, at the cursor position

	@param editor: an editor instance
	@type editor: an editor object
	"""

	# Get the cursor's iterator.
	cursor_iter = get_cursor_iter(editor)

	# If the cursor is at the beginning of the word, create a copy of the
	# cursor's iterator. Move copy to the end of the word. Select the range
	# between cursor and copy.
	if cursor_iter.starts_word():

		end_word_iter = cursor_iter.copy()
		end_word_iter.forward_word_end()
		editor.text_buffer.select_range(cursor_iter, end_word_iter)
		update_status_info(editor, msg0007, "yes")

	# If the cursor is between a word, create two copies of the cursor's
	# iterator. Move the first copy to the beginning of the word. Then move the
	# second copy to the end of the word. Select the range between the beginning
	# and end of the word.
	elif cursor_iter.inside_word():

		start_word_iter = cursor_iter.copy()
		start_word_iter.backward_word_start()
		end_word_iter = cursor_iter.copy()
		end_word_iter.forward_word_end()
		editor.text_buffer.select_range(start_word_iter, end_word_iter)
		update_status_info(editor, msg0007, "yes")

	# If the cursor is at the end of a word, create a copy of the cursor's
	# iterator. Move the copy to the beginning of the word. Select the range
	# between the copy and the cursor.
	elif cursor_iter.ends_word():

		start_word_iter = cursor_iter.copy()
		start_word_iter.backward_word_start()
		editor.text_buffer.select_range(start_word_iter, cursor_iter)
		update_status_info(editor, msg0007, "yes")

	else:

		# Send feedback to the statusbar when no words can be found
		update_status_info(editor, msg0008, "no")

	return


def select_line(editor):
	"""
	Select a line, if any, at the cursor position

	@param editor: an editor instance
	@type editor: an editor object
	"""

	cursor_line = get_current_line(editor)
	start_iter = editor.text_buffer.get_iter_at_line(cursor_line)

	if not start_iter.get_chars_in_line():

		# msg0009 is a string eligible for internationalization and localization.
		# It identifies the string "No text to select in line %d." See the
		# module internationalization.py for more information.
		message = msg0009 % (cursor_line + 1)

		# Feedback to the user via the statusbar
		update_status_info(editor, message, "no")

		return

	end_iter = start_iter.copy()
	end_iter.forward_to_line_end()
	editor.text_buffer.select_range(start_iter, end_iter)

	# msg0010 is a string eligible for internationalization and localization.
	# It identifies the string "Selected in line %d." See the module
	# internationalization.py for more information.
	message = msg0010 % (cursor_line + 1)

	# Feedback to the user via the statusbar
	update_status_info(editor, message, "yes")

	return


def select_sentence(editor):
	"""
	Select a sentence, if any, at the cursor position

	@param editor: an editor instance
	@type editor: an editor object
	"""

	cursor_iter = get_cursor_iter(editor)

	if cursor_iter.starts_sentence():

		start_iter = cursor_iter.copy()
		end_iter = cursor_iter.copy()
		end_iter.forward_sentence_end()
		editor.text_buffer.select_range(start_iter, end_iter)
		update_status_info(editor, msg0013, "yes")

	elif cursor_iter.inside_sentence():

		start_iter = cursor_iter.copy()
		end_iter = cursor_iter.copy()
		start_iter.backward_sentence_start()
		end_iter.forward_sentence_end()
		editor.text_buffer.select_range(start_iter, end_iter)
		update_status_info(editor, msg0013, "yes")

	elif cursor_iter.ends_sentence():

		start_iter = cursor_iter.copy()
		end_iter = cursor_iter.copy()
		start_iter.backward_sentence_start()
		editor.text_buffer.select_range(start_iter, end_iter)
		update_status_info(editor, msg0013, "yes")

	else:

		update_status_info(editor, msg0014, "no")

	return


def select_paragraph(editor):
	"""
	Select a paragraph, if any, at the cursor position

	@param editor: an editor instance
	@type editor: an editor object
	"""

	cursor_iter = get_cursor_iter(editor)

	if cursor_iter.starts_sentence() or cursor_iter.inside_sentence() or \
		cursor_iter.ends_sentence():

		start_iter = cursor_iter.copy()
		end_iter = cursor_iter.copy()

		while True:

			if start_iter.is_start():

				break

			start_iter.backward_char()

			if start_iter.get_char() == "\n" and start_iter.starts_line():

				start_iter.forward_char()

				break

		while True:

			if end_iter.is_end():

				break

			end_iter.forward_char()

			if end_iter.get_char() == "\n" and end_iter.starts_line():

				end_iter.backward_char()

				break

		editor.text_buffer.select_range(start_iter, end_iter)
		update_status_info(editor, msg0011, "yes")

		return

	else:

		update_status_info(editor, msg0012, "no")

	return


def search_and_select(editor, start_iter, cursor_iter, character):
	"""
	Search for closing brackets and select characters between opening and
	closing brackets.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	@param cursor_iter: Position of the opening bracket.
	@type cursor_iter: A gtk.TextIter object.

	@param character: Type of closing bracket to search for.
	@type character: A string object.

	"""

	end_iter = cursor_iter.copy()
	result = end_iter.forward_search(character, TEXT_SEARCH_TEXT_ONLY)

	if result:

		editor.text_buffer.select_range(start_iter, result[0])
		update_status_info(editor, msg0206, "yes")

	else:

		update_status_info(editor, msg0205, "no")

	return


def select_bracket(editor):
	"""
	Select all words inside double quotes, square and round brackets as well as
	parenthesis.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	"""

	cursor_iter = get_cursor_iter(editor)
	start_iter = cursor_iter.copy()

	while True:

		character = start_iter.get_char()

		if character in ("[", "(", "{"):

			start_iter.forward_char()
			break

		if start_iter.is_start():

			character = None
			break

		start_iter.backward_char()


	if character == "[":

		character = "]"
		search_and_select(editor, start_iter, cursor_iter, character)

	elif character == "(":

		character = ")"
		search_and_select(editor, start_iter, cursor_iter, character)

	elif character == "{":

		character = "}"
		search_and_select(editor, start_iter, cursor_iter, character)

	else:

		update_status_info(editor, msg0205, "no")

	return

