# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module contains a class that provides methods to display a color editor.
The color editor is a dialog interface that allows users to customize the
foreground, background and syntax colors of the text editor.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from gc import collect
from gconf import VALUE_STRING

from gobject import TYPE_OBJECT
from gtk import HBox, VBox, Label, CheckButton, STOCK_CLOSE, ColorButton
from gtk import WIN_POS_CENTER_ON_PARENT, Dialog, RESPONSE_CLOSE, TreeView
from gtk import CellRendererText, TreeViewColumn, TREE_VIEW_COLUMN_AUTOSIZE
from gtk import TreeStore, Alignment, STOCK_UNDO, Button, keysyms
from gtk.gdk import WINDOW_TYPE_HINT_DIALOG, GRAVITY_CENTER, color_parse
from gtksourceview import SourceLanguagesManager
from scribes_gconf import get_gconf_client
from internationalization import msg0187, msg0296, msg0297, msg0298, msg0299
from internationalization import msg0300, msg0301, msg0302, msg0303, msg0304
from internationalization import msg0321
from feedback import update_status_info
from utility import process_color, split_color, create_button
from scrollwindow import create_scroll_window
from escape import escape_string, unescape_string
from tooltips import create_tooltips, theme_check_button_tip
from tooltips import foreground_button_tip, background_button_tip
from tooltips import syntax_button_tip, bold_check_button_tip
from tooltips import italic_check_button_tip, reset_button_tip


class ColorEditor(object):
	"""
	Blueprint for a dialog that allows users to customize the foreground,
	background and syntax colors of the text editor's buffer.

	The ColorEditor class provides all the methods necessary to create and
	display a color editor dialog. Technically, the dialog displayed is only a
	front end to a GConf database. Thus, the primarily function of the class is
	to provide a robust mechanism and simple interface to read from and write to
	the GConf database.

	"""

	def __init__(self, editor):
		"""
		Initialize the ColorEditor object.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param editor: Reference to the editor.
		@type editor: An editor object.

		"""

		self._initialize_resuable_components(editor)
		self._set_dialog_window_properties()
		self._create_dialog_widgets()
		self._arrange_dialog_widgets()
		self.show_syntax_editor()


	def _initialize_resuable_components(self, editor):
		"""
		Initialize objects that will be used repeatedly only once.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param editor: Reference to the editor.
		@type editor: An editor object.

		"""

		self.client = get_gconf_client()
		self.editor = editor
		self.dialog = Dialog()
		self.dialog.connect("key-press-event", self.dialog_key_cb)
		self.dialog.connect("map-event", self.dialog_map_cb)
		self.tips = create_tooltips()
		self.language_list = SourceLanguagesManager().get_available_languages()
		self.language_names = [name.get_name() for name in self.language_list]
		self.scrollwindow = create_scroll_window()

		return


	def _set_dialog_window_properties(self):
		"""
		Define the dialog window properties.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		"""

		# Calculate resolution independent width and height of the preferences
		# dialog window.
		width, height = self.editor.calculate_resolution_independence(2, 2.133333333)
		dialog = self.dialog
		dialog.set_property("title", msg0296)
		dialog.set_property("role", "syntax color editor")
		dialog.set_property("type-hint", WINDOW_TYPE_HINT_DIALOG)
		dialog.set_property("modal", True)
		dialog.set_property("width-request", width)
		dialog.set_property("height-request", height)
		dialog.set_property("skip-taskbar-hint", True)
		dialog.set_property("skip-pager-hint", True)
		dialog.set_property("icon-name", "gnome-settings")
		dialog.set_property("window-position", WIN_POS_CENTER_ON_PARENT)
		dialog.set_property("gravity", GRAVITY_CENTER)
		dialog.set_property("resizable", True)
		dialog.set_property("has-separator", False)
		dialog.set_property("border-width", 12)
		dialog.vbox.set_spacing(12)
		dialog.add_button(STOCK_CLOSE, RESPONSE_CLOSE)
		dialog.set_transient_for(self.editor)

		return


	def _create_dialog_widgets(self):
		"""
		Create all widgets displayed in the syntax editor dialog.

		Labels, check, color and normal buttons as well as the tree view used
		in the syntax editor dialog are initialized and constructed by this
		function.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		"""

		self.label = self._create_syntax_dialog_labels()
		self.theme_check_button = self._create_theme_check_button(msg0297)
		self.foreground_color_button = self._create_foreground_color_button()
		self.background_color_button = self._create_background_color_button()
		self.elements_view = self._create_element_view()
		self.bold_check_button = self._create_bold_check_button(msg0302)
		self.italic_check_button = self._create_italic_check_button(msg0303)
		self.syntax_color_button = self._create_syntax_color_button()
		self.reset_button = self._create_reset_button(msg0304)

		return


	def _arrange_dialog_widgets(self):
		"""
		Pack widgets appropriately into the syntax editor dialog.

		This function controls the appearance and position of labels, buttons
		and views within the syntax editor.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		"""

		# Attach the "use default theme" check button to the dialog.
		self.dialog.vbox.pack_start(self.theme_check_button, False, False, 0)

		# Create a container to hold foreground and background labels and
		# color buttons.
		hbox = HBox(False, 12)

		# Create a container to hold foreground labels and color button.
		mini_hbox = HBox(False, 12)
		hbox.pack_start(mini_hbox, True, True, 0)
		self.dialog.vbox.pack_start(hbox, False, False, 0)
		self.label[0].set_mnemonic_widget(self.foreground_color_button)
		mini_hbox.pack_start(self.label[0], False, True, 0)
		mini_hbox.pack_start(self.foreground_color_button, False, True, 0)

		# Create a container to hold backgroung label and color button.
		mini_hbox = HBox(False, 12)
		hbox.pack_start(mini_hbox, True, True, 0)
		self.label[1].set_mnemonic_widget(self.background_color_button)
		mini_hbox.pack_start(self.label[1], False, True, 0)
		mini_hbox.pack_start(self.background_color_button, False, True, 0)

		# Create a container to hold syntax element tree view and attribute
		# area.
		hbox = HBox(False, 12)
		self.dialog.vbox.pack_start(hbox, True, True, 0)
		self.scrollwindow.add(self.elements_view)
		hbox.pack_start(self.scrollwindow, True, True, 0)

		# Create a container to hold the attribute widgets of each element in
		# the dialog's treeview.
		alignment = Alignment(yalign=0.8)
		vbox = VBox()
		alignment.add(vbox)
		hbox.pack_start(alignment, False, False, 5)
		mini_hbox = HBox()
		vbox.pack_start(mini_hbox, False, False, 5)

		# Create a container to hold syntax color label and button.
		self.label[2].set_mnemonic_widget(self.syntax_color_button)
		mini_hbox.pack_start(self.label[2], False, True, 5)
		mini_hbox.pack_start(self.syntax_color_button, False, True, 5)

		# Attach the "Bold" and "Italic" check button as well as the "reset to
		# default" button to attribute container.
		vbox.pack_start(self.bold_check_button, False, False, 5)
		vbox.pack_start(self.italic_check_button, False, False, 5)
		vbox.pack_start(self.reset_button, False, False, 40)

		return


	def show_syntax_editor(self):
		"""
		Display the syntax editor.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param editor: Reference to the editor.
		@type editor: An editor object.

		"""

		self.view.grab_add()
		self.view.grab_focus()
		self.dialog.show_all()
		update_status_info(self.editor, msg0321, "color", False)
		response = self.dialog.run()

		if response:

			self.dialog.hide_all()
			self.dialog.destroy()
			update_status_info(self.editor, msg0187, "info")
			del self
			collect()

		return


################################################################################
#
#					Color Dialog Callbacks
#
################################################################################


	def dialog_key_cb(self, dialog, event):
		"""
		Callback to the dialog's "key-press-event"

		The "key-press-event" signal is emitted when the user presses a key on
		the	keyboard.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.

		@param dialog: The color dialog.
		@type dialog: A Dialog object.

		@param key_press: Event when the Escape key is pressed.
		@type key_press: An Event object.

		"""

		if event.keyval == keysyms.Escape:

			self.dialog.response(RESPONSE_CLOSE)

		return False


	def dialog_map_cb(self, dialog, event):
		"""
		Handles callback when the color dialog is mapped to screen.

		This function selects an item in the "Language Elements" treeview. The
		item selected corresponds to the language of the source code being
		edited. If the document is not a source code file. The first item in the
		treeview is selected.

		@param self: Reference to the ColorEditor instance.
		@type self: A ColorEditor object.

		@param dialog: The color dialog.
		@type dialog: A Dialog object.

		@param event: An event that maps the color dialog to the display.
		@type event: A Event object.

		@return: True to prevent propagation of the mapped event to parent
			wigdets
		@rtype: A Boolean object.

		"""

		model = self.view.get_model()
		iterator = model.get_iter_first()

		if self.editor.language:

			while not model.get_value(iterator, 0) == self.editor.language:

				iterator = model.iter_next(iterator)

			if iterator:

				path = model.get_path(iterator)
				column = self.view.get_column(0)
				self.view.set_cursor_on_cell(path, column)
				self.view.scroll_to_cell(path, column, True, 0.5)

		else:

			column = self.view.get_column(0)
			self.view.set_cursor_on_cell(0, column)
			self.view.scroll_to_cell(0, column, True, 0.5)

		return True


################################################################################
#
#						Labels
#
################################################################################


	def _create_syntax_dialog_labels(self):
		"""
		Create labels for the syntax dialog editor.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@return: A list of labels for the syntax editor.
		@rtype: A list object.

		"""

		string_list =[msg0298, msg0299, msg0301]
		label_list = []

		# Convert strings to labels. Append the labels to a list.
		for string in string_list:

			label = Label(string)
			label.set_use_underline(True)
			label_list.append(label)

		return label_list


################################################################################
#
#						Theme Check Button
#
################################################################################


	def _create_theme_check_button(self, string):
		"""
		Create the "use default theme" check button on the syntax color dialog
		editor.

		The check button is used to toggle on or off the theme colors used to
		render but foreground and background colors of the text editor's
		buffer. The default value of the check button is fetched from the
		GConf database.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param string: The string, "Use default theme."
		@type string: A string object.

		"""

		active = self.client.get_bool("/apps/scribes/use_theme_colors")
		button = self._create_check_button(string)
		button.set_active(active)
		self.tips.set_tip(button, theme_check_button_tip)
		button.connect("toggled", self.theme_check_button_cb)

		return button


	def theme_check_button_cb(self, button):
		"""
		Handles callback when the "use default theme" check button on the syntax
		color dialog editor is toggled.

		The function retrieves the state of the check button. The state of the
		check button can either be True or False. True when the button is
		toggled on and False otherwise. The state of the button is sent to the
		GConf deamon and registered in the GConf database.

		The text editor is notified of changes via the GConf daemon. It proceeds
		to change the background and foreground color of the text editor
		depending on the value of the key, "/apps/scribes/use_theme_colors," in
		the GConf database.

		If the value of the key is True, then the editor uses the foreground and
		background color specified by the GTK+ theme. Otherwise, the editor
		uses the foreground and background color specified by the user.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param button: Check button to toggle on or off default background or
			foreground colors.
		@type button: A gtk.CheckButton object.

		@return: Do not propagate event to children widgets.
		@rtype: A boolean object.

		"""

		active_state = button.get_active()
		self.client.set_bool("/apps/scribes/use_theme_colors", active_state)
		self._set_widget_sensitivity(self.foreground_color_button,
										not active_state)
		self._set_widget_sensitivity(self.background_color_button,
										not active_state)

		return True


################################################################################
#
#						Bold Check Button
#
################################################################################


	def _create_bold_check_button(self, string):
		"""
		Create the "Bold" check button on the syntax editor dialog.

		The "Bold" check button is used to determine the font weight of elements
		for a particular source code language.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param string: Label for the "Bold" check button on the dialog.
		@type string: A string object.

		@return: A check button with the label "Bold."
		@rtype: A gtk.CheckButton object.

		"""

		button = CheckButton(string)# self._create_check_button(string)
		self._set_widget_sensitivity(button, False)
		self.tips.set_tip(button, bold_check_button_tip)
		self.bold_id = button.connect("toggled", self.bold_check_button_cb)

		return button


	def bold_check_button_cb(self, button):
		"""
		Handles callback when the "Bold" check button is toggled.

		When the "Bold" check button is toggled, enter the information into
		the GConf database. The text editor is notified of the changes made and
		proceeds to operate on the value.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param button: The "Bold" check button.
		@type button: A gtk.CheckButton

		@return: Do not propagate event to children widgets.
		@rtype: A boolean object.

		"""

		is_bold = button.get_active()
		is_italic = self.italic_check_button.get_active()
		button_color = split_color(self.syntax_color_button.get_color())
		selection = self.view.get_selection()
		model, iterator = selection.get_selected()
		parent_iter = model.iter_parent(iterator)
		language = escape_string(model.get_value(parent_iter, 0))
		tag_name = escape_string(model.get_value(iterator, 0))
		gconf_key = self._get_gconf_key(language, tag_name)
		gconf_value = list(button_color) + [str(is_bold), str(is_italic)]
		self.client.set_list(gconf_key, VALUE_STRING, gconf_value)
		self._set_widget_sensitivity(self.reset_button, True)

		return True


################################################################################
#
#						Italic Check Button
#
################################################################################


	def _create_italic_check_button(self, string):
		"""
		Create the "Italic" check button on the syntax editor dialog.

		The "Italic" check button is used to determine the slant of
		elements for a particular source code language.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param string: Label for the "Italic" check button on the dialog.
		@type string: A string object.

		@return: A check button with the label "Italic."
		@rtype: A gtk.CheckButton object.

		"""

		button = CheckButton(string)
		self._set_widget_sensitivity(button, False)
		self.tips.set_tip(button, italic_check_button_tip)
		self.italic_id = button.connect("toggled", self.italic_check_button_cb)

		return button


	def italic_check_button_cb(self, button):
		"""
		Handles callback when the "Italic" check button is toggled.

		When the "Italic" check button is toggled, enter the information into
		the GConf database. The text editor is notified of the changes made and
		proceeds to operate on the value.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param button: The "Italic" check button.
		@type button: A gtk.CheckButton

		@return: Do not propagate event to children widgets.
		@rtype: A boolean object.

		"""

		is_italic = button.get_active()
		is_bold = self.bold_check_button.get_active()
		button_color = split_color(self.syntax_color_button.get_color())
		selection = self.view.get_selection()
		model, iterator = selection.get_selected()
		parent_iter = model.iter_parent(iterator)
		language = escape_string(model.get_value(parent_iter, 0))
		tag_name = escape_string(model.get_value(iterator, 0))
		gconf_key = self._get_gconf_key(language, tag_name)
		gconf_value = list(button_color) + [str(is_bold), str(is_italic)]
		self.client.set_list(gconf_key, VALUE_STRING, gconf_value)
		self._set_widget_sensitivity(self.reset_button, True)

		return True


################################################################################
#
#						Foreground Color Button
#
################################################################################


	def _create_foreground_color_button(self):
		"""
		Create a color button to change the color of text in the text editor's
		buffer.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		"""

		button = ColorButton()
		self.tips.set_tip(button, foreground_button_tip)
		color = self.client.get_list("/apps/scribes/foreground_color",
										VALUE_STRING)
		foreground_color = process_color(color)
		button.set_color(foreground_color)
		button.connect("color-set", self.foreground_color_button_cb)
		value = self.client.get_bool("/apps/scribes/use_theme_colors")
		self._set_widget_sensitivity(button, not value)

		return button


	def foreground_color_button_cb(self, button):
		"""
		Handles callback when the color of the foreground color button changes.

		The function retrieves the color changes that occured. Processes the
		color to extract the red, green, blue and pixel values of the color. And
		then it stores these values in the GConf database via the GConf daemon.

		The text editor is notified of these changes via the GConf daemon. It
		then proceeds to change the color of text in the text editor based on
		the red, gree, blue and pixel value stored in the GConf database.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param button: Color button to change the color of the text editor's
			text.
		@type button: A gtk.ColorButton object.

		@return: Do not propagate event to children widgets.
		@rtype: A boolean object.

		"""

		color = button.get_color()
		red, green, blue, pixel = split_color(color)
		gconf_value = [red, green, blue, pixel]
		self.client.set_list("/apps/scribes/foreground_color", VALUE_STRING,
								gconf_value)

		return True


################################################################################
#
#						Background Color Button
#
################################################################################


	def _create_background_color_button(self):
		"""
		Create a color button to change the color of the text editor's buffer.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		"""

		button = ColorButton()
		self.tips.set_tip(button, background_button_tip)
		color = self.client.get_list("/apps/scribes/background_color",
										VALUE_STRING)
		background_color = process_color(color)
		button.set_color(background_color)
		button.connect("color-set", self.background_color_button_cb)
		value = self.client.get_bool("/apps/scribes/use_theme_colors")
		self._set_widget_sensitivity(button, not value)

		return button


	def background_color_button_cb(self, button):
		"""
		Handles callback when the color of the background color button changes.

		The function retrieves the color changes that occured. Processes the
		color to extract the red, green, blue and pixel values of the color. And
		then it stores these values in the GConf database via the GConf daemon.

		The text editor is notified of these changes via the GConf daemon. It
		then proceeds to change the color of the text editor's buffer based on
		the red, gree, blue and pixel value stored in the GConf database.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param button: Color button to change the color of the text editor's
			buffer.
		@type button: A gtk.ColorButton object.

		@return: Do not propagate event to children widgets.
		@rtype: A boolean object.

		"""

		color = button.get_color()
		red, green, blue, pixel = split_color(color)
		gconf_value = [red, green, blue, pixel]
		self.client.set_list("/apps/scribes/background_color", VALUE_STRING,
								gconf_value)

		return True


################################################################################
#
#						Syntax Color Button
#
################################################################################


	def _create_syntax_color_button(self):
		"""
		Create a color button to change the color of syntax elements in
		language source files.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		"""

		button = ColorButton()
		self.tips.set_tip(button, syntax_button_tip)
		self._set_widget_sensitivity(button, False)
		button.connect("color-set", self.syntax_color_button_cb)

		return button


	def syntax_color_button_cb(self, button):
		"""
		Handles callback when the color of the background color button changes.

		The function retrieves the color changes that occured. Processes the
		color to extract the red, green, blue and pixel values of the color. And
		then it stores these values in the GConf database via the GConf daemon.

		The text editor is notified of these changes via the GConf daemon. It
		then proceeds to change the color of the text editor's buffer based on
		the red, gree, blue and pixel value stored in the GConf database.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param button: Color button to change the color of the text editor's
			buffer.
		@type button: A gtk.ColorButton object.

		@return: Do not propagate event to children widgets.
		@rtype: A boolean object.

		"""

		color = button.get_color()
		red, green, blue, pixel = split_color(color)
		is_bold = self._get_bold_value()
		is_italic = self._get_italic_value()
		gconf_value = [red, green, blue, pixel, str(is_bold), str(is_italic)]
		selection = self.view.get_selection()
		model, iterator = selection.get_selected()
		parent_iter = model.iter_parent(iterator)
		language = escape_string(model.get_value(parent_iter, 0))
		tag_name = escape_string(model.get_value(iterator, 0))
		gconf_key = self._get_gconf_key(language, tag_name)
		self.client.set_list(gconf_key, VALUE_STRING, gconf_value)
		self._set_widget_sensitivity(self.reset_button, True)

		return True


################################################################################
#
#						Reset to Default Button
#
################################################################################


	def _create_reset_button(self, string):
		"""
		Reset element value to default.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param self: Label for the button titled.
		@type self: A gtk.Button object.

		@return: A button to reset syntax colors and styles to their default
			value.
		@rtype: A gtk.Button object.

		"""

		button = Button()
		self.tips.set_tip(button, reset_button_tip)
		box = create_button(STOCK_UNDO, string)
		button.add(box)
		self._set_widget_sensitivity(button, False)
		button.connect("clicked", self.reset_button_cb)

		return button


	def reset_button_cb(self, button):
		"""
		Reset the language's element to it's default color and style specified
		by gtksourceview.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param button: The "Reset to Default" button.
		@type button: A gtk.Button object.

		@return: Do not propagate event to children widgets.
		@rtype: A boolean object.

		"""

		# Get language and tag_name for GConf key.
		selection = self.view.get_selection()
		model, iterator = selection.get_selected()
		parent_iter = model.iter_parent(iterator)
		language = escape_string(model.get_value(parent_iter, 0))
		tag_name = escape_string(model.get_value(iterator, 0))
		gconf_key = self._get_gconf_key(language, tag_name)

		# Unset values for language key.
		self.client.unset(gconf_key)
		self._set_widget_sensitivity(self.reset_button, False)
		self.syntax_color_button.set_color(self._get_button_color())

		# Block the italic and bold check button toggle signal to prevent
		# setting the values in the GConf database.
		self.italic_check_button.handler_block(self.italic_id)
		self.bold_check_button.handler_block(self.bold_id)

		# Get the toggle state of the italic and bold check buttons.
		self.italic_check_button.set_active(self._get_italic_value())
		self.bold_check_button.set_active(self._get_bold_value())

		# Unblock toggle signal blocked in the afore-mentioned statement.
		self.italic_check_button.handler_unblock(self.italic_id)
		self.bold_check_button.handler_unblock(self.bold_id)

		return True


################################################################################
#
#						Syntax Editor TreeView
#
################################################################################


	def _create_element_view(self):
		"""
		Create the view containing languages and elements.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@return: A treeview populated with source languages and their elements.
		@rtype: A gtk.TreeView object.

		"""

		view = TreeView()
		self.view = view
		renderer = CellRendererText()
		column = TreeViewColumn(msg0300, renderer, text=0)
		view.append_column(column)
		column.set_expand(True)
		column.set_sizing(TREE_VIEW_COLUMN_AUTOSIZE)
		view.set_model(self._populate_element_view())
		view.columns_autosize()
		view.set_enable_search(True)
		view.set_search_column(0)
		view.set_property("rules-hint", True)
		view.connect("cursor-changed", self.cursor_changed_cb)
		view.connect("row-expanded", self.row_expanded_cb)
		view.connect("row-collapsed", self.row_collapsed_cb)

		return view


	def cursor_changed_cb(self, view):
		"""
		Handles callback when rows in the syntax editor's tree view are
		clicked or selected.

		Clicking or selecting the syntax editor's tree view's rows displays
		the text color and attribute of elements for a given language.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param view: The syntax color editor's tree view.
		@type view: A gtk.TreeView.

		@return: Do not propagate event to children widgets.
		@rtype: A boolean object.

		"""

		# Get the tree view's model as well as iterators to the parent and
		# child rows if available.
		selection = view.get_selection()
		model, iterator = selection.get_selected()
		button = self.syntax_color_button

		try:

			has_child = model.iter_has_child(iterator)

		except TypeError:

			return True

		# Disable the syntax color button, bold and italic check button if a
		# parent row is clicked or selected. Otherwise, enable the buttons.
		if not has_child:

			# Enable the color, bold and italic buttons.
			self._set_element_attribute_area_sensitivity(True)

			# Get and set the value of the color button.
			color = self._get_button_color()
			button.set_color(color)

			# Block the italic and bold check button toggle signal to prevent
			# setting the values in the GConf database.
			self.italic_check_button.handler_block(self.italic_id)
			self.bold_check_button.handler_block(self.bold_id)

			# Get the toggle state of the italic and bold check buttons.
			self.italic_check_button.set_active(self._get_italic_value())
			self.bold_check_button.set_active(self._get_bold_value())

			# Unblock toggle signal blocked in the afore-mentioned statement.
			self.italic_check_button.handler_unblock(self.italic_id)
			self.bold_check_button.handler_unblock(self.bold_id)

		else:

			# Disable the color, bold and italic buttons.
			self._set_element_attribute_area_sensitivity(False)

		return True


	def row_expanded_cb(self, view, iterator, path):
		"""
		Handles callback when the treeview's expander is clicked.

		This function generates the elements, or children nodes, for each
		language, just before the row is expanded.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param view: The view containing languages and elements.
		@type view: A gtk.TreeView object.

		@param iterator: An iterator pointing to the row that has just been
			clicked for expansion.
		@type iterator: A gtk.TreeIter object.

		@view path: Path to the row that needs to be expanded.
		@type path: A path object.

		@return: Do not propagate event to children widgets.
		@rtype: A boolean object.

		"""

		self._set_element_attribute_area_sensitivity(False)
		self._populate_child_node(iterator)

		return True


	def row_collapsed_cb(self, view, iterator, path):
		"""
		Handles callback when parent's children rows are collapsed.

		This function disables the color button, bold and italic check buttons.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param view: The syntax editor's tree view.
		@type view: A gtk.TreeView object.

		@param iterator: The position of the selected row.
		@type iterator: A gtk.TreeIter object.

		@param path: The path to the selected row.
		@type path: A gtk.Path object.

		@return: Do not propagate event to children widgets.
		@rtype: A boolean object.

		"""

		self._set_element_attribute_area_sensitivity(False)

		return True


	def _populate_element_view(self):
		"""
		Populate the elements view with languages.

		To improve launch time, the view is populated only with source
		languages. Language elements are not populated. Language elements will
		be populated when the user clicks on the treeview expander.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		"""

		# The model has two columns. One contains the language name. The other
		# contains the SourceLanguage objects for the language.
		model = TreeStore(str, TYPE_OBJECT)

		# Populate the model's parent nodes with language names and
		# SourceLanguage objects.
		for name in self.language_list:

			parent_iter = model.append(None)
			model.set(parent_iter, 0, name.get_name())
			model.set(parent_iter, 1, name)

			#Leave the children nodes blank. They will be populated later.
			for elements in [""]:

				child_iter = model.append(parent_iter)
				model.set(child_iter, 0, elements)

		return model


	def _populate_child_node(self, iterator):
		"""
		Populate the parent row with children rows when the expander is clicked.

		The children rows are populated with language elements associated with
		the language displayed in the parent row.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.


		@param iterator: Iterator of the row selected
		@type iterator: A gtk.TreeIter

		"""

		# Get the elements tree view's model.
		model = self.view.get_model()

		# Get the iterator of the first child of the row.
		child_iter = model.iter_nth_child(iterator, 0)

		# If the child row contains a blank line remove it. If the child row
		# does not contain a blank line, there is no need to populate the
		# child row because it has already been done.
		if not model.get_value(child_iter, 0):

			model.remove(child_iter)

		else:

			return

		language = model.get_value(iterator, 1)

		# Populate the parent row with children rows containing elements, or
		# syntax tags, for the language the parent row represents.
		for elements in language.get_tags():

			child_iter = model.append(iterator)
			model.set(child_iter, 0, unescape_string(elements.get_id()))

		return


################################################################################
#
#						Helper Methods
#
################################################################################


	def _create_check_button(self, string):
		"""
		Create a check button and define its properties

		@param string: String indicating the action associated with the check
			button.
		@type string: A string object.

		@return: A check button with a label and mnemonic accelarator.
		@rtype: A CheckButton object.

		"""

		button = CheckButton()
		button.set_property("use-underline", True)
		button.set_property("label", string)

		return button


	def _set_widget_sensitivity(self, widget, value=True):
		"""
		Enable or disable the widget depending on the boolean value.

		When the boolean value is True, the widget is enable and thus can
		recieve user input. Otherwise, the widget is disabled.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param widget: For this class mainly check, color and normal buttons.
		@type widget: A gtk.Widget object

		@param value: Either True or False. True to enable, False to disable.
		@type value: A boolean object.

		"""

		widget.set_property("sensitive", value)

		return


	def _set_element_attribute_area_sensitivity(self, value=True):
		"""
		Disable or enable the syntax color button, as well as the italic and
		bold check buttons.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param value: True to enable widgets. False otherwise.
		@type value: A boolean object.

		"""

		if not value:

			self.syntax_color_button.set_color(color_parse("black"))
			self.italic_check_button.set_active(False)
			self.bold_check_button.set_active(False)

		self._set_widget_sensitivity(self.syntax_color_button, value)
		self._set_widget_sensitivity(self.bold_check_button, value)
		self._set_widget_sensitivity(self.italic_check_button, value)
		self._set_widget_sensitivity(self.reset_button, value)

		return


	def _get_button_color(self):
		u"""
		Get the color of the syntax color button.

		The function checks the GConf database to see if the value of the color
		is stored there. The it uses gtksourceview's default value.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@return: The color on the syntax color button.
		@rtype: A gtk.gdk.Color object.

		"""

		selection = self.view.get_selection()
		model, iterator = selection.get_selected()
		parent_iter = model.iter_parent(iterator)
		language = escape_string(model.get_value(parent_iter, 0))
		tag_name = escape_string(model.get_value(iterator, 0))
		gconf_key = self._get_gconf_key(language, tag_name)

		value = self.client.get_list(gconf_key, VALUE_STRING)

		if value:

			color = process_color(value)
			self._set_widget_sensitivity(self.reset_button, True)

		else:

			self._set_widget_sensitivity(self.reset_button, False)
			language = model.get_value(parent_iter, 1)
			tag_style = language.get_tag_style(tag_name)
			color = tag_style.foreground

		return color


	def _get_bold_value(self):
		u"""
		Get the toggle state of the bold check button.

		The function checks the GConf database to see if the value of the toggle
		state is stored there. If it is not,  gtksourceview's default value is
		used instead.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@return: The toggle state of the "Bold" check button.
		@rtype: A boolean object.

		"""

		selection = self.view.get_selection()
		model, iterator = selection.get_selected()
		parent_iter = model.iter_parent(iterator)
		language = escape_string(model.get_value(parent_iter, 0))
		tag_name = escape_string(model.get_value(iterator, 0))
		gconf_key = self._get_gconf_key(language, tag_name)
		value = self.client.get_list(gconf_key, VALUE_STRING)

		if value:

			if value[4] == "True":

				value = True

			else:

				value = False

			return value

		else:

			language = model.get_value(parent_iter, 1)
			tag_style = language.get_tag_style(tag_name)

			return tag_style.bold


	def _get_italic_value(self):
		"""
		Get the toggle state of the italic check button.

		The function checks the GConf database to see if the value of the toggle
		state is stored there. If it is not,  gtksourceview's default value is
		used instead.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@return: The toggle state of the "Italic" check button.
		@rtype: A boolean object.

		"""

		selection = self.view.get_selection()
		model, iterator = selection.get_selected()
		parent_iter = model.iter_parent(iterator)
		language = escape_string(model.get_value(parent_iter, 0))
		tag_name = escape_string(model.get_value(iterator, 0))
		gconf_key = self._get_gconf_key(language, tag_name)
		value = self.client.get_list(gconf_key, VALUE_STRING)

		if value:

			if value[5] == "True":

				value = True

			else:

				value = False

			return value

		else:

			language = model.get_value(parent_iter, 1)
			tag_style = language.get_tag_style(tag_name)

			return tag_style.italic


	def _get_gconf_key(self, language, tag_name):

		if language.startswith("gettext"):

			language = "gettext-translation"
			gconf_key = "/apps/scribes/syntax/" + language + "/" + tag_name

		else:

			gconf_key = "/apps/scribes/syntax/" + language + "/" + tag_name

		return gconf_key

