# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
This module contains a class that provides methods to display a template editor.
The template editor is a dialog that enables users to add templates.
A template is a pre-configured string that can be inserted into the
editor. The insertion is triggered by typing special strings into the editor's
buffer and pressing the tab key.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from gc import collect
from re import findall

from gtk import Entry, Dialog, HBox, WIN_POS_CENTER_ON_PARENT, Alignment
from gtk import STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK, Label
from gtk import keysyms, WRAP_WORD, WRAP_NONE, STATE_NORMAL
from gtk.gdk import KEY_PRESS_MASK, GRAVITY_CENTER, WINDOW_TYPE_HINT_DIALOG
from gtksourceview import SourceView, SourceBuffer, SourceLanguagesManager

from scrollwindow import create_scroll_window
from templatedatabase import add_entry, open_template_database
from templatedatabase import close_template_database
from error import error_dialog
from textbuffer import create_text_buffer
from utility import process_color, response as uresponse
from keyevents import filter_key_events
from templates import generate_generic_templates, generate_language_templates
from internationalization import msg0309, msg0310, msg0311, msg0312


class TemplateAdd(object):
	"""
	This class provides a blueprint for a dialog that allows users to add new
	templates to the text editor. The class is instantiated when the "Add"
	button on the template editor is clicked. The dialog contains a name entry,
	a description entry and a mini editor.

	The name entry allows users to define template keys. The description entry
	provides a means of describing the template keys or the purpose of the
	template. The editor allows users to define templates.

	"""


	def __init__(self, templatedialog):
		"""
		Initialize the TemplateAdd object.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		@param editor: Reference to a TemplateDialog instance.
		@type editor: A TemplateDialog object.

		"""

		self._initialize_resuable_components(templatedialog)
		self._set_dialog_window_properties()
		self._create_dialog_widgets()
		self._arrange_dialog_widgets()
		self.show_template_editor()


	def _initialize_resuable_components(self, templatedialog):
		"""
		Initialize objects that will be used repeatedly only once.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		@param editor: Reference to the TemplateDialog instance.
		@type editor: An TemplateDialog object.

		"""

		self.templatedialog = templatedialog
		self.editor = templatedialog.editor
		self.dialog = Dialog()
		self.dialog.connect("key-press-event", self.key_press_event_cb)
		self.dialog.add_events(KEY_PRESS_MASK)
		self.language_list = SourceLanguagesManager().get_available_languages()

		return


	def _set_dialog_window_properties(self):
		"""
		Define the dialog window properties.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		"""

		# Calculate resolution independent width and height of the preferences
		# dialog window.
		width, height = self.editor.calculate_resolution_independence(2, 2.133333333)
		dialog = self.dialog
		dialog.set_property("title", msg0312)
		dialog.set_property("role", "template editor")
		dialog.set_property("type-hint", WINDOW_TYPE_HINT_DIALOG)
		dialog.set_property("modal", True)
		dialog.set_property("width-request", width)
		dialog.set_property("height-request", height)
		dialog.set_property("skip-taskbar-hint", True)
		dialog.set_property("skip-pager-hint", True)
		dialog.set_property("icon-name", "gnome-settings")
		dialog.set_property("window-position", WIN_POS_CENTER_ON_PARENT)
		dialog.set_property("gravity", GRAVITY_CENTER)
		dialog.set_property("resizable", True)
		dialog.set_property("has-separator", False)
		dialog.set_property("border-width", 12)
		dialog.vbox.set_spacing(12)
		dialog.action_area.set_spacing(6)
		dialog.add_button(STOCK_CANCEL, RESPONSE_CANCEL)
		dialog.add_button(STOCK_OK, RESPONSE_OK)
		dialog.set_transient_for(self.templatedialog.dialog)

		return


	def _create_dialog_widgets(self):
		"""
		Create all widgets displayed in the TemplateAdd dialog.

		@param self: Reference to the TemplateAdd.
		@type self: A TemplateAdd object.

		"""

		self.label = self._create_editor_labels()
		self.name_entry = self._create_name_entry()
		self.description_entry = self._create_description_entry()
		self.textview = self._create_textview()

		return


	def _arrange_dialog_widgets(self):
		"""
		Pack widgets appropriately into the TemplateAdd dialog.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		"""

		hbox = HBox(False, 12)
		alignment = Alignment(xalign=1.0, yalign=0.0, xscale=0.895, yscale=0.0)
		alignment.add(hbox)
		self.dialog.vbox.pack_start(alignment, False, False, 0)
		self.label[0].set_mnemonic_widget(self.name_entry)
		hbox.pack_start(self.label[0], False, False, 0)
		hbox.pack_start(self.name_entry, True, True, 0)

		hbox = HBox(False, 12)
		self.dialog.vbox.pack_start(hbox, False, False, 0)
		self.label[1].set_mnemonic_widget(self.description_entry)
		hbox.pack_start(self.label[1], False, False, 0)
		hbox.pack_start(self.description_entry, True, True, 0)

		hbox = HBox(False, 12)
		alignment = Alignment(xalign=1.0, yalign=0.0, xscale=0.969, yscale=1.0)
		alignment.add(hbox)
		self.dialog.vbox.pack_start(alignment, True, True, 0)
		self.label[2].set_mnemonic_widget(self.textview)
		alignment = Alignment(xalign=0.0)
		alignment.add(self.label[2])
		hbox.pack_start(alignment, False, False, 0)
		scrollwindow = create_scroll_window()
		scrollwindow.add(self.textview)
		hbox.pack_start(scrollwindow, True, True, 0)

		return


	def show_template_editor(self):
		"""
		Display the TemplateAdd dialog.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		"""

		self.dialog.show_all()
		response = self.dialog.run()

		if response == RESPONSE_OK:

			result = self._validate_entry()

			if result:

				name, description, template = self._get_dialog_values()
				self._update_template_database()
				self._update_template_dialog((name, description, template))
				generate_generic_templates(self.editor)
				generate_language_templates(self.editor)

		self.dialog.hide_all()
		self.dialog.destroy()
		del self
		collect()

		return


################################################################################
#
#						Callback to Close the dialog
#
################################################################################


	def key_press_event_cb(self, dialog, key_press):
		"""
		Callback to the dialog's "key-press-event"

		The "key-press-event" signal is emitted when the user presses a key on
		the	keyboard.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param dialog: The template editor.
		@type dialog: A TemplateDialog object.

		@param key_press: Event when the Escape key is pressed.
		@type key_press: An Event object.

		"""

		if key_press.keyval == keysyms.Escape:

			self.dialog.response(RESPONSE_CANCEL)

		return


################################################################################
#
#								Labels
#
################################################################################


	def _create_editor_labels(self):
		"""
		Create labels for the template editor.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		"""

		strings = [msg0309, msg0310, msg0311]
		label_list = []

		# Convert strings to labels.
		for item in strings:

			label = Label(item)
			label.set_use_underline(True)
			label_list.append(label)

		return label_list


################################################################################
#
#							Name Entry
#
################################################################################


	def _create_name_entry(self):
		"""
		Create an entry to hold the template key.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		@return: An entry to hold a key.
		@rtype: A gtk.Entry object.

		"""

		entry = Entry()
		entry.connect("changed", self.name_entry_cb)

		return entry


	def name_entry_cb(self, entry):
		"""
		Handles callback when the contents of the entry changes

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		"""

		value = len(entry.get_text())

		if value:

			self.description_entry.set_property("sensitive", True)
			self.textview.set_property("sensitive", True)

		else:

			self.description_entry.set_property("sensitive", False)
			self.textview.set_property("sensitive", False)

		return True


################################################################################
#
#							Description Entry
#
################################################################################


	def _create_description_entry(self):
		"""
		Create an entry to hold the description for the template key.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		@return: An entry to hold a key.
		@rtype: A gtk.Entry object.

		"""

		entry = Entry()
		entry.set_property("sensitive", False)

		return entry


################################################################################
#
#					Template Editor
#
################################################################################


	def _create_textview(self):
		"""
		Create an editor for adding or editing templates.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		@return: An entry to hold a key.
		@rtype: A gtk.Entry object.

		"""

		textview = SourceView(SourceBuffer())
		textview.set_property("sensitive", False)
		self._syntax_highlight(textview)
		editor = self.editor
		textview.set_wrap_mode(WRAP_NONE)
		textview.set_show_line_numbers(False)
		textview.set_show_line_markers(False)
		textview.set_auto_indent(True)
		textview.set_margin(editor.margin_position)
		textview.set_show_margin(editor.show_margin)
		textview.set_tabs_width(editor.tab_width)
		textview.set_insert_spaces_instead_of_tabs(False)
		textview.set_highlight_current_line(True)
		textview.modify_font(editor.font)

		if not editor.use_theme_colors:

			# Use foreground and background colors specified by the user stored in
			# GConf, the GNOME configuration database.
			foreground_color = process_color(editor.foreground_color)
			background_color = process_color(editor.background_color)
			textview.modify_base(STATE_NORMAL, background_color)
			textview.modify_text(STATE_NORMAL, foreground_color)

		return textview


################################################################################
#
#							Helper Methods
#
################################################################################


	def _get_dialog_values(self):
		"""
		Get the name, description and string of the new template.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		"""

		name = self.name_entry.get_text().strip()
		description = self.description_entry.get_text().strip()
		begin, end = self.textview.get_buffer().get_bounds()
		template = self.textview.get_buffer().get_text(begin, end)

		return name, description, template


	def _update_template_dialog(self, value):
		"""
		Update the template editor's template key and description treeview.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		@param value: name, description and template values
		@type value: A Tuple object.

		"""

		model = self.templatedialog.template_key_view.get_model()
		iterator = model.append([value[0], value[1]])
		path = model.get_path(iterator)
		column = self.templatedialog.template_key_view.get_column(0)
		self.templatedialog.template_key_view.set_cursor_on_cell(path, column)
		self.templatedialog.template_key_view.set_property("sensitive", True)

		return


	def _update_template_database(self):
		"""
		Update the template database with new values

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		"""

		name = self.name_entry.get_text().strip()
		description = self.description_entry.get_text().strip()
		txtbuffer = self.textview.get_buffer()
		begin, end = txtbuffer.get_bounds()
		template = txtbuffer.get_text(begin, end)

		selection = self.templatedialog.languageview.get_selection()
		model, iterator = selection.get_selected()
		value = model.get_value(iterator, 0)

		key = value + name

		add_entry(self.editor, key, (description, template))

		return


	def _validate_entry(self):
		"""
		Validate the name entry field and the template entry field.

		Validation ensures that the strings entered in the name entry and the
		template editor are valid ones. For the name field, ensure that the
		string entered is not a duplicate one. That is the string does not exist
		in the database. For the template editor, ensure only one ${cursor}
		variable is present in the buffer. Show an error dialog if neither of
		these criterias are met. Also ensure that neither the name entry field
		nor the editor is empty.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@return: True if validation is successful. False otherwise.
		@rtype: A Boolean object.

		"""

		selection = self.templatedialog.languageview.get_selection()
		model, iterator = selection.get_selected()
		language = model.get_value(iterator, 0)
		name = self.name_entry.get_text().strip()

		if not name:

			message = "Name field must contain a unique string"
			error_dialog(self.editor, message)
			return False

		else:

			database = open_template_database(self.editor)

			for key in database.keys():

				if key == language + name:

					close_template_database(database)
					message = "%s already in use. Please choose another string." \
						% (name)
					error_dialog(self.editor, message)
					return False

			close_template_database(database)

		txtbuffer = self.textview.get_buffer()
		begin, end = txtbuffer.get_bounds()
		text = txtbuffer.get_text(begin, end)

		if not text:

			message = "Template field cannot be empty."
			error_dialog(self.editor, message)
			return False

		else:

			pattern = "\$\{cursor\}"
			result = findall(pattern, text)
			occurence = len(result)

			if occurence > 1:

				message = "%d instances of ${cursor} detected. Use only one." \
					% occurence
				error_dialog(self.editor, message)
				return False

		return True


	def _syntax_highlight(self, textview):
		"""
		Determine whether or not to turn on syntax highlight.

		Syntax highlight is turned on for all languages except General.

		@param self: Reference to the TemplateAdd instance.
		@type self: A TemplateAdd object.

		"""

		selection = self.templatedialog.languageview.get_selection()
		model, iterator = selection.get_selected()
		language = model.get_value(iterator, 0)

		if not language == "General":

			for languages in self.language_list:

				if languages.get_name() == language:

					txtbuffer = textview.get_buffer()
					txtbuffer.set_highlight(True)
					txtbuffer.set_language(languages)
					break

		return

