# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
This module contains a class that provides methods to display a template editor.
The template editor is a dialog that enables users to add, edit and remove
templates. A template is a pre-configured string that can be inserted into the
editor. The insertion is triggered by typing special strings into the editor's
buffer and pressing the tab key.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from gc import collect

from gtk import Dialog, WIN_POS_CENTER_ON_PARENT, STOCK_CLOSE, RESPONSE_CLOSE
from gtk import keysyms, TreeView, CellRendererText, TreeViewColumn, WRAP_NONE
from gtk import TREE_VIEW_COLUMN_GROW_ONLY, ListStore, HBox, VBox
from gtk import POLICY_NEVER, POLICY_AUTOMATIC, STOCK_REMOVE, STOCK_ADD
from gtk import Button, TREE_VIEW_COLUMN_AUTOSIZE, STOCK_EDIT
from gtk.gdk import WINDOW_TYPE_HINT_DIALOG, GRAVITY_CENTER, KEY_PRESS_MASK
from gtk import STATE_NORMAL, SORT_ASCENDING
from gtksourceview import SourceLanguagesManager, SourceView, SourceBuffer

from internationalization import msg0305, msg0187, msg0306, msg0307, msg0308
from feedback import update_status_info
from scrollwindow import create_scroll_window
from utility import process_color
from templateadd import TemplateAdd
from templateedit import TemplateEdit
from templatedatabase import open_template_database, close_template_database
from templatedatabase import remove_entry


class TemplateDialog(object):
	"""
	The template editor is a dialog window. It provides all the methods
	necessary to add, edit and remove templates. All templates have a key. The
	key is a unique string used to trigger the insertion of the template into
	the editor's buffer. Technically, the template editor is a frontend to a
	database containing keys and their associated templates. The abstract format
	is as follows:

		"language" + "template_key": (description_of_key, template)

	The template editor consists of a ListView comprising of a column containing
	source code languages; another ListView comprising of two columns containing
	zero or more template keys and a description for each key, if any; a
	SourceView containing a SourceBuffer for previewing templates; and buttons
	to add, edit and remove template keys.

	"""


	def __init__(self, editor):
		"""
		Initialize the TemplateDialog object.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param editor: Reference to an editor instance.
		@type editor: An Editor object.

		"""

		self._initialize_resuable_components(editor)
		self._set_dialog_window_properties()
		self._create_dialog_widgets()
		self._arrange_dialog_widgets()
		self.show_template_editor()


	def _initialize_resuable_components(self, editor):
		"""
		Initialize objects that will be used repeatedly only once.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param editor: Reference to the editor instance.
		@type editor: An Editor object.

		"""

		self.editor = editor
		self.dialog = Dialog()
		self.dialog.connect("key-press-event", self.key_press_event_cb)
		self.dialog.connect("map-event", self._default_setup)
		self.dialog.add_events(KEY_PRESS_MASK)
		self.language_list = SourceLanguagesManager().get_available_languages()
		self.language_names = [name.get_name() for name in self.language_list]

		return


	def _set_dialog_window_properties(self):
		"""
		Define the dialog window properties.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		"""

		# Calculate resolution independent width and height of the preferences
		# dialog window.
		width, height = self.editor.calculate_resolution_independence(1.6, 1.6)
		dialog = self.dialog
		dialog.set_property("title", msg0305)
		dialog.set_property("role", "template_dialog")
		dialog.set_property("type-hint", WINDOW_TYPE_HINT_DIALOG)
		dialog.set_property("modal", True)
		dialog.set_property("width-request", width)
		dialog.set_property("height-request", height)
		dialog.set_property("skip-taskbar-hint", True)
		dialog.set_property("skip-pager-hint", True)
		dialog.set_property("icon-name", "gnome-settings")
		dialog.set_property("window-position", WIN_POS_CENTER_ON_PARENT)
		dialog.set_property("gravity", GRAVITY_CENTER)
		dialog.set_property("resizable", True)
		dialog.set_property("has-separator", False)
		dialog.set_property("border-width", 12)
		dialog.vbox.set_spacing(12)
		dialog.add_button(STOCK_CLOSE, RESPONSE_CLOSE)
		dialog.set_transient_for(self.editor)

		return


	def _create_dialog_widgets(self):
		"""
		Create all widgets displayed in the template dialog.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		"""

		self.language_view = self._create_language_view()
		self.template_key_view = self._create_template_key_view()
		self.preview = self._create_preview_window()
		self.add_button = self._create_add_button()
		self.edit_button = self._create_edit_button()
		self.remove_button = self._create_remove_button()

		return


	def _arrange_dialog_widgets(self):
		"""
		Pack widgets appropriately into the template dialog.

		This function controls the appearance and position of buttons
		and views within the template dialog.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		"""

		hbox = HBox(False, 12)
		self.dialog.vbox.pack_start(hbox, True, True, 0)
		scrollwin = create_scroll_window()
		scrollwin.add(self.language_view)
		scrollwin.set_policy(POLICY_NEVER, POLICY_AUTOMATIC)
		hbox.pack_start(scrollwin, False, False, 0)

		vbox = VBox(False, 6)
		hbox.pack_start(vbox, True, True, 0)
		hbox = HBox(False, 12)
		vbox.pack_start(hbox, True, True, 0)
		scrollwin = create_scroll_window()
		scrollwin.add(self.template_key_view)
		hbox.pack_start(scrollwin, True, True, 0)

		scrollwin = create_scroll_window()
		scrollwin.add(self.preview)
		vbox.pack_start(scrollwin, True, True, 0)

		vbox = VBox(False, 6)
		hbox.pack_start(vbox, False, True, 0)
		vbox.pack_start(self.add_button, False, True, 0)
		vbox.pack_start(self.edit_button, False, True, 0)
		vbox.pack_start(self.remove_button, False, True, 0)

		return


	def show_template_editor(self):
		u"""
		Display the template editor.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		"""

		self.dialog.show_all()
		response = self.dialog.run()

		if response:

			self.dialog.hide_all()
			self.dialog.destroy()
			update_status_info(self.editor, msg0187, "info")
			del self
			collect()

		return


################################################################################
#
#						Callback to for the dialog
#
################################################################################


	def key_press_event_cb(self, dialog, key_press):
		"""Callback to the dialog's "key-press-event"

		The "key-press-event" signal is emitted when the user presses a key on
		the	keyboard.

		@param self: Reference to the TemplateEditor instance.
		@type self: A TemplateEditor object.

		@param dialog: The template editor.
		@type dialog: A TemplateEditor object.

		@param key_press: Event when the Escape key is pressed.
		@type key_press: An Event object.

		"""

		if key_press.keyval == keysyms.Escape:

			self.dialog.response(RESPONSE_CLOSE)

		return False


	def _default_setup(self, dialog, event):
		"""
		Determine the rows in the template dialog's TreeViews to select when the
		dialog is displayed.

		This function selects the first row in the "Name-Description" TreeView
		and the row corresponding to the language of the source code in the
		Language TreeView. That is, if a Python source code is being edited,
		the Language TreeView's Python row will be selected by default when
		the template dialog is displayed.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param dialog: The template dialog.
		@type dialog: A gtk.Dialog object.

		@param event: The event triggered when the dialog is mapped to screen.
		@type event: A gtk.Event object.

		@return: True to prevent propagation of the "map-event" signal to
			parent widgets.
		@rtype: A Boolean object.

		"""

		if self.editor.language:

			selection = self.languageview.get_selection()
			model, iterator = selection.get_selected()

			while not model.get_value(iterator, 0) == self.editor.language:

				iterator = model.iter_next(iterator)

			if iterator:

				path = model.get_path(iterator)
				column = self.languageview.get_column(0)
				self.languageview.set_cursor_on_cell(path, column)
				self.languageview.scroll_to_cell(path, column, True, 0.5)

		return True


################################################################################
#
#					TreeView for Programming Languages
#
################################################################################


	def _create_language_view(self):
		"""
		Create the view containing languages.

		@param self: Reference to the TemplateEditor instance.
		@type self: A TemplateEditor object.

		@return: A treeview populated with the name of programming languages.
		@rtype: A gtk.TreeView object.

		"""

		view = TreeView()
		self.languageview = view
		renderer = CellRendererText()
		column = TreeViewColumn(msg0306, renderer, text=0)
		view.append_column(column)
		column.set_expand(False)
		column.set_sizing(TREE_VIEW_COLUMN_GROW_ONLY)
		view.set_model(self._populate_language_view())
		view.columns_autosize()
		view.set_enable_search(False)
		view.set_property("rules-hint", True)
		view.connect("cursor-changed", self.languageview_changed_cb)

		return view


	def languageview_changed_cb(self, view):
		u"""
		Handles callback when rows in the Language TreeView are selected.

		This function populates the "Name-Description" TreeView with the name
		and description of templates available for the language selected in the
		Language TreeView.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param view: The Language TreeView containing source code languages.
		@type view: A TreeView object.

		@return: True to prevent propagation of the "cursor-changed" signal to
			parent widgets.
		@rtype: A Boolean object.

		"""

		# Disable the preview window, "Remove" button and "Edit" button in the
		# template dialog.
		self.preview.set_property("sensitive", False)
		self.remove_button.set_property("sensitive", False)
		self.edit_button.set_property("sensitive", False)

		# Clear the preview window in the template dialog.
		txtbuffer = self.preview.get_buffer()
		txtbuffer.set_text("")

		# Clear the "Name-Description" TreeView.
		keymodel = self.template_key_view.get_model()
		keymodel.clear()

		# Get the name of the selected row in the Language TreeView.
		selection = self.languageview.get_selection()
		model, iterator = selection.get_selected()
		value = model.get_value(iterator, 0)

		# True if templates are found in the templates database.
		boolean = False

		# Open the templates database, search for templates available for name
		# of the language selected in the Language TreeView and populate the
		# "Name-Description" TreeView with the template names and descriptions
		# if any are found.
		database = open_template_database(self.editor)

		for key in database.keys():

			if key.startswith(value):

				boolean = True
				string = key.replace(value, "")
				description = database[key][0]
				keymodel.append([string, description])

		close_template_database(database)

		# Disable the "Name-Description" TreeView if no templates are found in
		# the database.
		self.template_key_view.set_property("sensitive", boolean)

		# Select the first row in the "Name-Description" TreeView.
		if self.template_key_view.get_property("sensitive"):

			column = self.template_key_view.get_column(0)
			self.template_key_view.set_cursor_on_cell(0, column)

		self._syntax_highlight(self.preview)

		return True


	def _populate_language_view(self):
		"""
		Populate the template editor's Language TreeView with the name of
		programming languages.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		"""

		model = ListStore(str)
		model.append(["General"])

		for name in self.language_names:

			model.append([name])

		return model


################################################################################
#
#				TreeView for Template Names and Description
#
################################################################################


	def _create_template_key_view(self):
		"""
		Create the TreeView containing names and descriptions of the templates.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@return: A treeview populated with the name and description of
			templates.
		@rtype: A gtk.TreeView object.

		"""

		view = TreeView()
		model = ListStore(str, str)
		model.set_sort_column_id(0, SORT_ASCENDING)
		self.keyview = view

		renderer = CellRendererText()
		column = TreeViewColumn(msg0307, renderer, text=0)
		view.append_column(column)
		column.set_expand(False)
		column.set_sizing(TREE_VIEW_COLUMN_AUTOSIZE)

		renderer = CellRendererText()
		column = TreeViewColumn(msg0308, renderer, text=1)
		view.append_column(column)
		column.set_expand(True)
		column.set_sizing(TREE_VIEW_COLUMN_AUTOSIZE)

		view.set_model(model)
		view.columns_autosize()
		view.set_enable_search(False)
		view.set_property("rules-hint", True)
		view.set_property("sensitive", False)
		view.connect("cursor-changed", self.keyview_cb)
		view.connect("row-activated", self.keyview_activate_cb)

		return view


	def keyview_cb(self, view):
		"""
		Handles callback when any of the rows in the "Name-Description" TreeView
		are selected.

		This function retrieves information represented by the selected row from
		the template database and previews the template the row represents in
		the template dialog's preview window.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param view: The "Name-Description" description TreeView.
		@type view: A  gtk.TreeView object.

		@return: True to prevent the propagation of the "cursor-changed" signal
			to parent widgets.
		@rtype: A Boolean object.

		"""

		# Enable "Edit"  and "Remove" buttons as well as the preview window.
		self.remove_button.set_property("sensitive", True)
		self.edit_button.set_property("sensitive", True)
		self.preview.set_property("sensitive", True)

		# Clear the buffer of the preview window.
		txtbuffer = self.preview.get_buffer()
		txtbuffer.set_text("")

		# Get the selected language in the "Language" TreeView.
		selection = self.languageview.get_selection()
		model, iterator = selection.get_selected()
		language = model.get_value(iterator, 0)

		# Get the selected Name in the "Name-Description" TreeView.
		selection = self.template_key_view.get_selection()
		model, iterator = selection.get_selected()
		key = model.get_value(iterator, 0)

		# Concatenate the language and the name to form the template key for the
		# template database. Use the key to retrieve the template from the
		# database and preview it.
		key = language + key
		database = open_template_database(self.editor)
		txtbuffer.set_text(database[key][1])
		close_template_database(database)

		return True


	def keyview_activate_cb(self, view, path, column):
		"""
		Handles callback when a row in the "Name-Description" TreeView is
		double-clicked.

		This function displays a template editor.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param view: The "Name-Description" TreeView.
		@type view: A gtk.TreeView object.

		@param path: The currently selected path in the "Name-Description"
			TreeView.
		@type path: A gtk.Path object.

		@param column: A column in the "Name-Description" TreeView.
		@type column: A gtk.TreeViewColumn object.


		@return: True to prevent the propagation of the "row-activated" signal
			to parent widgets.
		@rtype: A Boolean object.

		"""

		TemplateEdit(self)

		return True


################################################################################
#
#								Preview Window
#
################################################################################


	def _create_preview_window(self):
		"""
		Create the TextView widget on the Template Dialog.

		The TextView widget serves as a preview for templates represented by
		the rows in the "Name-Description" TreeView. Clicking on any of the rows
		will show a preview of the template the row represents.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@return: A window used to preview widgets.
		@rtype: A gtk.SourceView object.

		"""

		textview = SourceView(SourceBuffer())
		textview.connect("button-press-event", self.preview_cb)
		textview.set_property("sensitive", False)
		editor = self.editor
		textview.set_wrap_mode(WRAP_NONE)
		textview.set_editable(False)
		textview.set_show_line_numbers(False)
		textview.set_show_line_markers(False)
		textview.set_auto_indent(True)
		textview.set_cursor_visible(False)
		textview.set_tabs_width(editor.tab_width)
		textview.set_insert_spaces_instead_of_tabs(False)
		textview.modify_font(editor.font)
		textview.get_buffer().set_check_brackets(False)

		if not editor.use_theme_colors:

			# Use foreground and background colors specified by the user stored
			# in GConf, the GNOME configuration database.
			foreground_color = process_color(editor.foreground_color)
			background_color = process_color(editor.background_color)
			textview.modify_base(STATE_NORMAL, background_color)
			textview.modify_text(STATE_NORMAL, foreground_color)

		return textview


	def preview_cb(self, textview, event):
		"""
		Handles callback when the preview window in the template dialog is
		clicked.

		This function displays the template editor.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param textview: The preview window in the template dialog.
		@type textview: A gtk.TextView object.

		@param event: A button-press-event.
		@type event: A gtk.Event object.

		@return: True to prevent the propagation of the "button-press-event"
			signal to parent widgets.
		@rtype: A Boolean object.

		"""

		TemplateEdit(self)

		return True


################################################################################
#
#								Add button
#
################################################################################


	def _create_add_button(self):
		"""
		Create the "Add" button on the template dialog.

		This function creates the button used to add templates to the text
		editor.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@return: A button used to add templates to the text editor.
		@rtype: A gtk.Button object.

		"""

		button = Button(None, STOCK_ADD)
		button.connect("clicked", self.add_button_cb)

		return button


	def add_button_cb(self, button):
		"""
		Handles callback when the "Add" button is clicked.

		This function displays the Add Template Dialog. The dialog is used to
		add templates to the editor.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param button: The button used to display the Add Template Dialog.
		@type button: A gtk.Button object.

		@return: True to prevent the propagation of the "clicked" signal to
			parent widgets.
		@rtype: A Boolean object.

		"""

		TemplateAdd(self)

		return True


################################################################################
#
#							Edit Button
#
################################################################################


	def _create_edit_button(self):
		"""
		Create the "Edit" button on the template dialog.

		This function creates the button used to edit templates from the text
		editor.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@return: A button used to edit templates from the text editor.
		@rtype: A gtk.Button object.

		"""

		button = Button(None, STOCK_EDIT)
		button.connect("clicked", self.edit_button_cb)
		button.set_property("sensitive", False)

		return button


	def edit_button_cb(self, button):
		"""
		Handles callback when the "Edit" button is clicked.

		This function displays the Edit Template Dialog. The dialog is used to
		edit template already in use by the text editor.

		@param button: The button used to display the Edit Template Dialog.
		@type button: A gtk.Button object.

		@return: True to prevent the propagation of the "clicked" signal to
			parent widgets.
		@rtype: A Boolean object.

		"""

		TemplateEdit(self)

		return True


################################################################################
#
#							Remove Button
#
################################################################################


	def _create_remove_button(self):
		"""
		Create the "Remove" button on the template dialog.

		This function creates the button used to remove templates from the text
		editor.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@return: A button used to remove templates from the text editor.
		@rtype: A gtk.Button object.

		"""

		button = Button(None, STOCK_REMOVE)
		button.connect("clicked", self.remove_button_cb)
		button.set_property("sensitive", False)

		return button


	def remove_button_cb(self, button):
		"""
		Handles callback when the "Remove" button on the template dialog is
		clicked.

		This function removes templates from the template database and reflects
		the changes of the removal in the "Name-Description" TreeView as well as
		the template preview window. After the removal occurs it selects the
		nearest row in the "Name-Description" TreeView. If no rows are available
		for selection, the "Remove" and "Edit" buttons as well as the template
		preview window is disabled.

		@param self: Reference to the TemplateDialog instance.
		@type self: A TemplateDialog object.

		@param button: The "Remove" button.
		@type button: A gtk.Button object.

		@return: True to prevent propagation of the "clicked" signal to parent
			widgets.
		@rtype: A Boolean object.

		"""

		# "Remove" button should be disabled when no rows in the template key
		# TreeView is selected.
		button.set_property("sensitive", False)

		# Clear the contents of the preview window.
		txtbuffer = self.preview.get_buffer()
		txtbuffer.set_text("")
		self.preview.set_property("sensitive", False)

		# Get the selected language in the Language TreeView.
		selection = self.languageview.get_selection()
		model, iterator = selection.get_selected()
		language = model.get_value(iterator, 0)

		# Get the selected name in the "Name-Description" TreeView.
		selection = self.template_key_view.get_selection()
		model, iterator = selection.get_selected()
		key = model.get_value(iterator, 0)

		# Remove the selected name in the "Name-Description" TreeView.
		result = model.remove(iterator)

		# Removing a row from a TreeView leaves the TreeView with unselected
		# rows. Thus a row needs to be selected. result is true when removing a
		# a row provides an iterator pointing to a path.
		if result:

			# Select the row iterator points to.
			path = model.get_path(iterator)
			column = self.template_key_view.get_column(0)
			self.template_key_view.set_cursor(path, column)
			self.preview.set_property("sensitive", True)

		else:

			# When the last row in a TreeView is removed, the iterator pointing
			# to the row in question becomes invalid. Therefore determining
			# whether or not the TreeView has rows becomes necessary.
			if model.get_iter_first():

				# In this branch, it has been determined that the TreeView has
				# rows. Thus, select the last row in the TreeView.
				row = model[-1]
				column = self.template_key_view.get_column(0)
				self.template_key_view.set_cursor(row.path, column)
				self.preview.set_property("sensitive", True)

			else:

				# This branch indicates the TreeView has no rows. Therefore,
				# disable widgets that prevent editing, removing and viewing
				# templates.
				button.set_property("sensitive", False)
				self.preview.set_property("sensitive", False)
				self.template_key_view.set_property("sensitive", False)
				self.edit_button.set_property("sensitive", False)

		# Remove template from the template database.
		key = language + key
		remove_entry(self.editor, key)

		return True


################################################################################
#
#					Syntax Highlight for languages
#
################################################################################


	def _syntax_highlight(self, textview):
		"""
		Determine whether or not to turn on syntax highlight.

		Syntax highlight is turned on for all languages except General.

		@param self: Reference to the TemplateEdit instance.
		@type self: A TemplateEdit object.

		"""

		selection = self.languageview.get_selection()
		model, iterator = selection.get_selected()
		language = model.get_value(iterator, 0)

		if not language == "General":

			for languages in self.language_list:

				if languages.get_name() == language:

					txtbuffer = textview.get_buffer()
					txtbuffer.set_highlight(True)
					txtbuffer.set_language(languages)
					break

		return

