# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


"""
The Template class provides mechanisms for inserting pre-configured strings
in the text buffer. These strings are called templates. The templates that
may be inserted into the buffer falls into two categories. The ordinary
templates and variable ones.

Ordinary templates are just strings entered into the buffer. Variable
templates contain special strings called variables. A variable string has
the following format:

	${arbitrary_string}

When variable templates are inserted into the text editor's buffer, the
editor activates a special mode called template mode. In this mode all
variables are highlighted. The tab key is used to select and navigate
between variables. Variables that have the same name will be modified
simultaneously. Navigation occurs in a clockwise manner. When no more
variables are available to select, the editor exits template mode.
The ${cursor} variable can be used to establish the last position of
cursor before exiting template mode.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from re import findall, sub, escape, UNICODE
from gc import collect

from gtk import TEXT_SEARCH_VISIBLE_ONLY
from gnomevfs import get_mime_type

from pango import STYLE_ITALIC
from cursor import get_characters_to_cursor, get_cursor_iter
from autocomplete import block_word_completion, unblock_word_completion
from autocomplete import hide_completion_window
from gtksourceview import SourceLanguagesManager
from templatedatabase import close_template_database, open_template_database
from internationalization import msg0025


class Template(object):
	"""
	The Template class provides all the necessary mechanisms for inserting
	text as well as navigating and editing variables. The class' objective is to
	provide abstractions for searching, replacing, tagging, monitoring and
	marking areas within the template region.

	"""

	def __init__(self, editor, string, start_iter, cursor_iter, template):
		"""
		Initialize the Template object.

		@param self: Reference to the ColorEditor.
		@type self: A ColorEditor object.

		@param editor: Reference to the editor.
		@type editor: An editor object.

		"""


		self._initialize_resuable_components(editor, template)
		self.process_string(string, start_iter, cursor_iter)


	def _initialize_resuable_components(self, editor, template):
		"""
		Initialize objects that will be used repeatedly only once.

		@param self: Reference to the Template.
		@type self: A Template object.

		@param editor: Reference to the editor.
		@type editor: An editor object.

		"""

		self.editor = editor
		self.template = template

		# A list containing template variables.
		self.variables = []

		# A list containing markers specifying the position of variables in
		# the text editor's buffer.
		self.template_marks = []

		# True if the template has duplicate variables.
		self.mirror_flag = False

		# The editor's buffer.
		self.textbuffer = self.editor.text_buffer

		# The tag of template variables before modification.
		self.template_pre_tag = self.textbuffer.create_tag(foreground="blue",
															background="yellow")

		# The tag of template variables after modification.
		self.template_post_tag = self.textbuffer.create_tag(foreground="blue",
														style=STYLE_ITALIC)

		# The tag of duplicate template variables.
		self.mirror_tag = self.textbuffer.create_tag(foreground="cyan",
														background="brown")

		# A markers signifying the beginning and end of the template region in
		# the editor's buffer.
		self.template_begin = None
		self.template_end = None

		return


	def process_string(self, string, start_iter, cursor_iter):
		"""
		Process the template trigger retrieved from the text buffer.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		@param editor: Reference to an editor instance.
		@type editor: An editor object.

		@param start_iter: Starting position of the string in the text buffer.
		@type: A gtk.TextIter.

		@param cursor_iter: Ending position of the string in the text buffer.
		@type: A gtk.TextIter.

		"""

		# Check if the text to be inserted into the buffer has any special
		# variables.
		result = self._check_for_template_variables(self.template)

		# Text with no variables need minimal processing. However, text with
		# variables demand complex processing.
		if result:

			# A list containing special template variables. The editor will
			# be switched to template mode at this point.
			self.variables = result
			self._initialize_template_mode(string, result, start_iter,
											cursor_iter)

		else:

			# Insert ordinary text with no variables into the buffer and
			# destroy the template object.
			self._insert_template(string, start_iter, cursor_iter)
			self.textbuffer.delete_mark(self.template_begin)
			self.textbuffer.delete_mark(self.template_end)
			del self
			collect()

		return


	def _insert_template(self, string, begin, end):
		"""
		Insert the template into the text editor's buffer.

		The function deletes the template key and inserts the template the key
		represents into the text editor's buffer. For multiline templates the
		indentation of each line is based on the indentation pattern of the
		first line of the template. This indentation style is otherwise known as
		automatic indentation.

		@param self: Reference of the template.
		@type self: A Template object.

		@param string: A template key representing a template.
		@type string: A string object.

		@param begin: The first point of the template key.
		@type begin: A gtk.TextIter object.

		@param end: The end point of the template key.
		@type end: A gtk.TextIter object.

		"""

		# Disable automatic word completion.
		block_word_completion(self.editor)

		if begin.starts_line():

			# If the insertion point is at the beginning of the line, insert the
			# template with no indentation.
			self._delete_insert_text(self.template, begin, end)

		else:

			# This branch indicates the insertion point is not at the beginning
			# of the line. Hence, indentation issues arise.
			line = begin.get_line()
			start_iter = self.textbuffer.get_iter_at_line(line)

			if not start_iter.get_char() in (" ", "\t"):

				# If the beginning character of the line has no indentation,
				# insert the template with no indentation.
				self._delete_insert_text(self.template, begin, end)

			else:

				# This branch indicates the beginning character of the line has
				# some sort of indentation.

				# Get the indentation of the first character of the line and use
				# its indentation pattern to indent the template.
				padding = self._get_indentation_padding(start_iter)
				text = self._pad_template_string(self.template, padding)
				self._delete_insert_text(text, begin, end)

		# Enable automatic word completion.
		unblock_word_completion(self.editor)

		return


	def _initialize_template_mode(self, string, variables, begin, end):
		"""
		Switch the editor to template mode.

		In template mode, the editor's behavior is marginally different.
		The editor highlights variables that are eligible for substitution and
		allows one to navigate through each highlighted string. The tab key is
		used for navigation.

		@param self: Reference to the template object.
		@type self: A Template object.

		"""

		# Insert template into the buffer.
		self._insert_template(string, begin, end)

		# Escape, highlight and mark template variables.
		self._tag_variables(variables)

		# Select the first variable.
		begin = self.textbuffer.get_iter_at_mark(self.template_begin)
		self._select_next_tag(begin, self.template_pre_tag)

		# The count keeps track of the number of template objects currently in
		# use by the editor. It is useful when keeping track of arbitrarily
		# nested templates. Template objects are stored in a template list and
		# removed when they are no longer needed. Only templates with variables
		# are kept track off. There is no need to keep track of templates with
		# no variables. Hence, the following statements pertain to templates
		# with variables only.
		self.editor.template_mode_count += 1
		self.editor.template_list.append(self)
		self.editor.template_mode_flag = True

		return


	def _delete_insert_text(self, string, begin, end, tag=None):
		"""
		Delete the text specified at begin and end, then insert text at the
		cursor position.

		This function is used initially when inserting the template and then
		also when replacing the template variables with their escaped versions.
		Escaped variables are variables without the "${}" enclosure.

		@param self: Reference to the template.
		@type self: A Template object.

		@param begin: The first point of the template key.
		@type begin: A gtk.TextIter object.

		@param end: The end point of the template key.
		@type end: A gtk.TextIter object.

		@param string: The template to be inserted into the buffer.
		@type string: A string object.

		"""

		self.textbuffer.delete(begin, end)

		# Only templates with variables have tags. Ordinary templates have no
		# tags.
		if tag:

			if string == "cursor":

				# ${cursor} is a special reserved variable. ${cursor} represents
				# the final position the cursor moves to before exiting template
				# mode. ${cursor} is replaced with a special visible marker and
				# inserted into the editor's buffer.
				iterator = get_cursor_iter(self.editor)
				final_mark = self.textbuffer.create_mark(None, iterator,
							False)
				final_mark.set_visible(True)
				self.final_mark = final_mark

			elif not string:

				# The empty variable, ${} is replaced with a space character and
				# entered. GTK+ cannot tag empty strings and I do not believe it
				# is prudent to create markers for empty strings. Tagging an
				# empty space is visual and more consistent.
				iterator = get_cursor_iter(self.editor)
				self.textbuffer.insert_with_tags(iterator, " ", tag)

			else:

				# Enter escaped template variables. Escaped template variables
				# are variables that are not enclosed in ${}.
				iterator = get_cursor_iter(self.editor)
				self.textbuffer.insert_with_tags(iterator, string,
												tag)

		else:

			# This branch is used only when initially inserting templates into
			# the buffer.
			iterator = get_cursor_iter(self.editor)
			self.template_begin = self.textbuffer.create_mark(None, iterator,
																True)
			self.textbuffer.insert_at_cursor(string + u" ")
			iterator = get_cursor_iter(self.editor)
			self.template_end = self.textbuffer.create_mark(None, iterator,
																False)

		return


	def _tag_variables(self, variables):
		"""
		Escape the variables is the textbuffer and tag them.

		After templates with variables are inserted into the buffer, each
		template variable is searched for, escaped, replaced and tagged. This
		function performs all these activities in one swoop.

		@param self: Reference to text editor's buffer.
		@type self: An editor object.

		@param variables: Raw template variables that need to be escaped and
			tagged.
		@type variables: A list object.

		"""

		# The starting position of the template. The position is used as a
		# beginning reference point for searching.
		begin = self.textbuffer.get_iter_at_mark(self.template_begin)

		# The occurence dictionary contains the number of each variable in the
		# buffer. The dictionary is used during mirroring.
		# Mirroring is the process of substiting variables with the same name
		# in the buffer simultaneously.
		self.occurence_dictionary = self._generate_variable_occurence()

		# Search for variables in the editor's buffer. Mark their position.
		# Replace the variables with their escaped version. And tag their
		# escaped version.
		for item in variables:

			end = self.textbuffer.get_iter_at_mark(self.template_end)
			position = begin.forward_search(item, TEXT_SEARCH_VISIBLE_ONLY, end)
			self.textbuffer.place_cursor(position[0])
			mark_one = self.textbuffer.create_mark(None, position[0], True)
			variable = self._escape_string(item)
			self._delete_insert_text(variable, position[0], position[1],
											self.template_pre_tag)
			begin = get_cursor_iter(self.editor)
			mark_two = self.textbuffer.create_mark(None, begin, False)
			self.template_marks.append((mark_one, mark_two, variable))

		return


	def _check_for_template_variables(self, template):
		"""
		Check if the template to be inserted into the editor's buffer has any
		variables.

		@param self: Reference to the template instance.
		@type self: A Template object.

		@param string: Key for the template dictionary.
		@type string: A string object.

		@return: A list of template variables if any.
		@rtype: A list object.

		"""

		# Match zero or more alphanumeric strings enclosed in ${}.
		pattern = "\$\{\s*\w*\s*\}"
		result = findall(pattern, template, UNICODE)

		return result


	def jump_to_next_variable(self):
		"""
		Select the next tagged variable when the tab key is pressed.

		This function begins by looking for "Pre-tags." Pre-tags are tags for
		variables that have not been navigated to. Tags that have been navigated
		have "Post-tags." Thus, the next variable to jump to is always the
		first one with a "Pre-tag." If no "Pre-tags" are found, the cursor is
		placed at the last position marked by ${cursor}. And if there is no
		last position mark, then template mode is exited.

		@param self: Reference to the template instance.
		@type self: A Template object.

		"""

		# Hide the completion window if it is visible.
		if self.editor.completion_flag:

			hide_completion_window(self.editor)

		# Check if the cursor is within a template variable.
		result = self._get_tag_iters_at_cursor()

		if result:

			# In this branch the cursor is certainly within a template variable.
			# Remove the Pre-tags of the template and replace them with
			# Post-tags. Search for variable mirrors, or duplicates. If mirrors
			# are found, replace all of them and toggle the mirror flag.
			begin, end, string = result
			self.textbuffer.remove_tag(self.template_pre_tag, begin, end)
			self.textbuffer.apply_tag(self.template_post_tag, begin, end)
			result = self._check_for_mirrors()

			if result:

				self.mirror_flag = True

			# Select the next template variable with a "Pre-tag."
			result = self._select_pre_tag()

		else:

			# In this branch the cursor is not within a template variable. So
			# select the next template variable with a "Pre-tag."
			result = self._select_pre_tag()


		if not result:

			# If no template variables with "Pre-tags" are found, place the
			# cursor at the last position. If there is no last position mark,
			# exit template mode.
			result = self._move_to_last_position()

			if not result:

				self._clean_up()

		return


	def jump_to_previous_variable(self):
		"""
		Select the previous variable in the variable list.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		"""

		hide_completion_window(self.editor)

		# Check if the cursor is within a template variable.
		result = self._get_tag_iters_at_cursor()

		if result:

			if result[0].has_tag(self.template_pre_tag):

				self._switch_to_post_tag(result[0])

			self._select_previous_tag(result[0], self.template_post_tag)

		else:

			iterator = get_cursor_iter(self.editor)
			self._select_previous_tag(iterator, self.template_post_tag)

		return


	def _check_for_mirrors(self):
		"""
		Check for duplicate template variables.

		Mirrors are more than one template variables with the same name. When a
		change is made to one of the variables, the same change should be made
		to others simultaneously.

		The function's purpose is to check if mirrors exist, and if they do,
		ensure that changes made to one mirror affects others.

		@return: True if mirrors are found.
		@rtype: A boolean object.

		"""

		# Check the occurence dictionary to see if the variable within the
		# cursor position has more than one occurence. The cursor position at
		# this stage is within markers. The markers are associate with variables
		# in the self.variables. Each element in self.variables contains, the
		# beginning mark, ending mark, and the variables within them.
		result = False
		begin, end, escaped_string = self._get_tag_iters_at_cursor()
		count = self.occurence_dictionary[escaped_string]

		# Disable word completion.
		block_word_completion(self.editor)

		if count > 1:

			# In this branch mirrors are present. Get a list of all mirrors and
			# their position in the buffer.
			result = True
			duplicate_list = []

			for elements in self.template_marks:

				if escaped_string in elements:

					duplicate_list.append(elements)

			# Filter out the first mirror. The firs mirror has already been
			# replaced. The other mirrors have not.
			mirror_list = duplicate_list[1:]

			# Search for the position of mirrors in the buffer and replace them
			# all.
			for elements in mirror_list:

				begin = self.textbuffer.get_iter_at_mark(elements[0])
				end = self.textbuffer.get_iter_at_mark(elements[1])

				if begin.has_tag(self.template_post_tag):

					continue

				self.textbuffer.delete(begin, end)
				iterator = self.textbuffer.get_iter_at_mark(elements[0])
				sbegin = self.textbuffer.get_iter_at_mark(duplicate_list[0][0])
				send = self.textbuffer.get_iter_at_mark(duplicate_list[0][1])
				self.textbuffer.insert_range(iterator, sbegin, send)

		# Enable word completion.
		unblock_word_completion(self.editor)

		return result


	def _clean_up(self):
		"""
		Destroy the template object and remove all tags and marks associated
		with it.

		@param self: Reference to the Template instance.
		@type self: A Template object

		"""

		begin, end = self._get_iter_at_template_regions()
		self.editor.template_mode_count -= 1

		if not self.editor.template_mode_count:

			self.editor.template_mode_flag = False

		self.textbuffer.remove_tag(self.template_pre_tag, begin, end)
		self.textbuffer.remove_tag(self.template_post_tag, begin, end)

		for markers in self.template_marks:

			self.textbuffer.delete_mark(markers[0])
			self.textbuffer.delete_mark(markers[1])

		self.textbuffer.delete_mark(self.template_begin)
		self.textbuffer.delete_mark(self.template_end)

		try:

			self.textbuffer.delete_mark(self.final_mark)

		except AttributeError:

			pass

		del self.editor.template_list[-1]
		del self
		collect()

		return


	def _move_to_last_position(self):
		"""
		Determine the final position to place the cursor.

		The last position to place the cursor is always the last position
		marker. However when a last position marker does not exist, the cursor
		should be placed at the end of the template region and template mode
		should be exited.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		@return: False if template mode should be exited, True otherwise.
		@rtype: A boolean object.

		"""

		try:

			# The flag indicates that the cursor is currently at the last
			# position marker. Template mode needs to be exited at this point.
			if self.last_position:

				return False

		except AttributeError:

			try:

				# Place the cursor at the last position marker.
				iterator = self.textbuffer.get_iter_at_mark(self.final_mark)
				self.textbuffer.place_cursor(iterator)
				self.last_position = True
				return True

			except AttributeError:

				# During a mirror operation, when there is no other variable to
				# select place the cursor at the end of the bracket region and
				# exit template mode.
				if self.mirror_flag:

					position = self._get_iter_at_template_regions()
					self.textbuffer.place_cursor(position[1])

				else:

					return False

		return True


	def _escape_string(self, string):
		"""
		Remove ${} enclosure from template variables.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		@param string : A template variables enclosed in ${}
		@type string : A string object.

		@return: A template variable with the ${} enclosure.
		@rtype: A string object.

		"""

		for character in ("$", "{", "}"):

			string = string.replace(character, "")

		return string


	def _select_next_tag(self, iterator, tag):
		"""
		Select a tag in the template region.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		@param iterator: The position to begin searching from.
		@type iterator: A gtk.TextIter object.

		@param tag: A Pre-tag or Post-tag.
		@type tag: A gtk.TextTag object.

		@return: True if a tag was found.
		@rtype: A boolean object.

		"""

		result = iterator.forward_to_tag_toggle(tag)

		if result:

			begin, end =  self._get_tag_position(iterator, tag)
			self.textbuffer.select_range(begin, end)

		return result


	def _select_previous_tag(self, iterator, tag):
		"""
		Select a tag in the template region.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		@param iterator: The position to begin searching from.
		@type iterator: A gtk.TextIter object.

		@param tag: A Pre-tag or Post-tag.
		@type tag: A gtk.TextTag object.

		@return: True if a tag was found.
		@rtype: A boolean object.

		"""

		result = iterator.backward_to_tag_toggle(tag)

		if result:

			begin, end =  self._get_tag_position(iterator, tag)
			self.textbuffer.select_range(begin, end)

		return result


	def _get_indentation_padding(self, start_iter):
		"""
		Determine indentation padding for multiline templates.

		Indentation spacing is determined based on the indentation of the line
		where the template is to be inserted.

		@param self: Reference to the template.
		@type self: A Template object.

		@param start_iter: The start of the line.
		@type start_iter: A gtk.TextIter object.

		@return: Indentation to add to multiline templates.
		@rtype: A string object contain tabs and spaces.

		"""

		padding_list = []

		while start_iter.get_char() in (" ", "\t"):

			character = start_iter.get_char()
			padding_list.append(character)
			start_iter.forward_char()

		return "".join(padding_list)


	def _pad_template_string(self, string, padding):
		"""
		Add padding to multiline templates.

		The first line never needs to be padded. Only the second line onwards
		needs indentation padding.

		@param self: Reference to the template.
		@type self: A Template object.

		@param string: The template string to be inserted into the buffer.
		@type string: A String object.

		@param padding: Indentation to be added before each line.
		@type padding: A string object.

		"""

		string_list = string.split("\n")

		if len(string_list) > 1:

			first_line = string_list[0]
			string_list = string_list[1:]
			padded_strings_list = []

			for line in string_list:

				line = padding + line
				padded_strings_list.append(line)

			padded_strings_list.insert(0, first_line)

			return " \n".join(padded_strings_list)

		return string


	def _check_template_region(self):
		"""
		Monitor the movement of the cursor within the template region.

		This function is called everytime the editor's TexTView's "mark-set"
		signal is emitted. The signal is emitted everytime the cursor moves. The
		function checks if the cursor is within tags, most pertinently, the
		Pre-tag. If it is, it tries to select the tag.

		@note: This function should provide more beneficial uses in the future.
			I just can't come up with sometime interesting yet.
		@param self: Reference to the Template instance.
		@type self: A Template object.

		"""

		begin, end = self._get_iter_at_template_regions()
		iterator = get_cursor_iter(self.editor)

		if not get_cursor_iter(self.editor).in_range(begin, end):

			# If the cursor is out of the template region exit template mode.
			self._clean_up()

		elif iterator.has_tag(self.template_pre_tag):

			# If the cursor wanders to an area with the template region that
			# has a Pre-tag, the variable with the tag should be selected.
			if not self.textbuffer.get_selection_bounds():

				begin, end = self._get_tag_position(iterator,
													self.template_pre_tag)
				self.textbuffer.select_range(begin, end)

		return


	def _get_tag_position(self, iterator, tag):
		"""
		Get the position of a tag in the editor's buffer.

		@param iterator: position to begin searching.
		@type iterator: A gtk.TextIter object.

		@param tag: Pre-tag or Post-tag.
		@type tag: A gtk.TextTag object.

		@return: The beginning and end position of the tag.
		@rtype: A tuple object.

		"""

		if iterator.begins_tag(tag):

			begin = iterator.copy()
			end = iterator.copy()
			end.forward_to_tag_toggle(tag)

		elif iterator.ends_tag(tag):

			begin = iterator.copy()
			end = iterator.copy()
			begin.backward_to_tag_toggle(tag)

		else:

			begin = iterator.copy()
			end = iterator.copy()
			end.forward_to_tag_toggle(tag)
			begin.backward_to_tag_toggle(tag)

		return begin, end


	def _switch_to_post_tag(self, iterator):
		"""
		Replace Pre-tags on template variables with Post-tags.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		@param iterator: The position of the variables with the Pre-tag in the
			editor's buffer.
		@type iterator: A gtk.TextIter object.

		"""

		begin, end = self._get_tag_position(iterator, self.template_pre_tag)
		self.textbuffer.remove_tag(self.template_pre_tag, begin, end)
		self.textbuffer.apply_tag(self.template_post_tag, begin, end)

		return


	def _check_tag_region(self):
		"""
		When the cursor moves out of template variable that has the Pre-tag,
		switch the template variable to a Post-tag.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		"""

		iterator = get_cursor_iter(self.editor)

		if iterator.has_tag(self.template_pre_tag):

			self._switch_to_post_tag(iterator)
			self._check_for_mirrors()

		return


	def _get_tag_iters_at_cursor(self):
		"""
		Get the position of the tagged template variable at the cursor
		position

		@param self: Reference to the Template instance.
		@type self: A Template object.

		@return: The template variable and its position.
		@rtype: A Tuple object.

		"""

		cursor_iter = get_cursor_iter(self.editor)

		for markers in self.template_marks:

			result = False
			begin, end, string = self._get_iters_at_position(markers)

			if cursor_iter.in_range(begin, end) or cursor_iter.equal(begin)\
				or cursor_iter.equal(end):

				return begin, end, string

		return result


	def _select_pre_tag(self):
		"""
		Select a Pre-tag tagged variables in the editor's template region.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		@return: True is a template variable is selected.
		@rtype: A Boolean object.

		"""

		cursor_iter = get_cursor_iter(self.editor)
		result = self._select_previous_tag(cursor_iter, self.template_pre_tag)

		if not result:

			result = self._select_next_tag(cursor_iter, self.template_pre_tag)

		return result


	def _get_iters_at_position(self, markers):
		"""
		Convert markers in the template's variable list to iterators.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		@param markers: Markers representing a template variable position.
		@type markers: A gtk.TextMark object.

		@return: A template variable and its position represented by iterators.
		@rtype: A gtk.TextIter object.

		"""

		begin = self.textbuffer.get_iter_at_mark(markers[0])
		end = self.textbuffer.get_iter_at_mark(markers[1])
		escaped_variable = markers[2]

		return begin, end, escaped_variable


	def _check_for_exit_action(self):
		"""
		Check when to exit template mode.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		"""

		try:

			iterator = self.textbuffer.get_iter_at_mark(self.final_mark)
			cursor_iter = get_cursor_iter(self.editor)

			if cursor_iter.equal(iterator):

				self._clean_up()

		except AttributeError:

			begin, end, string = self._get_iters_at_position(self.template_marks[-1])
			cursor_iter = get_cursor_iter(self.editor)

			if cursor_iter.in_range(begin, end) or cursor_iter.equal(begin) or\
				cursor_iter.equal(end):

				self._clean_up()

		return


	def _get_iter_at_template_regions(self):
		"""
		Get the beginning and end regions of the template.

		@param self: Reference to the Template instance.
		@type self: A Template object.

		@return: The beginning and end position of the template region.
		@rtype: A tuple object.

		"""

		begin = self.textbuffer.get_iter_at_mark(self.template_begin)
		end = self.textbuffer.get_iter_at_mark(self.template_end)

		return begin, end


	def _generate_variable_occurence(self):
		"""
		Calculate the occurence of each template variable in the buffer.

		@param self: Reference to the Template.
		@type self: A Template object.

		@return: Return the occurence of each variable in the text editor's
			buffer.
		@rtype: A dictionary object.

		"""

		occurence_dictionary = {}
		filter_list = []

		for variable in self.variables:

			string = self._escape_string(variable)
			filter_list.append(string)
			occurence_dictionary[string] = filter_list.count(string)

		return occurence_dictionary


def generate_generic_templates(editor):
	"""
	Update the editor's template dictionary with generic templates.

	This function is always called when the editor is launched.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	database = open_template_database(editor)

	for elements in database.keys():

		if elements.startswith("General"):

			editor.templates[elements] = database[elements][1]

	close_template_database(database)

	return


def generate_language_templates(editor):
	"""
	Update the editor's template dictionary with language specific templates.

	This function is only useful for source code file.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	database = open_template_database(editor)

	if not editor.filename == msg0025.decode(editor.preferredencoding).encode("utf-8"):

		if editor.language:

			for elements in database.keys():

				if elements.startswith(editor.language):

					editor.templates[elements] = database[elements][1]

	close_template_database(database)

	return


def check_for_generic_template_key(editor, string):
	"""
	Check the editor's template dictionary for a template key matching string.

	Template keys can either be general or language specific. This function
	looks for general keys.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	@param string: Characters before the cursor.
	@type string: A String object.

	@return: Return a template/snippet if the string is a template key or None
		otherwise.
	@rtype: A String object or None.

	"""

	value = None
	key = "General" + string

	if key in editor.templates.keys():

		value = editor.templates[key]

	return value


def check_for_language_template_key(editor, string):
	"""
	Check the editor's template dictionary for a template key matching string.

	Template keys can either be general or language specific. This function
	looks for language specific keys.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	@param string: Characters before the cursor.
	@type string: A String object.

	@return: Return a template/snippet if the string is a template key or None
		otherwise.
	@rtype: A String object or None.

	"""

	value = None

	# If the file is source code, get the language of the source code and use
	# the language's name to form a template key with the string.
	if editor.language:

		# The template key is derived by concatenating the language of the
		# source code and string.
		key = editor.language + string

		if key in editor.templates.keys():

			value = editor.templates[key]

	return value


def check_for_template_insertion(editor):
	"""
	Determine if the string before the cursor is elligible for template
	completion.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	@return: Return True to prevent the Tab character from being inserted into
		the buffer. That is to block the Tab key event.
	@rtype: A Boolean object.

	"""

	if editor.readonly:

		return False

	try:

		string, start_iter, cursor_iter = get_characters_to_cursor(editor)

	except TypeError:

		string = ""

	result = check_for_language_template_key(editor, string)

	if not result:

		result = check_for_generic_template_key(editor, string)

	if result:

		hide_completion_window(editor)
		Template(editor, string, start_iter, cursor_iter, result)
		value = True

	else:

		if editor.template_mode_flag:

			hide_completion_window(editor)
			editor.template_list[-1].jump_to_next_variable()
			value = True

		else:

			value = False

	return value

