# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Textbuffer functions for Scribes

"""

from os import path

from pygtk import require
require("2.0")
from gobject import timeout_add, source_remove
from gtksourceview import SourceBuffer

from timeout import save_file_timeout
from autocomplete import validate_cursor_string
from bookmark import update_bookmark_list
from modes import add_mode


################################################################################
#
#						Text Buffer Callbacks
#
################################################################################


def buffer_changed_cb(textbuffer, editor):
	"""
	Handles callback when text is inserted into the buffer

	The "changed" signal is emitted when text is inserted in textbuffer.

	@param textbuffer:	editor buffer object
	@type textbuffer: 	A gtksourceview.SourceBuffer object
	@param editor:	A reference to the editor instance
	@type editor:		An editor object

	"""

	editor.is_indexed = False
	# A flag indicating the editor's buffer has been tainted.
	editor.dirty = True

	# This flag shows the file has already been saved and doesn't need
	# further saving. Used by the automatic saving function of the
	# editor. When the flag is true, the file is saved. Otherwise, the
	# file is unsaved.
	editor.save_flag = False

	# If the file is not a new file and text has been inserted into the
	# editor's display area, then prepend an asterik to the name of the
	# file in the editor window's title.
	if not editor.isNewFile:

		window_title = "*" + path.basename(path.expanduser(editor.filename))
		editor.set_title(window_title)

	return True


def modified_cb(textbuffer, editor):
	"""
	Handles callback when the modification flag of the buffer has changed

	The "modified-changed" signal is emitted when the modification flag is
	changed.

	@param textbuffer:	editor buffer object
	@type textbuffer: 	A gtksourceview.SourceBuffer object
	@param editor:	A reference to the editor instance
	@type editor:		An editor object

	"""

	try:

		# Remove all save timers to avoid conflict
		source_remove(editor.save_timer)

	except AttributeError:

		pass

	# Check the saved state of the file. If the file has already been saved
	# exit this function.
	if editor.save_flag:

		return

	if not editor.mode_edit_flag:

		add_mode(editor, "edit")
		editor.mode_edit_flag = True

	# If the file is not a new file, call the save function after 21 secs.
	if not editor.isNewFile:

		editor.save_timer = timeout_add(21000, save_file_timeout, editor)

	return True


def can_undo_cb(textbuffer, boolean, editor):
	"""Handles callback to determine whether undo events can occur.

	The "can-undo" signal is emitted when there is a change in the buffer's
	ability to undo an operation.

	@param textbuffer:	editor buffer object
	@type textbuffer: 	A gtksourceview.SourceBuffer object
	@param boolean:		Whether or not an undo procedure is possible
	@type boolean:		A boolean object
	@param editor:		A reference to the editor instance
	@type editor:			An editor object

	"""

	if boolean:

		editor.actiongroup.get_action("undo_edits").set_sensitive(True)

	else:

		editor.actiongroup.get_action("undo_edits").set_sensitive(False)

	return True


def can_redo_cb(textbuffer, boolean, editor):
	"""
	Handles callback to determine whether redo events can occur.

	The "can-redo" signal is emitted when there is a change in the buffer's
	ability to redo an operation.

	@param textbuffer:	editor buffer object
	@type textbuffer: 	A gtksourceview.SourceBuffer object
	@param boolean:		Whether or not an undo procedure is possible
	@type boolean:		A boolean object
	@param editor:		A reference to the editor instance
	@type editor:			An editor object

	"""

	if boolean:

		editor.actiongroup.get_action("redo_edits").set_sensitive(True)

	else:

		editor.actiongroup.get_action("redo_edits").set_sensitive(False)

	return True


def insert_text_cb(textbuffer, iterator, character, length, editor):
	"""
	Handles callback when text is inserted into the editor's buffer.

	The "insert-text" signal is emitted when text of the size specified by
	length is inserted into textbuffer at the location specified by iter.

	@param textbuffer:	The textbuffer that received the signal.
	@type textbuffer: A gtk.TextBuffer.

	@param iterator: Iterator of inserted text.
	@type iterator:	A gtk.TextIter.

	@param text: The text inserted in textbuffer.
	@type text: A string.

	@param length: The length of the text inserted in textbuffer.
	@type length: An integer.

	@param editor: A reference to the editor.
	@type editor: An editor object.

	"""

	# Determine when to exit the editor's template mode. See module templates.py
	# for documentation and implementation details of the _check_for_exit_action
	# method.
	if editor.template_mode_flag:

		timeout_add(3, editor.template_list[-1]._check_for_exit_action)

	# Check for the possibility of word completion
	timeout_add(3, validate_cursor_string, editor)

	return True


def mark_deleted_cb(textbuffer, textmark, editor):
	"""
	Handles callback when markers are deleted from the text editor's buffer.

	@param textbuffer: The text editor's buffer.
	@type textbuffer: A gtksourceview.SourceBuffer

	@param textmark: Marker to be removed or deleted.
	@type textmark: A gtksourceview.Marker.

	@param editor: Reference to the text editor instance.
	@type editor: An editor object.

	"""

	if editor.bookmark_flag:

		if textmark in editor.bookmark_list:

			update_bookmark_list(editor, "remove", textmark)

	return True


def begin_cb(textbuffer, editor):

	print "begin"

	return True


def end_cb(textbuffer, editor):

	print "end"

	return True


def delete_range_cb(textbuffer, start, end, editor):

	return True


def block_user_action(editor):

	if editor.user_action_flag is False:

		editor.text_buffer.handler_block(editor.tb_busr_id)
		editor.text_buffer.handler_block(editor.tb_eusr_id)
		editor.user_action_flag = True

	return


def unblock_user_action(editor):

	if editor.user_action_flag:

		editor.text_buffer.handler_unblock(editor.tb_busr_id)
		editor.text_buffer.handler_unblock(editor.tb_eusr_id)
		editor.user_action_flag = False

	return


################################################################################
#
#							Text Buffer
#
################################################################################


def create_text_buffer(editor):
	"""
	Create a text buffer for the editor

	@param editor: A reference to the editor instance
	@type editor: An editor object

	@return: text buffer object
	@rtype: A gtksourceview.SourceBuffer object

	"""

	editor.user_action_flag = False
	text_buffer = SourceBuffer()
	text_buffer.word_buffer = ["", None, None]
	editor.text_buffer = text_buffer
	editor.tb_chg_id = text_buffer.connect("changed", buffer_changed_cb,
										editor)
	editor.tb_mod_id = text_buffer.connect("modified-changed",
										modified_cb, editor)
	editor.tb_ins_id = text_buffer.connect("insert-text", insert_text_cb,
												editor)
#	editor.tb_busr_id = text_buffer.connect("begin-user-action", begin_cb, editor)
#	editor.tb_eusr_id = text_buffer.connect("end-user-action", end_cb, editor)
	#block_user_action(editor)
	text_buffer.connect("can-undo", can_undo_cb, editor)
	text_buffer.connect("can-redo", can_redo_cb, editor)
	text_buffer.connect("mark-deleted", mark_deleted_cb, editor)
	#text_buffer.connect("delete-range", delete_range_cb, editor)
	text_buffer.set_max_undo_levels(0)
	text_buffer.set_text("")
	text_buffer.set_modified(False)

	return text_buffer

