# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Textview functions for Scribes

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from pygtk import require
require("2.0")
from gobject import timeout_add, source_remove
from gtk import SeparatorMenuItem, ImageMenuItem, STOCK_ABOUT, MenuItem
from gtk import RESIZE_PARENT, WRAP_WORD, WRAP_NONE
from gtk import DEST_DEFAULT_ALL, STOCK_INDENT, STOCK_UNINDENT, Image
from gtk import ICON_SIZE_MENU, clipboard_get, TEXT_WINDOW_TEXT, STATE_NORMAL
from gtk.gdk import ACTION_COPY, POINTER_MOTION_MASK, SCROLL_MASK, XTERM
from gtk.gdk import Cursor
from gtksourceview import SourceView

try:

	from gtksourceview import source_iter_find_matching_bracket as match_bracket

except ImportError:

	from gtksourceview import gtk_source_iter_find_matching_bracket as \
match_bracket

from about import create_about_dialog
from cursor import cursor_position, get_current_line, get_cursor_iter
from timeout import fullscreen_timeout
from feedback import update_status_info
from internationalization import msg0086, msg0087, msg0088, msg0089, msg0090
from internationalization import msg0091, msg0092, msg0093, msg0094, msg0145
from brackets import check_bracket_region
from bookmark import check_for_line_bookmark
from autocomplete import hide_completion_window, block_word_completion
from autocomplete import unblock_word_completion
from findbar import hide_findbar
from gotobar import hide_gotobar
from replacebar import hide_replacebar
from utility import response, process_color
from keyevents import filter_key_events


def make_buffer_editable(editor):
	"""
	Make the editor's buffer editable

	@param editor:	Reference to the editor instance
	@type editor:		An editor object

	"""

	editor.text_view.set_editable(True)

	return


################################################################################
#
#						Text Buffer Callbacks
#
################################################################################


def mark_set_cb(textbuffer, iterator, textmark, editor):
	"""
	Hanldes callback when editor's cursor moves

	The "mark-set" signal is emitted when textmark is being set at the location
	specified by iter in textbuffer.

	@param textbuffer:	editor buffer object
	@param iterator: an iterator pointing at the location where textmark will be
					set
	@param textmark:	the gtk.TextMark  that is being set
	@param editor:	Reference to the editor instance
	@type editor:		An editor object

	"""

	if editor.template_mode_flag:

		editor.template_list[-1]._check_template_region()

	if editor.bracket_region_flag:

		check_bracket_region(editor)

	if editor.bookmark_flag:

		if textmark in editor.bookmark_list:

			bookmarks = check_for_line_bookmark(editor)

			if bookmarks:

				editor.actiongroup.get_action("remove_bookmark").activate()

	cursor_position(editor)

	return True


def mark_delete_cb(textbuffer, textmark, editor):
	"""
	Hanldes callback when selection bound deletion occurs

	The "mark-deleted" signal is emitted when textmark is being deleted from
	textbuffer.

	@param textbuffer:	editor buffer object
	@param textmark:	the gtk.TextMark  that is being set
	@param editor:	Reference to the editor instance
	@type editor:		An editor object

	"""

	cursor_position(editor)

	return True


def move_cursor_cb(textview, step_size, count, selection, editor):
	"""
	document_header

	"""

	# Monitor the cursor to determine when it enters and leaves a tag region.
	# See _check_tag_region method in module templates.py for documentation and
	# implemenation details.
	if editor.template_mode_flag:

		editor.template_list[-1]._check_tag_region()

	return True


################################################################################
#
#					Text View Popup Menu Callback
#
################################################################################


def about_menuitem_cb(menuitem, editor):
	"""
	Handles callback when the about menuitem is clicked.

	@param menuitem: about_menuitem
	@type menuitem:	A gtk.Menuitem object

	"""

	# Create and show the about dialog
	create_about_dialog(editor)

	return True


def bracket_menuitem_cb(menuitem, editor):
	"""
	Handles callback when the "Find Matching Bracket" menuitem is clicked.

	@param menuitem: about_menuitem
	@type menuitem:	A gtk.Menuitem object
	@param editor: Reference to the editor instance
	@type editor: An editor object

	"""

	editor.actiongroup.get_action("match_bracket").activate()

	return True


def indent_menuitem_cb(menuitem, editor):
	"""
	Handles callback when the "Shift Right" menuitem is clicked.

	@param menuitem: about_menuitem
	@type menuitem:	A gtk.Menuitem object
	@param editor: Reference to the editor instance
	@type editor: An editor object

	"""

	editor.actiongroup.get_action("indent").activate()

	return True


def unindent_menuitem_cb(menuitem, editor):
	"""
	Handles callback when the "Shift Left" menuitem is clicked.

	@param menuitem: about_menuitem
	@type menuitem:	A gtk.Menuitem object
	@param editor: Reference to the editor instance
	@type editor: An editor object

	"""

	editor.actiongroup.get_action("unindent").activate()

	return True


################################################################################
#
#						Text View Callbacks
#
################################################################################


def scroll_event_cb(textview, event, editor):
	"""
	Handles callback when the scroll button on the mouse is activated.

	@param textview: The text editor's textview widget.
	@type textview: A gtk.SourceView object.

	@param event: A mouse scroll event.
	@type event: A gtk.Event object

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Hide the completion window if it is visible.
	if editor.completion_flag:

		hide_completion_window(editor)

	return False


def button_press_event_cb(textview, button_press, editor):
	"""
	Handles callback when mouse clicks are detected in the textview object

	@param textview: text_view object
	@type textview: A gtksourceview.SourceView
	@param button_press: left mouse click
	@type button_press: A gtk.Event
	@param editor: Reference to the editor instance
	@type editor: An editor object

	"""

	# Hide the completion window if it is visible.
	if editor.completion_flag:

		hide_completion_window(editor)

	# Hide and destroy an instance of the gotobar if it exists.
	if editor.gotobar_flag:

		hide_gotobar(editor)

	# Hide and destroy an instance of the findbar if it exists.
	if editor.findbar_flag:

		hide_findbar(editor)

	# Hide and destroy an instance of the replacebar if it exists.
	if editor.replacebar_flag:

		hide_replacebar(editor)

	return False


def toggle_overwrite_cb(textview, editor):
	"""
	Handles callback when Insert key is pressed

	The "toggle-overwrite" signal is emitted when the user presses the
	Insert key.

	@param textview: text_view object
	@type textview: A gtksourceview.SourceView
	@param editor: Reference to the editor instance
	@type editor: An editor object

	"""

	# Set textview overwrite mode
	if not textview.get_overwrite():

		mode = "OVR"

	else:

		mode = "INS"

	# Let status bar reflect correct textview mode
	editor.status_three.pop(editor.status_three_id)
	editor.status_three_id = editor.status_three.get_context_id(mode)
	editor.status_three.push(editor.status_three_id, mode)

	return True


def motion_cb(textview, context, x, y, time, editor):

	if "text/uri-list" in context.targets:

		return True

	return False


def drop_cb(textview, context, x, y, time, editor):

	return True


def received_cb(textview, context, xcord, ycord, selection_data, info,
				timestamp, editor):
	"""
	Handles callback when drop operation is completed

	The "drag-data-received" signal is emitted when the drag destination
	receives the data from the drag operation. If the data was received in order
	to determine whether the drop will be accepted, the handler is expected to
	call the gtk.gdk.DragContext.drag_status() method and not finish the drag.
	If the data was received in response to a "drag-drop" signal (and this is
	the last target to be received), the handler for this signal is expected to
	process the received data and then call the gtk.gdk.DragContext.finish()
	method, setting the success parameter to TRUE if the data was processed
	successfully.

	textview -- text_view object
	context -- object containing data to manipulated
	xcord -- the X position of the drop
	ycord -- the Y position of the drop
	data -- a gtk.SelectionData object
	info -- an integer ID for the drag
	timestamp -- the time of the drag event
	editor -- editor object

	"""

	if info == 80:

		# Load file
		from main import load_uris
		load_uris(editor, selection_data.get_uris())
		context.finish(True, False, timestamp)

	return True


def populate_popup_cb(textview, menu, editor):
	"""
	Handles callback when editor's textview pop up menu is displayed

	@param textview: text_view object
	@type textview: A gtksourceview.SourceView

	@param menu: text_view context menu object
	@type menu: A gtk.Menu object

	"""

	about_menuitem = ImageMenuItem(STOCK_ABOUT)
	about_menuitem.connect("activate", about_menuitem_cb, editor)
	menu.append(SeparatorMenuItem())
	menu.append(about_menuitem)
	separator = SeparatorMenuItem()
	menu.append(separator)
	menu.reorder_child(separator, 4)

	# Indent Menuitem

	# In readonly only mode, make the indent menuitem insensitive at all times
	if editor.readonly:

		indent = False

	else:

		indent = True

	indent_menuitem = ImageMenuItem(msg0086)
	indent_menuitem.connect(u"activate", indent_menuitem_cb, editor)
	indent_menuitem.set_sensitive(indent)
	image = Image()
	image.set_from_stock(STOCK_INDENT, ICON_SIZE_MENU)
	indent_menuitem.set_image(image)
	menu.append(indent_menuitem)
	menu.reorder_child(indent_menuitem, 5)

	# Unindent Menuitem
	if not editor.readonly:

		cursor_line = get_current_line(editor)
		start_iter = editor.text_buffer.get_iter_at_line(cursor_line)

		# Get selection bound if any.
		bounds = editor.text_buffer.get_selection_bounds()

		if not bounds:

			# If the first character in a line is a space or tab character,
			# indentation is possible. If not, indentation is impossible. If
			# indentation is possible, make the menuitem sensitive, otherwise,
			# make it insensitive.
			if start_iter.get_char() == " " or start_iter.get_char() == "\t":

				indent = True

			else:

				indent = False

		else:

			# Get the first and last line of a selection bound.
			start_line = bounds[0].get_line()
			end_line = bounds[1].get_line()

			# If the first character in a line is a space or tab character,
			# indentation is possible. If not, indentation is impossible. If
			# indentation is possible, make the menuitem sensitive, otherwise,
			# make it insensitive.
			if start_line == end_line:

				if start_iter.get_char() == " " or \
					start_iter.get_char() == "\t":

					indent = True

				else:

					indent = False

			else:

				# A count to keep track of lines that can be indented.
				count = 0

				# Iterate through the selection of lines and count lines that
				# can if indented.
				for lines in range(start_line, end_line+1):

					start_iter = editor.text_buffer.get_iter_at_line(lines)

					if start_iter.get_char() == " " or \
						start_iter.get_char() == "\t":

						count += 1

				# If at least one line can be indented, set make the menuitem
				# sensitive otherwise, make the it insensitive.
				if count:

					indent = True

				else:

					indent = False

	else:

		# In readonly only mode, make the indent menuitem insensitive at all
		# times
		indent = False

	unindent_menuitem = ImageMenuItem(msg0087)
	unindent_menuitem.connect(u"activate", unindent_menuitem_cb, editor)
	unindent_menuitem.set_sensitive(indent)
	image = Image()
	image.set_from_stock(STOCK_UNINDENT, ICON_SIZE_MENU)
	unindent_menuitem.set_image(image)
	menu.append(unindent_menuitem)
	menu.reorder_child(unindent_menuitem, 6)

	# Matching Bracket Menuitem
	cursor_iter = get_cursor_iter(editor)

	if match_bracket(cursor_iter):

		bracket = True

	else:

		bracket = False

	label = msg0088
	bracket_menuitem = MenuItem(label, use_underline=True)
	bracket_menuitem.connect(u"activate", bracket_menuitem_cb, editor)
	bracket_menuitem.set_sensitive(bracket)
	menu.append(bracket_menuitem)
	menu.reorder_child(bracket_menuitem, 7)
	menu.show_all()

	return True


def copy_cb(textview, editor):
	"""
	Handles callback when text is copied into the editor's buffer.

	The "copy-clipboard" signal is emitted when a selection is copied to the
	clipboard from textview.

	@param textview: Reference to the editor's SourceView object
	@type textview: A gtksourceview.SourceView object

	@param editor: Reference to an editor instance
	@type editor: An editor object

	"""

	# Get selection bounds
	selection = editor.text_buffer.get_selection_bounds()
	editor.mode_edit_flag = True

	# Depending on whether or not text is selected and copied, send appropriate
	# feedback back to the user via the statusbar.
	if selection:

		update_status_info(editor, msg0089, "copy")

	else:

		update_status_info(editor, msg0090, "no")

	return True


def cut_cb(textview, editor):
	"""
	Handles callback when text in cut from the editor's buffer.

	The "cut-clipboard" signal is emitted when a selection is cut from textview
	to the clipboard.

	@param textview: Reference to the editor's SourceView object
	@type textview: A gtksourceview.SourceView object

	@param editor: Reference to an editor instance
	@type editor: An editor object

	"""

	if editor.readonly:

		# Feedback to the status bar indicating the save operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")

		return

	# Get selection bounds
	selection = editor.text_buffer.get_selection_bounds()
	editor.mode_edit_flag = True

	# Depending on whether or not text is selected and cut, send appropriate
	# feedback back to the user via the statusbar.
	if selection:

		update_status_info(editor, msg0091, "cut")

	else:

		update_status_info(editor, msg0092, "no")

	return True


def paste_cb(textview, editor):
	"""
	Handles callback when text in pasted in the editor's buffer.

	The "paste-clipboard" signal is emitted when the contents of the clipboard
	are pasted into textview.

	@param textview: Reference to the editor's SourceView object
	@type textview: A gtksourceview.SourceView object

	@param editor: Reference to an editor instance
	@type editor: An editor object

	"""

	block_word_completion(editor)

	if editor.readonly:

		# Feedback to the status bar indicating the save operation cannot be
		# performed.
		update_status_info(editor, msg0145, "no")
		unblock_word_completion(editor)

		return

	# Check to see if there is text in the clipboard manager.
	clipboard = clipboard_get()
	selection = clipboard.wait_for_text()
	editor.mode_edit_flag = True

	# Depending on whether or not there is text in the clipboard manager, send
	# appropriate feedback back to the user via the statusbar.
	if selection:

		update_status_info(editor, msg0093, "paste")

	else:

		update_status_info(editor, msg0094, "no")

	unblock_word_completion(editor)

	return True


def motion_notify_event_cb(widget, event, editor):
	"""
	Handles callback when the mouse pointer is in motion.

	The "motion-notify-event" signal is emitted when the mouse pointer moves
	while over widget.

	@param widget: Reference to textview
	@type widget: A gtksourceview.SourceView object

	@event event: An event
	@type event: A gtk.gdk.Event object

	@param editor: Reference to an editor instance
	@type editor: An editor object

	@return: Returns false to prevent the event from been propagated to parent
			widgets.
	@rtype: A boolean object
	"""

	# Show the cursor anytime it is move within the textview.
	editor.window.get_pointer()
	textview_win = editor.text_view.get_window(TEXT_WINDOW_TEXT)
	textview_win.set_cursor(Cursor(XTERM))

	try:

		# Set the hidden_cursor flag to false when the cursor is shown.
		editor.hidden_cursor = False

	except:

		pass

	# If the editor is in fullscreen mode and the mouse pointer moves, show the
	# toolbar. Otherwise, hide the toolbar after 5 seconds of mouse pointer
	# inactivity.
	if editor.fullscreen_flag:

		source_remove(editor.ft_id)
		editor.toolbar_area.show_all()
		editor.toolbar_flag = True
		editor.ft_id = timeout_add(5000, fullscreen_timeout, editor)

	return False


def key_press_event_cb(entry, key_press, editor):
	"""
	Callback to the editor's window "key-press-event"

	The "key-press-event" signal is emitted when the user presses a key on
	the	keyboard.

	@param key_press: A key press event.
	@type key_press: GTK Signal

	@param editor: editor
	@type editor: editor object

	"""

	return filter_key_events(editor, key_press.keyval)


def focus_out_event_cb(view, event, editor):
	"""
	Handles callback when the text editor's textview looses focus.

	@param view: The text editor's textview widget.
	@type view: A gtk.SourceView object.

	@param event: An event that occurs when the text editor's texview looses
				focus.
	@type event: A gtk.Event object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	hide_completion_window(editor)

	return False


def undo_cb(view, editor):
	"""
	Handles callback when an undo operation occurs.

	@param view: The text editor's textview widget.
	@type view: A gtk.SourceView object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	block_word_completion(editor)
	hide_completion_window(editor)
	unblock_word_completion(editor)

	return True


def redo_cb(view, editor):
	"""
	Handles callback when a redo operation occurs.

	@param view: The text editor's textview widget.
	@type view: A gtk.SourceView object.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	block_word_completion(editor)
	hide_completion_window(editor)
	unblock_word_completion(editor)

	return True


def insert_cb(textview, string):

	print string

	return False


################################################################################
#
#							Text View
#
################################################################################


def create_text_view(editor):
	"""
	Create a SourceView object for the editor

	@param editor: Reference to an editor instance
	@type editor: An editor object

	@return: A SourceView object
	@rtype: A gtksourceview.SourceView object

	"""

	# Create a textview object
	text_view = SourceView(editor.text_buffer)
	editor.text_view = text_view
	text_view.connect("button-press-event", button_press_event_cb, editor)
	editor.txtkeypress = text_view.connect("key-press-event",
											key_press_event_cb, editor)
	text_view.connect("focus-out-event", focus_out_event_cb, editor)
	text_view.connect("scroll-event", scroll_event_cb, editor)
	editor.ovrid = text_view.connect("toggle-overwrite", toggle_overwrite_cb,
									editor)
	editor.text_view.connect("drag_motion", motion_cb, editor)
	editor.text_view.connect("drag_drop", drop_cb, editor)
	editor.txtid = editor.text_view.connect("drag_data_received", received_cb,
											editor)
	editor.text_view.connect("populate-popup", populate_popup_cb, editor)
	editor.markid = editor.text_buffer.connect("mark-set", mark_set_cb, editor)
	editor.marksetid = editor.text_buffer.connect("mark-deleted", mark_delete_cb,
												editor)
	editor.text_view.add_events(POINTER_MOTION_MASK)
	editor.text_view.add_events(SCROLL_MASK)
	editor.text_view.connect("copy-clipboard", copy_cb, editor)
	editor.text_view.connect("cut-clipboard", cut_cb, editor)
	editor.text_view.connect("paste-clipboard", paste_cb, editor)
	editor.text_view.connect("motion-notify-event", motion_notify_event_cb, editor)
	editor.text_view.connect("scroll-event", motion_notify_event_cb, editor)
	editor.text_view.connect("redo", redo_cb, editor)
	editor.text_view.connect("undo", undo_cb, editor)
	editor.text_view.connect("move-cursor", move_cursor_cb, editor)
	editor.text_view.connect("insert-at-cursor", insert_cb)

	# Drag and drop properties
	target_type_text = 80
	targets = [("text/uri-list", 0, 80),
			("GTK_TEXT_BUFFER_CONTENTS", 0, 82), ("UTF8_STRING", 0, 83),
			("COMPOUND_TEXT", 0, 84), ("text/plain", 0, 81),]
	text_view.drag_dest_set(DEST_DEFAULT_ALL, targets, ACTION_COPY)

	# Set Widget Properties
	text_view.set_resize_mode(RESIZE_PARENT)
	text_view.queue_resize()
	text_view.resize_children()
	text_view.grab_focus()

	# Set Properties
	if editor.wrap_mode_bool:

		text_view.set_wrap_mode(WRAP_WORD)

	else:

		text_view.set_wrap_mode(WRAP_NONE)

	text_view.set_show_line_numbers(True)
	text_view.set_auto_indent(True)
	text_view.set_margin(editor.margin_position)
	text_view.set_show_margin(editor.show_margin)
	text_view.set_tabs_width(editor.tab_width)
	text_view.set_insert_spaces_instead_of_tabs(False)
	text_view.set_highlight_current_line(True)
	text_view.modify_font(editor.font)

	if not editor.use_theme_colors:

		# Use foreground and background colors specified by the user stored in
		# GConf, the GNOME configuration database.
		foreground_color = process_color(editor.foreground_color)
		background_color = process_color(editor.background_color)
		text_view.modify_base(STATE_NORMAL, background_color)
		text_view.modify_text(STATE_NORMAL, foreground_color)

	return text_view

