# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
This module contains timeout functions. They are callback functions called after
a certain time has elasped.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gc import collect

from pygtk import require
require("2.0")
from gtk import TEXT_WINDOW_TEXT
from gtk.gdk import Cursor, Pixmap, Color

from internationalization import msg0025, msg0044, msg0045
from modes import add_mode, reset_mode


###############################################################################
#
#						Timeout Functions
#
################################################################################


def load_string_timeout(editor):
	"""
	Show the URI of the open text file in the editor's status bar.

	@param editor: 	editor
	@type editor: 	an editor object

	@return : 		False
	@rtype: 		boolean
	"""

	# If name is "Unsaved Document" don't show any message in the status bar.
	if editor.filename == msg0025:

		editor.status_one.pop(editor.status_one_id)
		editor.status_one_id = editor.status_one.get_context_id("")
		editor.status_one.push(editor.status_one_id, "")

		return False

	# If the file is in readonly mode, show the readonly icon in the statusbar
	# and display a message saying the file in open in readonly mode. Otherwise
	# show the absolute path to the file and show the correct icon mode.
	if editor.readonly:

		add_mode(editor, "readonly")
		string = msg0044 % editor.prettyname

	else:

		reset_mode(editor)
		string = msg0045 % editor.prettyname

	if editor.message_id:

		editor.status_one.pop(editor.status_one_id)

	editor.status_one_id = editor.status_one.get_context_id(string)
	editor.status_one.push(editor.status_one_id, string)

	return False


def save_file_timeout(editor):
	"""
	Save a modified document after 21 secs of modification.

	@param editor: 	editor
	@type editor: 	an editor object

	@return : 		False
	@rtype: 		boolean
	"""

	# Save the file if it has not already been saved.
	if not editor.save_flag:

		editor.actiongroup.get_action(u"save_file").activate()

	return False


def syntax_highlight_timeout(editor):
	"""
	Highlight buffer after a text file has been saved.

	@param editor: 	editor
	@type editor: 	an editor object

	@return : 		False
	@rtype: 		boolean
	"""
	pass
#	syntax_highlight(editor, editor.file_uri)

	return False


def unblock_insert_signal_timeout(editor):
	"""
	Handles a timeout callback to unblock an "insert_text" signal after an
	undo, redo or paste operation.

	The signal is unblocked to enable automatic word completion.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	"""

	editor.text_buffer.handler_unblock(editor.tb_ins_id)
	editor.blocked_word_completion = False

	return False


def user_action_timeout(editor):
	"""
	Handles a timeout callback to reset the statusbar's icon mode.

	The icon mode can either be reset to "new", "edit" or "readonly."

	@param editor:	Reference to the editor instance
	@type editor:		An editor object

	"""

	reset_mode(editor)

	try:

		# The flag indicates that status area is currently busy and should not
		# be updated until this operation is over
		editor.status_busy_flag = False

	except AttributeError:

		pass

	return False


def activate_progressbar_timeout(editor):
	"""
	Show the progressbar widget in the statusbar area after 1 sec when a file
	is being loaded.

	@param editor:	Reference to the editor instance
	@type editor:		An editor object

	"""

	if editor.loading_flag:

		add_mode(editor, "", progress=True)

	return False


def fullscreen_timeout(editor):
	"""
	Handles callbact to hide the toolbar after a few seconds of pointer
	inactivity during fullscreen mode.

	@param editor:		Reference to the editor instance
	@type editor:			An editor object

	"""

	if not editor.fullscreen_flag:

		return False

	else:

		if editor.toolbar_flag:

			editor.toolbar_area.hide_all()
			editor.toolbar_flag = False

		return False


def quit_mainloop(editor):
	"""
	Quit the application

	@return : False
	@rtype: boolean
	"""

	editor.actiongroup.get_action(u"close_file").activate()

	return False


def garbage(editor):
	"""
	Handles a timeout callback to call the garbage collector every 30 minutes

	@return: Returns True so the timeout is called repeatedly
	@type: boolean

	"""

	# Run the garbage collector to free unused objects in memory. Do not run
	# the collector if the file is saving. Bad things happen.
	if not editor.saving_flag:

		collect()

	return True


def hide_cursor_timeout(editor):
	"""
	Handles a timeout callback to hide the cursor after 5 seconds of cursor
	inactivity.

	@return: Returns True so the timeout is called repeatedly
	@type: boolean

	"""

	try:

		# Check to see if the cursor is busy. If it is don't hide the cursor.
		if editor.busy_cursor:

			return True

	except AttributeError:

		pass

	try:

		# Check to see if the editor.hidden_cursor has already been initialized.
		# If it hasn't, an exception is thrown and it is initialized to false.
		if editor.hidden_cursor:

			pass

	except AttributeError:

		editor.hidden_cursor = False

	try:

		# If the cursor is not hidden, hide it.
		if not editor.hidden_cursor:

			textview_win = editor.text_view.get_window(TEXT_WINDOW_TEXT)
			pixmap = Pixmap(None, 1, 1, 1)
			color = Color()
			textview_win.set_cursor(Cursor(pixmap, pixmap, color, color, 0, 0))
			editor.hidden_cursor = True

	except AttributeError:

		pass

	return True

