# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


"""
Common miscellaneous functions used by the editor

This module helper functions that perform activities such as finding Scribes
data and module files, pushing feedback to the editor's status bar and creating
customized buttons.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""


from sys import argv
from os import path
from locale import getpreferredencoding


from pygtk import require
require("2.0")
from gtk import HBox, Image, ICON_SIZE_BUTTON, Label, events_pending
from gtk import main_iteration
from gtk.gdk import Color

from internationalization import msg0131


def find_app_data_dir():
	"""
	Search for Scribes' data folder

	@return: path Scribes' data folder
	@rtype: string
	"""

	head, tail = path.split(path.split(path.abspath(argv[0]))[0])

	if tail ==  "bin":

		data_dir = path.join(head, "share")

		if path.exists(data_dir):

			return data_dir

	raise "FileNotFoundError", msg0131


def find_toolbar_folder():
	"""
	Search for Scribes' toolbar folder

	@return: path to the toolbar folder
	@rtype: string
	"""

	head, tail = path.split(path.split(path.abspath(argv[0]))[0])

	if tail == 'bin':

		filepath = path.join(head, 'share', 'scribes')

		if path.isdir(filepath):

			return filepath

	raise "FileNotFoundError", msg0131


def create_button(stock_id, string):
	"""
	Create a button with a stock icon and a custom label


	@param stock_id: a GTK stock icon identifier
	@type stock_id: string
	@param string: text to be displayed on the button
	@type string: GTK Label

	@return: a stock icon and label
	@rtype: HBox object
	"""

	encoding = getpreferredencoding(True)
	hbox = HBox(False, 6)
	image = Image()
	image.set_from_stock(stock_id, ICON_SIZE_BUTTON)
	label = Label(string)
	label.set_property("use-underline", True)
	hbox.pack_start(image, False, False, 0)
	hbox.pack_start(label, False, False, 0)

	return hbox


def response():
	"""
	Allow scribes GUI to be responsive when performing tasks that can
	potentially	lock the GUI.

	@warning: 	Use this function sparringly and preferably within loops in
				callbacks. Never use this function in a timeout callback.
	@note:		There is a tradeoff between responsiveness and performance, so
				test to see that when using this function, you have not
				introduced performance hits in other areas of the editor.
	"""
	while events_pending():
		main_iteration(False)
	return


def process_color(color):

	red = int(color[0])
	green = int(color[1])
	blue = int(color[2])
	pixel = long(color[3])
	color = Color(red, green, blue, pixel)

	return color


def split_color(color):

	red = str(color.red)
	green = str(color.green)
	blue = str(color.blue)
	pixel = str(color.pixel)

	return red, green, blue, pixel


def split_string(pattern, string_length, text):

	result = None
	response()
	# Scan the buffer for words eligible for word completion. If any eligible
	# words are found, place the words in a list. See the function's
	# documentation for the criteria for determining eligible words.
	from re import split
	text_list = split(pattern, text.decode("utf-8"))
	word_list = [items for items in text_list if len(items) > string_length]

	if word_list:

		dictionary = {}

		# Index strings based on their occurence in the editor's buffer and
		# place them in a dictionary.
		for string in word_list:

			# Ensure the text editor's graphic user interface does not lock
			# during indexing operations, especially, when indexing very large
			# documents.
			response()

			# Ignore strings that are likely to be ASCII embellishments.
			if string.startswith("---") or string.startswith("___"):

				continue

			if string in dictionary.keys():

				dictionary[string] += 1

			else:

				dictionary[string] = 1

		# Update the editor's word completion dictionary.
		result = dictionary

	return result


def get_text_at_line(textbuffer, line):
	"""
	Get all strings on a specific line in the text editor's buffer.

	@param textbuffer: Any textbuffer.
	@type textbuffer: A gtk.TextBuffer object.

	@param line: An number representing a line in a text buffer.
	@type line: An Integer object.

	@return: All the strings on a particular line in a text buffer.
	@rtype: A String object.

	"""

	start_iter = textbuffer.get_iter_at_line(line)
	end_iter = start_iter.copy()
	end_iter.forward_to_line_end()
	text = textbuffer.get_text(start_iter, end_iter)

	return text

