# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


"""
This module exposes a class responsible for loading text files into the text
editor's buffer.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""


class FileLoader(object):
	u"""
	This class defines attributes and methods needed to load text files into the
	text editor's buffer. Files are represented by Uniform Resource Identifiers,
	URI for short. The objective of the class is to load any valid URI, both
	local or remote, into the text editor's buffer, effectively making the text
	editor network transparent and URI aware. This class exploits the GNOME-VFS
	libraries to achieve its objective.

	"""


	def __init__(self, editor, uri, readonly=False):
		u"""
		Initialize the FileLoader object and prepare the editor for loading.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		@param editor: Reference to the text editor instance.
		@type editor: An Editor object.

		@param uri: A Uniform Resource Identifiers representing an existing file.
		@type uri: A String object.

		"""

		result = self._needs_new_window(editor)

		if result:

			from gobject import timeout_add
			timeout_add(2, self._load_file_in_new_window, uri, readonly)

		else:

			editor.dirty = True
			editor.readonly = readonly
			self._init_attributes(editor, uri)
			self._process_uri()


	def __index_word_completion_string(self, editor, text):

		from re import split
		match_list = split(editor.pattern, text.decode("utf-8"))
		match_list = [items for items in match_list if len(items) > 3]

		for string in match_list:

			if string in editor.word_completion_dictionary.keys():

				editor.word_completion_dictionary[string] = \
					editor.word_completion_dictionary[string] + 1

			else:

				editor.word_completion_dictionary[string] = 1

		return


	def _init_attributes(self, editor, uri):
		u"""
		Initialize attributes that will be used repeatedly throught the
		FileLoader class.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		@param editor: Reference to the text editor instance.
		@type editor: An Editor object.

		"""

		from gnomevfs import URI

		# Instance of the text editor.
		self.editor = editor

		self.editor.loading_flag = True

		# True if user has permission to read from or write to a URI on a
		# remote location
		self.access_flag = False

		# Whitelist of MIME Types that the text editor can accept.
		self.mime_list = [u"image/svg+xml", u"application/x-shellscript",
						u"application/rtf", u"application/x-glade",
						u"application/x-php", u"application/octet-stream",
						u"application/x-desktop", u"application/x-trash",
						u"application/x-csh", u"audio/x-mpegurl",
						u"application/x-perl", u"application/x-ruby",
						u"application/x-javascript", u"application/x-theme",
						u"application/xml", u"application/x-m4",
						"application/docbook+xml"]

		# Encoding set by user if any. The default value is None, if no encoding
		# is set by the user.
		self.encoding = self.editor.filechooser_encoding

		# Start Iterator. FIXME
		self.iter = None

		# True indicates certain buffer signals have been blocked. Used to check
		# if certain signals should be unblocked.
		self.blocked_signals = False

		# URI of the file to open.
		self.fileuri = self.editor.fileuri = uri

		# An object instantiated from a URI string. Note, a URI object differs
		# from a URI string. See the GNOME-VFS manual for more details.
		self.uri_object = self.editor.uri_object = URI(self.fileuri)

		# Initialize filename attributes for the text editor.
		self._redefine_filename()

		return


	def _process_uri(self):
		u"""
		Retrieve necessary information about a URI and load the file pointed to
		by the URI.

		An exception is thrown when the get_file_info GNOME-VFS function does
		not have access rights to a URI, usually a remote one. When that happens
		the exception is handled by showing an authentication dialog. If the
		users choose to authenticate themselves this function is called again
		with the password to be used to access the URI. Otherwise, loading the
		URI is terminated.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		"""

		from gnomevfs import URI, get_file_info, FILE_INFO_DEFAULT
		from gnomevfs import FILE_INFO_GET_MIME_TYPE, AccessDeniedError
		from gnomevfs import FILE_INFO_FORCE_SLOW_MIME_TYPE
		from gnomevfs import FILE_INFO_FOLLOW_LINKS, NotFoundError
		from gnomevfs import GenericError
		from os import getuid, getgid
		from cursor import show_cursor
		show_cursor(self.editor, "busy")
		value = False

		try:

			FILE_INFO_GET_ACCESS_RIGHTS = 1 << 4
			self.fileinfo = get_file_info(self.uri_object,
												FILE_INFO_DEFAULT |
												FILE_INFO_GET_MIME_TYPE |
												FILE_INFO_FORCE_SLOW_MIME_TYPE |
												FILE_INFO_FOLLOW_LINKS |
												FILE_INFO_GET_ACCESS_RIGHTS)

			if not self.fileinfo.name:

				raise AccessDeniedError

			value = True

		except AccessDeniedError:

			print "yer"
			value = False
			from gobject import timeout_add
			timeout_add(10, self._show_authentication_dialog)
			value = False

		except NotFoundError:

			print "and yers"
			value = False
			from internationalization import msg0037

			# Show the error dialog if the file does not exist.
			self._show_error_dialog(msg0037)

		except GenericError:

			print "A Generic Error occured."
			value = False

		if value:

			self._load_uri()

		else:

			show_cursor(self.editor)

		return


	def _show_authentication_dialog(self):
		u"""
		Show authentication dialog to allow users access files at remote
		locations.

		@param self: Reference to the PasswordDialog instance.
		@type self: A PasswordDialog object.

		"""

		from passworddialog import PasswordDialog
		PasswordDialog(self)

		return False


	def _load_uri(self):
		u"""
		Analyze the URI for loading.

		This function validates the URI to ensure it is a text file. It also
		checks the permission needed to read from or write to the file, provides
		feedback to users via the editor's statusbar, cursor and titlebar and
		finally loads the file into the editor's buffer.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		"""

		# Optimal size of file to read per unit.
		self.editor.read_size = self._get_optimal_read_size()

		result = self._check_file_type()

		if result:

			result = self.__check_uri_permission()

		if result:

			# Asynchronously open.
			from gnomevfs.async import open as vfsopen
			vfsopen(self.uri_object, self.open_cb)

			# Activate syntax highlight.
			from syntax import syntax_highlight
			syntax_highlight(self.editor, self.fileuri)

			# Show the progress bar for files that take more than a second to
			# load.
			from gobject import timeout_add
			self.editor.gtimer = timeout_add(1000,
									self.activate_progressbar_timeout)

			# Recall the window position, buffer state and encoding of the file
			# if it has an entry in Scribes' database.
			from position import get_metadata_info
			get_metadata_info(self.editor)
			self.editor.show_all()

			# Block certain buffer signals to prevent flag corruption.
			self._block_buffer_signals()

			# Hide the cursor.
			self._set_cursor_visibility(False)

			# Make toolbar buttons insensitive while loading the file.
			self._set_toolbar_buttons_sensitivity(False)

			# Feedback to statusbar area indicating that a file is being loaded.
			from feedback import update_status_info
			from internationalization import msg0032
			update_status_info(self.editor, msg0032, "run", False)

			# Set the titlebar to show the file is currently being loaded.
			from internationalization import msg0335
			title = self.uri_object.short_name
			self.editor.set_title(msg0335 % title)

		return


	def open_cb(self, handle, result):

		if not result:

			# Clear the text buffer and begin reading the file to be loaded.
			self.editor.text_buffer.set_text("")
			self.iter = self.editor.text_buffer.get_start_iter()
			self._set_buffer_editability(False)
			handle.read(self.editor.read_size, self.read_cb)

		else:

			from cursor import show_cursor
			show_cursor(self.editor)
			handle.close(self.close_cb)

		return True


	def read_cb(self, handle, textbuffer, result, bytes):

		if not result:

			# Progress bar information
			if self.editor.progress_flag:

				self.editor.opt_count = self.editor.opt_count + \
										self.editor.read_size

			unicode_txt = self._decode_text_to_unicode(textbuffer)
			output = self._encode_text_to_utf8(unicode_txt)
			self._insert_text_to_buffer(output, handle)

		else:

			self.editor.progress_flag = False
			from feedback import update_status_info
			from cursor import show_cursor
			from internationalization import msg0030
			message = msg0030 % self.editor.prettyname
			update_status_info(self.editor, message, "open")
			self._set_toolbar_buttons_sensitivity(True)
			self._determine_cursor_placement()
			self._determine_bookmark_placement()
			self._set_buffer_editability(True)
			self.editor.text_buffer.set_modified(False)
			self._set_cursor_visibility(True)
			self._unblock_buffer_signals()
			self._set_finalization_flags()
			#self.editor.encoding = self.encoding
			self.editor.fileuri = self.fileuri
			show_cursor(self.editor)

			if not self.editor.readonly:

				title = self.uri_object.short_name
				self.editor.set_title(title)

			else:

				from readonly import activate_readonly_mode
				activate_readonly_mode(self.editor)

			handle.close(self.close_cb)

		return True


	def close_cb(self, *args):

		self._clean_up()

		return True


	def _get_optimal_read_size(self):
		u"""
		Calculate the size of the file to read at a time.

		The value of this differs accross filesystems. The GNOME-VFS library is
		used determine the optimal size, which is usually 4096bytes. When
		GNOME-VFS is unable to determine the optimal size, usually on remote
		filesystems, it is set to 4096bytes.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		"""

		from gnomevfs import FILE_INFO_FIELDS_IO_BLOCK_SIZE

		if self.fileinfo.valid_fields & FILE_INFO_FIELDS_IO_BLOCK_SIZE:

			value = self.fileinfo.io_block_size

		else:

			value = 4096

		return value


	def _recheck_file_encoding(self):
		u"""
		Check the current encoding of a file.

		The encoding of a file is determined by three factors, namely: the user;
		the editor and; the editor's database. The user has control over which
		encoding to use to open a file. In such instances, the user's encoding
		of choice overrides that of editor's or that entered in the editor's
		database.

		When a file is saved in a particular encoding, the encoding is
		associated with the file and stored in the editor's database. So that
		when next the file is loaded, the encoding stored in the database is
		used. If the user chooses no encoding, or no encoding is stored in the
		editor's database, then the editor's default encoding, UTF-8 is used
		when loading the file.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		"""

		if self.encoding:

			self.editor.encoding = self.encoding

		else:

			self.encoding = self.editor.encoding

		return self.encoding


	def _decode_text_to_unicode(self, text):
		u"""
		Decode text in files to Unicode.

		Unicode is the universal encoding understood by Python and GNOME.
		Strings must first be decoded to Unicode before they can be encoded to
		any other character set.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		@param text: Text retrieved from a file.
		@type text: An encoded text.

		@return: A unicode string.
		@rtype: A unicode string object.

		"""

		try:

			self.encoding = self._recheck_file_encoding()
			txt = text.decode(self.encoding, "replace")

		except UnicodeDecodeError:

			self.editor.progress_flag = False
			from internationalization import msg0031
			self._show_error_dialog(msg0031)
			txt = None

		return txt


	def _encode_text_to_utf8(self, text):
		u"""
		Encode unicode text to UTF-8.

		gtk.TextBuffer only accepts UTF-8 encoded characters, thank goodness.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		@param text: Unicode text.
		@type text: A unicode string.

		@return: An encoded string.
		@rtype: A encoded string object.

		"""

		try:

			txt = text.encode("utf-8")

		except UnicodeEncodeError:

			self.editor.progress_flag = False
			from internationalization import msg0153
			self._show_error_dialog(msg0153)
			txt = None

		except AttributeError:

			txt = None

		return txt


	def _insert_text_to_buffer(self, text, handle):
		u"""
		Inserted UTF-8 encoded character into the text editor's buffer.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		@param text: UTF-8 encoded text.
		@type text: A UTF-8 encoded string object.

		@param handle: A reference to the file object to load into the buffer.
		@type handle: A gnomevfs handle.

		"""

		try:

			self.editor.text_buffer.insert(self.iter, text)
			self.iter = self.editor.text_buffer.get_end_iter()
			handle.read(self.editor.read_size, self.read_cb)
			from gobject import timeout_add
			timeout_add(3, self.__index_word_completion_string, self.editor,
						text)

		except TypeError:

			pass

		return


	def _check_file_type(self):
		u"""
		Check the MIME Type of the file.

		This function performs a validation check of the file represented by
		the URI. The text editor should only load text files. Files that are not
		in the MIME Type whitelist or are not text files are rejected.

		The GNOME-VFS MIME Type checking function on a few occassions can turn
		up false positives. That is text files may be identified as other types.
		For such occassions, a whitelist is created to ensure such files are
		loaded.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		@return: Returns True if the file is a valid text file and false
			otherwise
		@rtype: A Boolean object.

		"""

		from gnomevfs import FILE_INFO_FIELDS_MIME_TYPE
		value = True

		if self.fileinfo.valid_fields & FILE_INFO_FIELDS_MIME_TYPE:

			self.editor.mimetype = mimetype = self.fileinfo.mime_type

			if not mimetype.startswith("text/") \
				and mimetype not in self.mime_list:

				# Show an error dialog if the MIME type of the file is not a
				# text, SVG, shellscript or rtf one.
				from internationalization import msg0192
				message = msg0192 % mimetype
				self._show_error_dialog(message)
				value = False
				self._clean_up()

		else:

			# Show an error dialog if the MIME type of the file cannot be
			# determined.
			from internationalization import msg0191
			message = msg0191 % self.editor.mimetype
			self._show_error_dialog(message)
			value = False
			self._clean_up()

		return value


	def _check_file_permissions(self):
		u"""
		Check if the user has permission to read from or write to the file.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		@return: Returns True if the user has permission to read the file.
		@rtype: A Boolean object.

		"""

		from gnomevfs import PERM_USER_READ, PERM_USER_WRITE
		from gnomevfs import FILE_INFO_FIELDS_PERMISSIONS
		FILE_INFO_FIELDS_ACCESS = 1 << 14
		PERM_ACCESS_READABLE   = 1 << 16
		PERM_ACCESS_WRITABLE   = 1 << 17
		value = False

		if self.fileinfo.valid_fields & FILE_INFO_FIELDS_PERMISSIONS:

			if self.fileinfo.valid_fields & FILE_INFO_FIELDS_ACCESS:


				if self.fileinfo.permissions & PERM_ACCESS_READABLE :

					value = True

					if self.fileinfo.permissions & PERM_ACCESS_WRITABLE:

						pass

					else:

						self.editor.readonly = True

				else:

					# Show an error dialog if the user does not have permission to
					# view the file.
					from internationalization import msg0035
					self._show_error_dialog(msg0035)
					value = False
					self._clean_up()

			else:

				if not self.uri_object.is_local:

					if self.fileinfo.permissions & PERM_USER_READ:

						value = True

						if self.fileinfo.permissions & PERM_USER_WRITE:

							pass

						else:

							self.editor.readonly = True

					else:

						# Show an error dialog if the user does not have
						# permission to view the file.
						from internationalization import msg0035
						self._show_error_dialog(msg0035)
						value = False
						self._clean_up()

				else:

					value = False
					print "Could not get access rights"

		else:

			if not self.access_flag:

				self.editor.readonly  = True

			value = True

		return value

	def __check_uri_permission(self):
		result = None
		if self.uri_object.is_local or str(self.uri_object).startswith("file:///"):
			from gnomevfs import get_local_path_from_uri
			local_path = get_local_path_from_uri(str(self.uri_object))
			from os import access, W_OK, R_OK
			value = access(local_path, R_OK)
			if value:
				result = True
				value = access(local_path, W_OK)
				if value is False:
					self.editor.readonly = True
			else:
				# Show an error dialog if the user does not have permission
				# to view the file.
				from internationalization import msg0035
				self._show_error_dialog(msg0035)
		else:
			result = True
			# This branch is the most common scenario for files on remote
			# location.
			from gnomevfs import get_uri_scheme
			scheme = get_uri_scheme(str(self.uri_object))
			# Open the file in readonly mode if the URI scheme is not
			# writable.
			writable_vfs_schemes = ["ssh", "sftp", "smb", "dav", "davs", "ftp"]
			if not scheme in writable_vfs_schemes:
				self.editor.readonly = True
		return result

	def _redefine_filename(self):
		u"""
		Define the file's URI, filename and statusbar name

		The file's URI is used by GNOMEVFS to load files. The filename is the
		full path to the file. And the statusbar name is the full path to the
		file as it will appear in the statusbar.

		@param self: Reference to a FileLoader instance.
		@type self: A FileLoader object.

		@param filename: The name of the file to be loaded.
		@type filename: A string.

		"""

		from gnomevfs import unescape_string_for_display
		from os import environ

		self.editor.filename = unescape_string_for_display(self.uri_object.dirname) \
			+ "/" + self.uri_object.short_name

		if self.editor.filename.startswith(environ['HOME']):

			filename = self.editor.filename.replace(environ['HOME'], "~")
			self.editor.prettyname = filename.strip()

		else:

			self.editor.prettyname = self.editor.filename.strip()

		return


	def _needs_new_window(self, editor):
		u"""
		Check if the new file should be loaded in a new editor instance.

		The text editor follows a spatial and document centric paradigm. Thus,
		each file should be loaded in its own editor window. This is known as
		a single document interface. The function checks to see if the editor
		currently holds a document. It returns True if the editor already
		contains a document or is in the process of loading one, and False
		otherwise

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		@param editor: Reference to the text editor instance.
		@type editor: An Editor object.

		@return: True to load file in new window, False to load in current one.
		@rtype: A Boolean object.

		"""

		value = False

		# Return True if the editor already contains a document or is in the
		# process of loading one.

		if editor.dirty:

			value = True

		return value


	def _set_toolbar_buttons_sensitivity(self, value=True):
		u"""
		Disable or enable some toolbar buttons.

		To prevent users from corrupting the buffer when loading a file, some
		toolbar buttons are disabled. The replace toolbar button, in particular,
		is disabled in readonly mode. As users should not be able to perform
		such text operations in readonly mode.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		@param value: True to enable toolbar buttons. False to disable them.
		@type value: A boolean object.

		"""

		self.editor.actiongroup.get_action("saveas_file").set_sensitive(value)
		self.editor.actiongroup.get_action("replace_text").set_sensitive(value)
		self.editor.actiongroup.get_action("print_file").set_sensitive(value)
		self.editor.actiongroup.get_action("goto_line").set_sensitive(value)
		self.editor.actiongroup.get_action("search_text").set_sensitive(value)
		self.editor.actiongroup.get_action("preferences").set_sensitive(value)

		# Always disable the replace toolbar button in readonly mode.
		if self.editor.readonly:

			self.editor.actiongroup.get_action("replace_text").set_sensitive(False)

		return


	def _set_buffer_editability(self, value=True):
		u"""
		Set or Unset whether or not the buffer should be editable.

		The buffer should not be editable in readonly mode.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		@param value: True if the buffer should be editable, False otherwise.
		@type value: A boolean object.

		"""

		if self.editor.readonly:

			value = False

		self.editor.text_view.set_editable(value)

		return


	def _block_buffer_signals(self):
		u"""
		Block key buffer signals while loading a file.

		Block the "changed" and "modified changed" text buffer signals. These
		signals should be blocked to prevent the callbacks associated with
		them from being activated when loading a file.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		"""

		self.editor.text_buffer.handler_block(self.editor.tb_mod_id)
		self.editor.text_buffer.handler_block(self.editor.tb_chg_id)
		self.editor.text_buffer.handler_block(self.editor.tb_ins_id)
		self.editor.text_buffer.begin_not_undoable_action()

		# The flag is true if buffer signals have been blocked.
		self.blocked_signals = True

		return


	def _unblock_buffer_signals(self):
		u"""
		Unblock key buffer signals after a file has been loaded.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		"""

		self.editor.text_buffer.handler_unblock(self.editor.tb_mod_id)
		self.editor.text_buffer.handler_unblock(self.editor.tb_chg_id)
		self.editor.text_buffer.handler_unblock(self.editor.tb_ins_id)
		self.editor.text_buffer.end_not_undoable_action()

		# The flag is true if buffer signals have been blocked.
		self.blocked_signals = False

		return


	def _blocked_buffer_signals(self):
		u"""
		Check if buffer signals are blocked.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		@return: Return True if buffer signals are blocked and false otherwise.
		@rtype: A boolean object.

		"""

		if self.blocked_signals:

			value = True

		else:

			value = False

		return value


	def _set_cursor_visibility(self, value=True):
		u"""
		Show or hide the cursor.

		The cursor is hidden when the file is being loaded and shown when the
		it has finished loading. The cursor is always hidden in readonly mode.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		@param value: True to show the cursor false otherwise.
		@type value: A boolean object.

		"""

		if self.editor.readonly:

			value = False

		self.editor.text_view.set_cursor_visible(value)
		self.editor.text_view.set_highlight_current_line(value)

		return


	def _load_file_in_new_window(self, uri, readonly):
		u"""
		Create a new instance of the text editor and load the file passed as a
		parameter into the new editor instance.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		@param uri: A Uniform Resource Identifier representing an existing file.
		@type uri: A string object.

		"""

		from gtk import quit_add, main as gmain
		quit_add(0, gmain)
		from main import launch_new_editor
		editor = launch_new_editor([uri], readonly)
		self._clean_up()

		return


	def _set_finalization_flags(self):
		u"""
		Set flags at the end of loading a file.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		@note: See editor.py for comments on the flags.

		"""

		FileLoader.__can_load = True
		self.editor.loading_flag = False
		self.editor.dirty = True
		self.editor.isNewFile = False

		return


	def _determine_cursor_placement(self):
		u"""
		Determine where to place the cursor when a file is loaded.

		The editor stores the last cursor position in a database so that
		when a file is loaded the cursor is placed at the last position in the
		file.

		However, it is possible that another software program alters the state
		of the file. Thus making the information in the editor's database
		obsolete. In such a case, the editor tries to determine where to place
		the cursor.

		If the changes are too drastic, the editor places the cursor at the
		begining of the file. If the changes are minor, the editor places the
		cursor at an approximate position close the the last position the cursor
		was last seen.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		"""

		start_iter = self.editor.text_buffer.get_start_iter()

		# Place the cursor at the begining of the file if the file has no entry
		# in the database.
		if self.editor.position_flag:

			# If the number of lines in the text editor's buffer is less than
			# that stored in the database, then place the cursor at the begining
			# of the file.
			number_of_lines = self.editor.text_buffer.get_line_count()
			last_line = self.editor.pos[0][0]

			if last_line > number_of_lines:

				self.editor.text_buffer.place_cursor(start_iter)
				self.editor.text_view.queue_draw()

				return

			# If the byte index on the line is less than that stored in the
			# database, then place the cursor at the position of smaller byte
			# index.
			line_iter = \
			self.editor.text_buffer.get_iter_at_line(last_line-1)
			number_of_bytes_in_line = line_iter.get_bytes_in_line()
			last_line_index = self.editor.pos[0][1]

			if last_line_index > number_of_bytes_in_line:

				line_iter.set_line_index(number_of_bytes_in_line)

			else:

				line_iter.set_line_index(last_line_index)

			self.editor.text_buffer.place_cursor(line_iter)
			cursor_mark = self.editor.text_buffer.get_insert()
			self.editor.text_view.scroll_to_mark(cursor_mark, 0.3,
								use_align=False, xalign=0.5,
								yalign=0.5)

		else:

			self.editor.text_buffer.place_cursor(start_iter)

		self.editor.text_view.queue_draw()

		return


	def _determine_bookmark_placement(self):
		u"""
		Check whether or not the document should be bookmarked and place
		the bookmark in the right places if it should.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		"""

		if not self.editor.readonly and self.editor.bookmark_flag:

			from bookmark import bookmark_line

			for lines in self.editor.bookmarked_lines:

				bookmark_line(self.editor, lines)

		return


	def _error_recovery(self):
		u"""
		Reset the editor to a neutral state after an error occurs.

		This function is called when a error occurs while a file can not be
		loaded for several reasons. The function returns the editor window to a
		neutral state as if a new window has just been launched.

		@param self:	fileloader object

		"""

		from feedback import update_status_info

		if self._blocked_buffer_signals():

			self._unblock_buffer_signals()

		self.editor.initialize_flags()
		self._set_cursor_visibility(True)
		self._set_buffer_editability(True)
		self.editor.text_buffer.set_text("")
		self._set_toolbar_buttons_sensitivity(True)
		self.editor.text_buffer.set_modified(False)
		from internationalization import msg0025
		self.editor.set_title(msg0025)
		self.editor.filename = msg0025
		update_status_info(self.editor, "", "")

		return


	def _show_error_dialog(self, message):
		u"""
		Show an error dialog when a fatal event occurs.

		@param self: Reference to the FileLoader object.
		@type self: A FileLoader object.

		@param message: Message to display on the dialog window.
		@type message: A string object.

		"""

		from cursor import show_cursor
		from error import error_dialog
		show_cursor(self.editor)
		error_dialog(self.editor, message)
		self._error_recovery()

		return


	def activate_progressbar_timeout(self):
		"""
		Show the progressbar widget in the statusbar area after 1 sec when a
		file is being loaded.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		"""

		if self.editor.loading_flag:

			from modes import add_mode
			self.editor.opt_count = 0.0
			self.editor.file_size = self.fileinfo.size
			add_mode(self.editor, "", progress=True)

		return False


	def _clean_up(self):
		u"""
		Destroy the file loader object and call the garbage collector to free
		memory if necessary.

		@param self: Reference to the FileLoader instance.
		@type self: A FileLoader object.

		"""

		from gc import collect
		del self
		collect()

		return

